package applets;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JColorChooser;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Color;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

//Library imports
import drawing.Stars;
import drawing.StarsParameters;

//static imports
import static drawing.StarsParameters.*;

/**
 *  Applet for showing the Stars image effect.
 *
 **/

public class StarsApplet extends JApplet implements ChangeListener, ActionListener {

  /*** member variables ***/
  JSlider m_pPikeNumSlider, m_pLineNumberSlider;
  JLabel m_pCanvas;
  JButton m_pLineColorButton;
  int m_iLanguageId;
  StarsParameters m_pStarsParameters;

  /*** label strings: id 0 = English, id 1 = German ***/
  String[] m_pPikeNumText = {"Number of pikes:", "Anzahl der Spitzen:"};
  String[] m_pLineNumText = {"Number of lines:", "Anzahl der Linien:"};
  String[] m_pLineColorText = {"Line color:", "Linienfarbe:"};
  String[] m_pLineColorButtonText = {"Change", "Ändern"};

  //Called when this applet is loaded into the browser. Creates the GUI.
  public void init() {
    //Execute a job on the event-dispatching thread; creating this applet's GUI.
    try {

      //1. fetch the language parameter and set the language id      
      m_iLanguageId = 0;
      try {
        String pLanguage = getParameter("language");
        if (pLanguage != null) {
          pLanguage = pLanguage.trim().toLowerCase();
          if (pLanguage.equals("german")) {
            m_iLanguageId = 1;
          }
        }
      }
      catch (Exception pException) {
        //nothing to do, no parameter found
      }

      //2. create the gui
      
      //2.1. create the control panel holding the control elements          
      JPanel pControlPanel = new JPanel();
      pControlPanel.setLayout(new GridLayout(3,2));

      JLabel pPikeNumLabel = new JLabel(m_pPikeNumText[m_iLanguageId]);
      pControlPanel.add(pPikeNumLabel);
      m_pPikeNumSlider = new JSlider(3, 10, 4);
      m_pPikeNumSlider.addChangeListener(this);
      pControlPanel.add(m_pPikeNumSlider);

      JLabel pLineNumLabel = new JLabel(m_pLineNumText[m_iLanguageId]);
      pControlPanel.add(pLineNumLabel);
      m_pLineNumberSlider = new JSlider(5, 20, 10);
      m_pLineNumberSlider.addChangeListener(this);
      pControlPanel.add(m_pLineNumberSlider);
      
      JLabel pLineColorLabel = new JLabel(m_pLineColorText[m_iLanguageId]);
      pControlPanel.add(pLineColorLabel);
      m_pLineColorButton = new JButton(m_pLineColorButtonText[m_iLanguageId]);
      m_pLineColorButton.addActionListener(this);
      pControlPanel.add(m_pLineColorButton);
      
          
      //2.2. create the drawing canvas and fill it with the initial image
      m_pStarsParameters = new StarsParameters();
      Stars pStars = new Stars(m_pStarsParameters);
      m_pCanvas = null;
      try {
        m_pCanvas = new JLabel(new ImageIcon(pStars.process()));
      }
      catch (Exception pException) {
        m_pCanvas = new JLabel("The image could not be created!");
      }
          
      //2.3. set the overall layout and add the components
      setLayout(new BorderLayout());
      add(m_pCanvas, BorderLayout.CENTER);
      add(pControlPanel, BorderLayout.SOUTH);
      
    } catch (Exception e) {
      System.err.println("createGUI didn't complete successfully");
    }
    
    showStatus("started!!! ");
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  public void stateChanged(ChangeEvent e) {

  
    //1. set the new parameters
    int iPikeNum = m_pPikeNumSlider.getValue();
    int iLineNum = m_pLineNumberSlider.getValue();
    m_pStarsParameters.setIntegerParameterValue(PARAM_INT_SPIKE_NUM, iPikeNum);
    m_pStarsParameters.setIntegerParameterValue(PARAM_INT_LINE_NUM, iLineNum);
    
    //2. recreate canvas
    recreateCanvas();

  }
  
  //////////////////////////////////////////
  // ACTION LISTENER - BUTTON EVENTS      //
  //////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {
    
    //1. open a modal color chooser to fetch a new color
    String pTitle = m_pLineColorText[m_iLanguageId];
    Color pStartColor = m_pStarsParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR);
    
    Color pNewColor = JColorChooser.showDialog(this, pTitle, pStartColor);   //is no longer the default way in JFXLibrary.
    m_pStarsParameters.setColorParameterValue(PARAM_COLOR_LINE_COLOR, pNewColor);
  
    //2. recreate canvas
    recreateCanvas();
  }
  
  
  ///////////////////////////////////////////
  // HELPER METHODS                        //
  ///////////////////////////////////////////
  public void recreateCanvas() {
    //1. create and set the image
    Stars pStars = new Stars(m_pStarsParameters);
    try {
      m_pCanvas.setIcon(new ImageIcon(pStars.process()));
    }
    catch (Exception pException) {
      m_pCanvas.setText("The image could not be created!");
    }
    
    //2. repaint the applet
    m_pCanvas.repaint();
    
    showStatus(createStatusMessage());

  }
  
  
  
  public String createStatusMessage() {
    String pStatusMessage = "";
    
    pStatusMessage += m_pPikeNumText[m_iLanguageId] + " ";
    pStatusMessage += m_pPikeNumSlider.getValue();
    
    
    pStatusMessage += " - ";
    pStatusMessage += m_pLineNumText[m_iLanguageId] + " ";
    pStatusMessage += m_pLineNumberSlider.getValue();
    
    Color pColor = m_pStarsParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR);
    pStatusMessage += " - ";
    pStatusMessage += m_pLineColorText[m_iLanguageId] + " ";
    pStatusMessage += pColor.getRed() + ", " + pColor.getGreen() + ", " + pColor.getBlue();
    
    return pStatusMessage;
  }

  
  
 
  
  
  



}
