package color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;

//library imports
import standard.EffectParameters;
import standard.IntegerParameter;
import standard.DoubleParameter;
import standard.BooleanParameter;
import standard.ObjectParameter;
import util.image.ExampleImageGenerator;


/**
 *   Encapsulates the parameters that can be specified for the Noise Generator object. 
 **/
public class NoiseGeneratorParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the boolean parameter: create salt and pepper noise  ***/
  public static final String PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE = "CREATE_SALT_AND_PEPPER_NOISE";
  
  /*** KEY for the boolean parameter: create gaussian noise  ***/
  public static final String PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE = "CREATE_GAUSSIAN_NOISE";
  
  /*** KEY for the boolean parameter: create random color noise  ***/
  public static final String PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE = "CREATE_RANDOM_COLOR_NOISE";
  
  /*** KEY for the double parameter: intensity for salt and pepper  ***/
  public static final String PARAM_DOUBLE_SALT_AND_PEPPER_INTENSITY = "SALT_AND_PEPPER_INTENSITY";
  
  /*** KEY for the double parameter: intensity for gaussian noise  ***/
  public static final String PARAM_DOUBLE_GAUSSIAN_INTENSITY = "GAUSSIAN_INTENSITY";
  
  /*** KEY for the double parameter: standard deviation for gaussian noise  ***/
  public static final String PARAM_DOUBLE_GAUSSIAN_STANDARD_DEVIATION = "GAUSSIAN_STANDARD_DEVIATION";
  
  /*** KEY for the double parameter: intensity for gaussian noise  ***/
  public static final String PARAM_DOUBLE_RANDOM_COLOR_INTENSITY = "RANDOM_COLOR_INTENSITY";
  
  
  
  /*** KEY for the object parameter: image ***/
  public static final String PARAM_OBJECT_IMAGE = "IMAGE";
    
   /***
   *   Creates a default parameter set for the noise generator effect. The following settings are used:
   *
   *   <ul>
   *     <li>An empty image with the dimension 400 x 400 pixels</li>
   *     <li>The salt and pepper noise is activated.</li>
   *     <li>The intensity is set to 50 percent.</li>
   *   </ul>
   *
   **/
  public NoiseGeneratorParameters() {

    //1. create the parameters and set them
    //setIntegerParameters(createIntegerParameters());
    setBooleanParameters(createBooleanParameters());
    setObjectParameters(createObjectParameters());
    setDoubleParameters(createDoubleParameters());
  }
  
  
  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, BooleanParameter> createBooleanParameters() {
    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE, new BooleanParameter(true));
    pParameters.put(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE, new BooleanParameter(false));
    pParameters.put(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE, new BooleanParameter(false));
    
    //3. return the hashtable
    return pParameters;
  }
  
  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_SALT_AND_PEPPER_INTENSITY, new DoubleParameter(0.5, 0, 1));
    pParameters.put(PARAM_DOUBLE_GAUSSIAN_INTENSITY, new DoubleParameter(0.5, 0, 1));
    pParameters.put(PARAM_DOUBLE_GAUSSIAN_STANDARD_DEVIATION, new DoubleParameter(128, 0, 255));
    pParameters.put(PARAM_DOUBLE_RANDOM_COLOR_INTENSITY, new DoubleParameter(0.5, 0, 1));
    
    //3. return the hashtable
    return pParameters;
  
  }
  
  
  
  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    ExampleImageGenerator pGenerator = new ExampleImageGenerator();
    pParameters.put(PARAM_OBJECT_IMAGE, new ObjectParameter(pGenerator.getImage()));
  
    //3. return the hashtable
    return pParameters;
  }
  
}
  
  
  
