package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import java.util.Random;


//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.color.RandomColor;

//static imports
import static drawing.BrickTextureParameters.*;


/**
 *  Implements the brick texture effect.
 **/
public class BrickTexture extends ImageEffect {


  /*** private member variables ***/
  private BrickTextureParameters m_pParameters;

  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code BrickTexture} object with the default parameters.
   *
   **/
  public BrickTexture() {
    this(new BrickTextureParameters());
  }
  
  /**
   *   Creates a {@code BrickTexture} object with the specified parameters.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public BrickTexture(BrickTextureParameters pParameters) {
    m_pParameters = pParameters;
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the brick texture effect. The parameters are only set, 
   *   if the object is of the type {@code BrickTextureParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof BrickTextureParameters) {
      m_pParameters = (BrickTextureParameters)pEffectParameters;
    }
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the parameters that are currently used for the brick texture effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns a brick texture image according to the parameters that were set.
   *
   *  @return A brick texture image as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    //1. make some checks
    if (m_pParameters == null) throw new NullPointerException("No parameters were specified!");
    
   
    //2. fetch the parameters
    //2. 1. do the normal fetch
    int iWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_WIDTH);
    int iHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_HEIGHT);
    int iBrickWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_BRICK_WIDTH);
    int iBrickHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_BRICK_HEIGHT);
    int iBevelMode = m_pParameters.getIntegerParameterValue(PARAM_INT_BEVEL_MODE);
    int iRandomMode = m_pParameters.getIntegerParameterValue(PARAM_INT_RANDOM_MODE);
    
    double dRandomizeMagnitude = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_RANDOMIZE_MAGNITUDE);
    
    Color pBrickColor = m_pParameters.getColorParameterValue(PARAM_COLOR_BRICK);
    Color pBorderColor = m_pParameters.getColorParameterValue(PARAM_COLOR_BORDER);
    
    boolean bDrawBorder = m_pParameters.getBooleanParameterValue(PARAM_BOOLEAN_DRAW_BORDER);
    
    //2.2. calculate helping variables for the random mode
    /*
    0 = RGB, 1 = R, 2 = G, 3 = B, 4 = RG, 5 = RB, 6 = GB, 
    7 = HSB, 8 = H, 9 = S, 10 = B, 11 = HS, 12 = HB, 13 = SB
    */
    boolean bRGB = (iRandomMode <= 6);
    int iRandomModeHelp = iRandomMode;
    if (iRandomMode > 6) iRandomModeHelp -= 7;
    
    boolean[] bRandomFlags = new boolean[] {true, true, true};
    if (iRandomModeHelp == 1) {
      bRandomFlags[1] = false;
      bRandomFlags[2] = false;
    }
    else if (iRandomModeHelp == 2) {
      bRandomFlags[0] = false;
      bRandomFlags[2] = false;
    }
    else if (iRandomModeHelp == 3) {
      bRandomFlags[0] = false;
      bRandomFlags[1] = false;
    }
    else if (iRandomModeHelp == 4) {
      bRandomFlags[2] = false;
    }
    else if (iRandomModeHelp == 5) {
      bRandomFlags[1] = false;
    }
    else if (iRandomModeHelp == 6) {
      bRandomFlags[0] = false;
    }
    
    
    
   
    //3. create an image and configure the graphics context
    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    //pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    
    //4. do the painting
    int iRowCounter = 0;
    RandomColor pRandomColor = new RandomColor(pBrickColor);
    pRandomColor.setRandomMagnitude(dRandomizeMagnitude);
    
    for (int y = 0; y < iHeight; y += iBrickHeight) {

      //4.1. create the offset for each row
      int iXOffset = 0;
      if ((iRowCounter % 2) == 1) {
        iXOffset = -iBrickWidth/2;
      }
      iRowCounter++;

      //4.2. make the settings for the first color
      Color pFirstColor = null;

      //4.3. do the real painting
      for (int x = iXOffset; x < iWidth; x += iBrickWidth) {

        //I. fetch color  and set the first color if it is the last brick in the row
        Color pColor = null;
        if (bRGB) {
          pColor = pRandomColor.getNextColorRGB(bRandomFlags[0], bRandomFlags[1], bRandomFlags[2]);
        }
        else {
          pColor = pRandomColor.getNextColorHSB(bRandomFlags[0], bRandomFlags[1], bRandomFlags[2]);
        }
        if (pFirstColor == null) pFirstColor = pColor;
        if (x + iBrickWidth >= iWidth) pColor = pFirstColor;

        

        //II. draw Rectangle
        pGraphics.setColor(pColor);
        if (iBevelMode == 0) {
          pGraphics.fillRect(x, y, iBrickWidth, iBrickHeight);
        }
        else if (iBevelMode == 1) {
          pGraphics.fill3DRect(x, y, iBrickWidth, iBrickHeight, true);
        }
        else {
          pGraphics.fill3DRect(x, y, iBrickWidth, iBrickHeight, false);
        }

        

        //III. draw Border
        if (bDrawBorder) {
          pGraphics.setColor(pBorderColor);
          pGraphics.drawRect(x, y, iBrickWidth, iBrickHeight);
        }

      }

    }

    //5. return the shitty fuck
    return pImage;
    
  }

}
