package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java imports
import java.util.Hashtable;

import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.BooleanParameter;
import standard.IntegerParameter;
import standard.DoubleParameter;
import standard.ColorParameter;



/**
 *   Encapsulates the parameters that can be specified for the BrickTexture effect. 
 **/
public class BrickTextureParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  
  /*** KEY for the integer parameter: width ***/
  public static final String PARAM_INT_WIDTH = "WIDTH";

  /*** KEY for the integer parameter: height ***/
  public static final String PARAM_INT_HEIGHT = "HEIGHT";

  /*** KEY for the integer parameter: brick width ***/
  public static final String PARAM_INT_BRICK_WIDTH = "BRICK_WIDTH";

  /*** KEY for the integer parameter: brick height ***/
  public static final String PARAM_INT_BRICK_HEIGHT = "BRICK_HEIGHT";

  /*** KEY for the integer parameter: bevel mode: 0 = NONE, 1 = RAISED, 2 = SUNK ***/
  public static final String PARAM_INT_BEVEL_MODE = "BEVEL_MODE";
  
  /*** KEY for the integer parameter: random mode: 
    0 = RGB, 1 = R, 2 = G, 3 = B, 4 = RG, 5 = RB, 6 = GB, 
    7 = HSB, 8 = H, 9 = S, 10 = B, 11 = HS, 12 = HB, 13 = SB
  ***/
  public static final String PARAM_INT_RANDOM_MODE = "RANDOM_MODE";


  /*** KEY for the color parameter: brick color ***/
  public static final String PARAM_COLOR_BRICK = "BRICK_COLOR";
  
  /*** KEY for the color parameter: border color ***/
  public static final String PARAM_COLOR_BORDER = "BORDER_COLOR";
  
  /*** KEY for the double parameter: randomize magnitude ***/
  public static final String PARAM_DOUBLE_RANDOMIZE_MAGNITUDE = "RANDOMIZE_MAGNITUDE";

  /*** KEY for the boolean parameter: draw border ***/
  public static final String PARAM_BOOLEAN_DRAW_BORDER = "DRAW_BORDER";



  /***
   *   Creates a default parameter set for the brick texture effect. The following settings are used:
   *
   *   <ul> 
   *     <li>A 400 x 400 pixel image is created.</li>
   *     <li>No border is drawn.</li>
   *     <li>The brick size is set to 20 x 10 pixels.</li>
   *     <li>Bevel mode RAISED.</li>
   *     <li>Random color mode RGB.</li>
   *     <li>Base color CYAN.</li>
   *     <li>Randomizer magnitude 150.</li>
   *   </ul>
   *
   **/
  public BrickTextureParameters() {

    //1. create the parameters and set them
    setBooleanParameters(createBooleanParameters());
    setIntegerParameters(createIntegerParameters());
    setColorParameters(createColorParameters());
    setDoubleParameters(createDoubleParameters());

  }


  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, BooleanParameter> createBooleanParameters() {

    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_BOOLEAN_DRAW_BORDER, new BooleanParameter(false));
    
    //3. return the hashtable
    return pParameters;
  }




  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_WIDTH, new IntegerParameter(400, 10, 10000));
    pParameters.put(PARAM_INT_HEIGHT, new IntegerParameter(400, 10, 10000));
    pParameters.put(PARAM_INT_BRICK_WIDTH, new IntegerParameter(20, 2, 1000));
    pParameters.put(PARAM_INT_BRICK_HEIGHT, new IntegerParameter(10, 2, 1000));
    pParameters.put(PARAM_INT_BEVEL_MODE, new IntegerParameter(1, 0, 2));
    pParameters.put(PARAM_INT_RANDOM_MODE, new IntegerParameter(0, 0, 13));
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, ColorParameter> createColorParameters() {
    //1. create the hashtable
    Hashtable<String, ColorParameter> pParameters = new Hashtable<String, ColorParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_COLOR_BRICK, new ColorParameter(Color.CYAN));
    pParameters.put(PARAM_COLOR_BORDER, new ColorParameter(Color.BLACK));
    
    //3. return the hashtable
    return pParameters;
  
  }
  
  
  
  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_RANDOMIZE_MAGNITUDE, new DoubleParameter(150, 0, 500));
    
    //3. return the hashtable
    return pParameters;
  
  }




}
