package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import java.util.Vector;
import java.util.Random;

import java.awt.geom.Line2D;


//library imports
import standard.ImageEffect;
import standard.EffectParameters;

//static imports
import static drawing.NetworkParameters.*;


/**
 *  Implements the network effect, that paints dots and connects them with lines.
 **/
public class Network extends ImageEffect {


  /*** private member variables ***/
  private NetworkParameters m_pParameters;
  
  /*** member variables for the data structures ***/
  private Vector<NetworkNode> m_pPoints;
  private Vector<Line2D> m_pLines;
  private int m_iMaximumPointSize;
  
  private int m_iOldPointNum, m_iOldMaxConnectionNum, m_iOldWidth, m_iOldHeight, m_iOldIntend;
  
   /*** member constants ***/
  private static final int BLACK_RGB = Color.BLACK.getRGB();

  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code Network} object with the default parameters.
   *
   **/
  public Network() {
    this(new NetworkParameters());
  }
  
  /**
   *   Creates a Network effect object that draws a network image with the specified parameters.
   *
   *   @param pParameters The parameters that configure the Network drawing.
   **/
  public Network(NetworkParameters pParameters) {
    m_pParameters = pParameters;
    
    m_pPoints = null;
    m_pLines = null;
    
    m_iOldPointNum = 0;
    m_iOldMaxConnectionNum = 0;
    m_iOldWidth = 0;
    m_iOldHeight = 0;
    m_iOldIntend = 0;
   
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the network effect. The parameters are only set, 
   *   if the object is of the type {@code NetworkParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof NetworkParameters) {
      m_pParameters = (NetworkParameters)pEffectParameters;
    }
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the parameters that are currently used for the network effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an network image according to the parameters that were set.
   *
   *  @return A network effect image as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    //1. make some checks
    if (m_pParameters == null) throw new NullPointerException("No parameters were specified!");
    
    
    
   
    //2. fetch the parameters
    int iNewPointNum = m_pParameters.getIntegerParameterValue(PARAM_INT_POINT_NUM);
    int iMaxConnectionNum = m_pParameters.getIntegerParameterValue(PARAM_INT_MAX_CONNECTION_NUM);
    int iMaximumDrawingPointSize = m_pParameters.getIntegerParameterValue(PARAM_INT_MAX_POINT_DRAWING_SIZE);
    int iWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_WIDTH);
    int iHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_HEIGHT);
    int iIntend = m_pParameters.getIntegerParameterValue(PARAM_INT_POINT_INTEND);
    Color pBackgroundColor = m_pParameters.getColorParameterValue(PARAM_COLOR_BACKGROUND_COLOR);
    Color pLineColor = m_pParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR);
    
    //3. check if the data structure has to be initiated
    boolean bInit = false;
    if (m_iOldPointNum != iNewPointNum) bInit = true;
    if (m_iOldMaxConnectionNum != iMaxConnectionNum) bInit = true;
    if (m_iOldWidth != iWidth) bInit = true;
    if (m_iOldHeight != iHeight) bInit = true;
    if (m_iOldIntend != iIntend) bInit = true;
    
    if (bInit) {
      initDatastructures();
      if (!isRunning()) return null;
    }
    
    
    //4. create the image
    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    
    //5. create the background
    pGraphics.setColor(pBackgroundColor);
    pGraphics.fillRect(0, 0, iWidth, iHeight);
    
    //6. draw the line connections
    pGraphics.setColor(pLineColor);
    int iLineNum = m_pLines.size();
    for (int i = 0; i < iLineNum; i++) {
      Line2D pLine = m_pLines.get(i);
      pGraphics.draw(pLine);
      if (!isRunning()) return null;
    }
    
    
    //7. draw the points
    Color pPointColor1 = m_pParameters.getColorParameterValue(PARAM_COLOR_POINT_COLOR1);
    Color pPointColor2 = m_pParameters.getColorParameterValue(PARAM_COLOR_POINT_COLOR2);
    Color pPointColor3 = m_pParameters.getColorParameterValue(PARAM_COLOR_POINT_COLOR3);
    Color pPointColor4 = m_pParameters.getColorParameterValue(PARAM_COLOR_POINT_COLOR4);
    pGraphics.setColor(pPointColor1);
    int iPointNum = m_pPoints.size();
    for (int i = 0; i < iPointNum; i++) {
      NetworkNode pPoint = m_pPoints.get(i);
      int iPointSize = pPoint.getPointSize();
      double dPointSizePercentage = (double)iPointSize/(double)m_iMaximumPointSize;
      
      if (dPointSizePercentage > 0.75) {
        pGraphics.setColor(pPointColor4);
      }
      else if (dPointSizePercentage > 0.5) {
        pGraphics.setColor(pPointColor3);
      }
      else if (dPointSizePercentage > 0.25) {
        pGraphics.setColor(pPointColor2);
      }
      else {
        pGraphics.setColor(pPointColor1);
      }

      iPointSize = (int)(iMaximumDrawingPointSize*dPointSizePercentage);
      int iHalfPointSize = iPointSize/2;
      pGraphics.fillOval(pPoint.x-iHalfPointSize, pPoint.y-iHalfPointSize, iPointSize, iPointSize);
      if (!isRunning()) return null;
    }
    
    
    
    return pImage;
    
  }
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void initDatastructures() {

    //1. init the lists and the randomizer
    int iPointNumDesign = m_pParameters.getIntegerParameterValue(PARAM_INT_POINT_NUM);
    m_pPoints = new Vector<NetworkNode>(iPointNumDesign);
    m_pLines = new Vector<Line2D>();
    Random pRandom = new Random();
    
    
    //2. create the points
    BufferedImage pMask = null; //TO DO: fetch this parameter from the parameters -> introduce imageparameters   
    int iWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_WIDTH);
    int iHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_HEIGHT);
    int iIntend = m_pParameters.getIntegerParameterValue(PARAM_INT_POINT_INTEND);
    
    if (pMask == null) {

      int iRandomWidth = iWidth-2*iIntend;
      int iRandomHeight = iHeight-2*iIntend;
      for (int i = 0; i < iPointNumDesign; i++) {
        int iXPos = pRandom.nextInt(iRandomWidth) + iIntend;
        int iYPos = pRandom.nextInt(iRandomHeight) + iIntend;

        m_pPoints.add(new NetworkNode(iXPos, iYPos));
      }
      if (!isRunning()) return;
    }
    else {
      for (int i = 0; i < iPointNumDesign; i++) { //TO DO: Improve this, to search only the pixels of the mask
        for (int j = 0; j < 10; j++) {

          int iXPos = pRandom.nextInt(iWidth);
          int iYPos = pRandom.nextInt(iHeight);

          if (isInMask(iXPos, iYPos, pMask)) {
            m_pPoints.add(new NetworkNode(iXPos, iYPos));
            break;
          }
        }
        
      }
    }

    

    
    //3. create the lines (connections)
    double dPointCheckPercentage = m_pParameters.getDoubleParameterValue(PARAM_DOUBLE_POINT_CHECK_PERCENTAGE);
    int iPointNumActual = m_pPoints.size();
    int iPointCheckNum = (int)(iPointNumActual*dPointCheckPercentage);
    int iMaxConnectionNum = m_pParameters.getIntegerParameterValue(PARAM_INT_MAX_CONNECTION_NUM);

    for (int i = 0; i < iPointNumActual; i++) {

      NetworkNode pP1 = m_pPoints.get(i);
      int iConnectionNum = pRandom.nextInt(iMaxConnectionNum) + 1;

      for (int k = 0; k < iConnectionNum; k++) {
        NetworkNode pP2 = m_pPoints.get(getNextDifferentInt(pRandom, iPointNumActual, i));
        double dDist = Math.abs(pP1.x - pP2.x) + Math.abs(pP1.y - pP2.y);

        for (int j = 0; j < iPointCheckNum; j++) {
          NetworkNode pP2New = m_pPoints.get(getNextDifferentInt(pRandom, iPointNumActual, i));
          double dDistNew = Math.abs(pP1.x - pP2New.x) + Math.abs(pP1.y - pP2New.y);

          if (dDistNew < dDist) {
            pP2 = pP2New;
            dDist = dDistNew;
          }
        }

        m_pLines.add(new Line2D.Double(pP1.x, pP1.y, pP2.x, pP2.y));
        pP1.increasePointSize();
        pP2.increasePointSize();
        
        if (!isRunning()) return;
      }
    }
    
    //4. calculate the maximum point size
    m_iMaximumPointSize = 1;
    for (int i = 0; i < iPointNumActual; i++) {
      int iCurrentPointSize = m_pPoints.get(i).getPointSize();
      if (iCurrentPointSize > m_iMaximumPointSize) m_iMaximumPointSize = iCurrentPointSize;
    }
    
    //5. set the current values for the next initiate check
    m_iOldPointNum = iPointNumDesign;
    m_iOldMaxConnectionNum = iMaxConnectionNum;
    m_iOldWidth = iWidth;
    m_iOldHeight = iHeight;
    m_iOldIntend = iIntend;
    
    
    
  }
    
    
  private static int getNextDifferentInt(Random pRandom, int iMaxNum, int iId) {

    //1. make some checks
    if (pRandom == null) return 0;
    if (iMaxNum < 2) return 0;

    //2. evaluate the next different integer
    int iNextDifferentInt = pRandom.nextInt(iMaxNum);
    while (iNextDifferentInt == iId) {
      iNextDifferentInt = pRandom.nextInt(iMaxNum);
    }
    return iNextDifferentInt;

  }

  

  private boolean isInMask(int iXPos, int iYPos, BufferedImage pMask) {

    //1. make some checks
    if (pMask == null) return false;
    if (iXPos < 0) return false;
    if (iYPos < 0) return false;
    if (iXPos >= pMask.getWidth()) return false;
    if (iYPos >= pMask.getHeight()) return false;

    //2. do the test
    int iRgb = pMask.getRGB(iXPos, iYPos);
    if (iRgb == BLACK_RGB) return false; //if the coordinate is in a black pixel return false

    return true;

  }
    



}
