package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;


//library imports
import standard.ImageEffect;
import standard.EffectParameters;

//static imports
import static drawing.StarsParameters.*;


/**
 *  Implements the stars effect, by taking the parameters and making some line drawings.
 **/
public class Stars extends ImageEffect {


  /*** private member variables ***/
  private StarsParameters m_pParameters;

  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code Stars} object with the default parameters.
   *
   **/
  public Stars() {
    this(new StarsParameters());
  }
  
  /**
   *   Creates a stars effect object that draws a stars image according to the specified parameters.
   *
   *   @param pParameters The parameters that configure the stars drawing.
   **/
  public Stars(StarsParameters pParameters) {
    m_pParameters = pParameters;
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the stars effect. The parameters are only set, 
   *   if the object is of the type {@code StarsParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof StarsParameters) {
      m_pParameters = (StarsParameters)pEffectParameters;
    }
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the parameters that are currently used for the stars effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an stars image according to the parameters that were set.
   *
   *  @return A stars effect image as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    //1. make some checks
    if (m_pParameters == null) throw new NullPointerException("No parameters were specified!");
    
   
    //2. fetch the parameters
    // TO DO
    int iWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_WIDTH);
    int iHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_HEIGHT);
    int iXTranslate = m_pParameters.getIntegerParameterValue(PARAM_INT_HORIZONTAL_TRANSLATION);
    int iYTranslate = m_pParameters.getIntegerParameterValue(PARAM_INT_VERTICAL_TRANSLATION);
    int iSpikeLength = m_pParameters.getIntegerParameterValue(PARAM_INT_SPIKE_LENGTH);
    int iSpikeNum = m_pParameters.getIntegerParameterValue(PARAM_INT_SPIKE_NUM);
    int iLineNum = m_pParameters.getIntegerParameterValue(PARAM_INT_LINE_NUM);
   
    Color pBackgroundColor = m_pParameters.getColorParameterValue(PARAM_COLOR_BACKGROUND);
    Color pLineColor = m_pParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR);
    
    double dRotation = m_pParameters.getIntegerParameterValue(PARAM_INT_ROTATION);
    dRotation = dRotation*2*Math.PI/360;
    
    //3. create an image and configure the graphics context
    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    
    //4. draw background
    pGraphics.setColor(pBackgroundColor);
    pGraphics.fillRect(0, 0, iWidth, iHeight);

    
    //5. set some drawing help variables
    int iXStart = iWidth/2 + iXTranslate;
    int iXEnd = iXStart + iSpikeLength;
    int iYStart = iHeight/2 + iYTranslate;


    //6. calculate the rotation: slopes of the first spike
    double dOldXVal = 1;
    double dOldYVal = 0;

    if (dRotation != 0) {
      dOldXVal = Math.cos(dRotation);
      dOldYVal = Math.cos(dRotation);
    }





    //7. draw all spikes
    pGraphics.setColor(pLineColor);
    
    for(int i = 0; i <= iSpikeNum; i++) {

      //draw a spike-line
      double dCurrAngle = ((double)i/(double)iSpikeNum)*(2*Math.PI) + dRotation;

      if (i == iSpikeNum) {
        dCurrAngle = 2*Math.PI + dRotation;
      }

      double dNewXVal = ((double)iSpikeLength)*Math.cos(dCurrAngle);
      double dNewYVal = ((double)iSpikeLength)*Math.sin(dCurrAngle);

      int iEndX2 = (int)((double)iXStart + dNewXVal);
      int iEndY2 = (int)((double)iYStart + dNewYVal);

      pGraphics.drawLine(iXStart,iYStart,iEndX2,iEndY2);

      //draw connections between spikes
      for (int j = 0; j < iLineNum; j++) {
        int iX1 = (int)((double)iXStart + dOldXVal*(((double)iLineNum-j)/(double)iLineNum));
        int iY1 = (int)((double)iYStart + dOldYVal*(((double)iLineNum-j)/(double)iLineNum));
        int iX2 = (int)((double)iXStart + dNewXVal*(((double)j)/(double)iLineNum));
        int iY2 = (int)((double)iYStart + dNewYVal*(((double)j)/(double)iLineNum));

        pGraphics.drawLine(iX1,iY1,iX2,iY2);

      }
      dOldXVal = dNewXVal;
      dOldYVal = dNewYVal;
    }
    
    return pImage;
    
  }

}
