package fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */
 


//standard imports
import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.color.ColorGradient;
import util.math.ComplexNumber;

import static fractals.FractalSetParameters.*;

 
public abstract class FractalSetEffect extends ImageEffect {
  
  private FractalSetParameters m_pParameters = null;
  private Iterator m_pIterator = null;
  private Raster m_pRaster = null;
  private boolean m_bFetchParameters = true;
  private boolean m_bRecalculateRaster = true;
  
  
   //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *    Returns the complex number that it is represented by the specified position in the image.
   *    
   *    @param iXPos The horizontal position as integer.
   *    @param iYPos The vertical position as integer.
   *
   *    @return The {@code ComplexNumber} object representing the complex number of the position in the image.
   **/
  public ComplexNumber getComplexNumberAt(int iXPos, int iYPos) {
    if (m_pIterator == null) return null;
    return m_pIterator.getComplexNumberAt(iXPos, iYPos);
  }
  
  public Point getNearestPixelOfComplexNumber(ComplexNumber pComplexNumber) {
    if (m_pIterator == null) return null;
    return m_pIterator.getNearestPixelOfComplexNumber(pComplexNumber);
  }
  
  
  /**
   *   Returns the parameters that are currently used for the Mandelbrot set effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the Mandelbrot set effect. 
   *
   *   @param pParameters The parameters that configure the effect. 
   **/
  public void setFractalSetParameters(FractalSetParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    //2. set the member variables
    m_pParameters = pParameters;
    m_bFetchParameters = true;
  }
  
  /**
   *   Sets the parameters for the Mandelbrot set effect. The parameters are only set, 
   *   if the object is of the type {@code MandelbrotSetParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof FractalSetParameters) {
      setFractalSetParameters((FractalSetParameters)pEffectParameters);
    }
  }
  
  


 
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image of the Julia set according to the parameters that were set.
   *
   *  @return An image of the Julia set as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    //1. check if the parameters have to be reset
    if (m_bFetchParameters || (m_pIterator == null)) {
      fetchParameters();
    }
    
    
    //2. if necessary recalculate the raster
    Raster pRaster = m_pIterator.getRaster(this);
    
    //3. create the image
    ColorGradient pColorGradient = null;
    try {
      pColorGradient = (ColorGradient)m_pParameters.getObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT);
    }
    catch (Exception pException) {
      pColorGradient = new ColorGradient();  
    }
    
    BufferedImage pImage = pRaster.getImage(this, pColorGradient, Color.BLACK);
    
    return pImage;
    
  }
  
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void fetchParameters() {
    
       
    //3. recreate the iterator if necessary
    if (m_pIterator == null) {
      m_pIterator = new Iterator(m_pParameters);
    }
    else  {
      m_pIterator.setParameters(m_pParameters);
    }
    
    //4. set the flags
    m_bFetchParameters = false;
    
    
    
  }
  
  
  
}