package fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;
import java.util.Vector;

//library imports
import standard.EffectParameters;
import standard.DoubleParameter;
import standard.IntegerParameter;
import standard.ObjectParameter;

import util.color.ColorGradient;

import util.math.ComplexNumber;

/**
 *   Encapsulates the parameters that can be specified for the MandelbrotSet object.
 **/
public abstract class FractalSetParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: iteration depth  ***/
  public static final String PARAM_INT_ITERATION_DEPTH = "ITERATION_DEPTH";

  /*** KEY for the integer parameter: width ***/
  private static final String PARAM_INT_WIDTH = "WIDTH";

  /*** KEY for the int parameter: height ***/
  private static final String PARAM_INT_HEIGHT = "HEIGHT";

  /*** KEY for the double parameter: decision limit ***/
  public static final String PARAM_DOUBLE_DECISION_LIMIT = "DECISION_LIMIT";


  /*** KEY for the object parameter: left upper window corner (ComplexNumber) ***/
  private static final String PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER = "UPPER_LEFT_WINDOW_CORNER";

  /*** KEY for the object parameter: right lower window corner (ComplexNumber) ***/
  private static final String PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER = "LOWER_RIGHT_WINDOW_CORNER";

  /*** KEY for the object parameter: right lower window corner (ColorGradient) ***/
  public static final String PARAM_OBJECT_COLOR_GRADIENT = "COLOR_GRADIENT";

  /*** KEY for the object parameter: complex number z  ***/
  public static final String PARAM_OBJECT_COMPLEX_NUMBER_Z = "COMPLEX_NUMBER_Z";

  /*** KEY for the object parameter: complex number c  ***/
  public static final String PARAM_OBJECT_COMPLEX_NUMBER_C = "COMPLEX_NUMBER_C";




  ///////////////////////////////////////
  //  PROTECTED METHODS FOR SUBCLASSES //
  ///////////////////////////////////////
  protected void createParameters() {
    //1. create the parameters and set them
    setIntegerParameters(createIntegerParameters());
    setProtectedIntegerParameters(createProtectedIntegerParameters());
    setDoubleParameters(createDoubleParameters());
    setObjectParameters(createObjectParameters());
    setProtectedObjectParameters(createProtectedObjectParameters());
  }

  protected abstract ComplexNumber getDefaultUpperLeftWindowCorner();

  protected abstract ComplexNumber getDefaultLowerRightWindowCorner();

  protected abstract Hashtable<String, ComplexNumber> getComplexNumberParameters();





  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_ITERATION_DEPTH, new IntegerParameter(100, 5, 50000));

    //3. return the hashtable
    return pParameters;
  }

  private Hashtable<String, IntegerParameter> createProtectedIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_WIDTH, new IntegerParameter(400, 2, 50000));
    pParameters.put(PARAM_INT_HEIGHT, new IntegerParameter(400, 2, 50000));

    //3. return the hashtable
    return pParameters;
  }


  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_DECISION_LIMIT, new DoubleParameter(2, 0.01, 100));

    //3. return the hashtable
    return pParameters;
  }

  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_OBJECT_COLOR_GRADIENT, new ObjectParameter(new ColorGradient()));

    //TODO: check if those should be protected?? but then they cannot be handed over to the iterator
    Hashtable<String, ComplexNumber> pComplexNumbers = getComplexNumberParameters();
    Vector<String> pParamNames = new Vector<String>();
    pParamNames.add(PARAM_OBJECT_COMPLEX_NUMBER_Z);
    pParamNames.add(PARAM_OBJECT_COMPLEX_NUMBER_C);

    for (int i = 0; i < pParamNames.size(); i++) {
      String pParam = pParamNames.get(i);
      if (pComplexNumbers.containsKey(pParam)) {
        pParameters.put(pParam, new ObjectParameter(pComplexNumbers.get(pParam)));
      }
      else {
        pParameters.put(pParam, new ObjectParameter(null));
      }
    }

    //3. return the hashtable
    return pParameters;
  }



  private Hashtable<String, ObjectParameter> createProtectedObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, new ObjectParameter(new ComplexNumber(-2, 1.5)));
    pParameters.put(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, new ObjectParameter(new ComplexNumber(1, -1.5)));



    //3. return the hashtable
    return pParameters;
  }


  //////////////////////////////
  // CHANGE METHODS           //
  //////////////////////////////
  public void moveDown(double dStepSize) {
    moveWorker(0, -1*dStepSize);
  }

  public void moveUp(double dStepSize) {
     moveWorker(0, dStepSize);
  }

  public void moveLeft(double dStepSize) {
     moveWorker(-1*dStepSize, 0);
  }

  public void moveRight(double dStepSize) {
    moveWorker(dStepSize, 0);
  }

  public void setWindowCorners(double dUpperLeftReal, double  dUpperLeftImaginary, double dLowerRightReal, double dLowerRightImaginary) {
    setWindowCorners(
      new ComplexNumber(dUpperLeftReal, dUpperLeftImaginary),
      new ComplexNumber(dLowerRightReal, dLowerRightImaginary));
  }

  public void setWindowCorners(ComplexNumber pUpperLeft, ComplexNumber pLowerRight) {
    boolean bLowerRightCorner = false;

    if (pUpperLeft != null) {
      if (!pUpperLeft.equals(getUpperLeftWindowCorner())) {
        setProtectedObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, pUpperLeft);
        bLowerRightCorner = true;
      }
    }

    if (pLowerRight != null) {
      setProtectedObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, pLowerRight);
    }

    recalculateCorner(bLowerRightCorner);
  }




  public void setEffectSize(int iWidth, int iHeight) {
    setProtectedIntegerParameterValue(PARAM_INT_WIDTH, iWidth);
    setProtectedIntegerParameterValue(PARAM_INT_HEIGHT, iHeight);

    recalculateCorner(true);
  }

  public void setDefaultFractalSet() {
    setProtectedObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, getDefaultUpperLeftWindowCorner());
    setProtectedObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, getDefaultLowerRightWindowCorner());
    recalculateCorner(true);
  }



  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  public ComplexNumber getUpperLeftWindowCorner() {
    return (ComplexNumber)getProtectedObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER);
  }

  public ComplexNumber getLowerRightWindowCorner() {
    return (ComplexNumber)getProtectedObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER);
  }

  public int getWidth() {
    return getProtectedIntegerParameterValue(PARAM_INT_WIDTH);
  }

  public int getHeight() {
    return getProtectedIntegerParameterValue(PARAM_INT_HEIGHT);
  }


  ////////////////////////////////
  // PRIVATE METHODS            //
  ////////////////////////////////
  private void recalculateCorner(boolean bLowerRightCorner) {
    double dAspectRatio = (double)getHeight()/(double)getWidth();
    ComplexNumber pUpperLeft = getUpperLeftWindowCorner();
    ComplexNumber pLowerRight = getLowerRightWindowCorner();


    double dWindowWidth = pLowerRight.m_dReal - pUpperLeft.m_dReal;
    double dNewWindowHeight = dWindowWidth*dAspectRatio;

    if (bLowerRightCorner) {
      ComplexNumber pNewLowerRight = new ComplexNumber(pLowerRight.m_dReal, pUpperLeft.m_dImaginary-dNewWindowHeight);
      setProtectedObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, pNewLowerRight);
    }
    else {
      ComplexNumber pNewUpperLeft = new ComplexNumber(pUpperLeft.m_dReal, pLowerRight.m_dImaginary+dNewWindowHeight);
      setProtectedObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, pNewUpperLeft);
    }



  }

  private void moveWorker(double dXOffset, double dYOffset) {
    ComplexNumber pUpperLeft = getUpperLeftWindowCorner();
    ComplexNumber pLowerRight = getLowerRightWindowCorner();
    ComplexNumber pAdd = new ComplexNumber(dXOffset, dYOffset);

    pUpperLeft.addToThis(pAdd);
    pLowerRight.addToThis(pAdd);

    setWindowCorners(pUpperLeft, pLowerRight);
  }

}
