package fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//TODO: temporary imports
import java.awt.Point;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

//library imports
import standard.ImageEffect;
import util.math.ComplexNumber;

//static imports
import static fractals.FractalSetParameters.*;

/***
 *
 *  An iterator for juliasets and the mandelbrot set and their modified kinds to calculate a 
 *  two dimensional Raster which can be used to create images.
 *
 **/
public class Iterator {

  /*** private member variables ***/
  private int m_iIterationDepth;   //how long should we iterate
  private double[] m_pWindow;      //in which area should we iterate
  private int m_iWidthPixels;      //width pixels
  private int m_iHeightPixels;     //height pixels
  private ComplexNumber m_pZ;      //complex number z
  private ComplexNumber m_pC;      //complex number c
  private double m_dDecisionLimit; //decision limit
  private Raster m_pRaster;
  private boolean m_bRecalculateRaster;
  //TO DO FORMULA
  
  /////////////////////////////////
  // CONSTRUCTORS                //
  /////////////////////////////////
  /**
   *   This standard constructor initializes the iterator with the defined parameters. If no parameters are set a raster for the 
   *   Mandelbrot set is created. In fact the following values are used:
   *   <ul>
   *     <li>Iteration depth: 100</li>
   *     <li>Window X(-2,1) Y(-1.5,1.5)</li>
   *     <li>Raster size 400 x 400</li>
   *     <li>Formulae z = z*z + c</li>
   *     <li>z = 0</li>
   *     <li>c = P(x,y)</li>
   *     <li>Decision limit = 2</li>
   *   </ul>
   *
   *   @param pParameters The iterator parameters that should be used. If it is {@code null} then the default values are used.
   **/
  public Iterator(FractalSetParameters pParameters) {
  
    if (pParameters == null) pParameters = new MandelbrotSetParameters();
    initParameters();
    setParameters(pParameters);
    
    
  }
  
  ////////////////////////////////////
  // PRIVATE HELPER METHOD          //
  ////////////////////////////////////
  
  
  
  /////////////////////////////////
  // SET METHODS                 //
  /////////////////////////////////
  /**
   *   Sets the parameters for the iterator to calculate the raster. 
   *
   *   @param pParameters The parameters that configure the raster calculation. 
   **/
  public void setParameters(FractalSetParameters pParameters) {
    //1. check the parameters
    if (pParameters == null) return;
    
    //2. fetch the parameters
    fetchParameters(pParameters);
  }


  
  
  
  /////////////////////////////////
  // GET METHODS                 //
  /////////////////////////////////
  /***
   *    Calculates and returns the raster according to the properties that were set.
   *    
   *    @param pImageEffect The image effect object that is used to check if the process is still running. (TO DO: switch to a listener interface!)
   *
   *    @return A Raster object holding a two-dimensional integer array with iteration values for each position entry.
   **/
  public Raster getRaster(ImageEffect pImageEffect) {
    
    if (pImageEffect == null) return null;
    if (!m_bRecalculateRaster) return m_pRaster;
    
    
      
    //1. create the raster
    int[][] pRaster = new int[m_iWidthPixels][m_iHeightPixels];
    
    //2. check which number has to be set to the complex number that the current pixel represents
    boolean bCalculateZ = (m_pZ == null);
    boolean bCalculateC = (m_pC == null);
    
    //3. calculate the x and y step size
    double dXStep = getXStep();
    double dYStep = getYStep();
    
   
    
    
    //4. fill the raster
   double dXPos = m_pWindow[0];
    for (int x = 0; x < m_iWidthPixels; x++) {
      double dYPos = m_pWindow[2];
    
      for (int y = 0; y < m_iHeightPixels; y++) {
        
        if (bCalculateZ) m_pZ = new ComplexNumber(dXPos, dYPos);
        if (bCalculateC) m_pC = new ComplexNumber(dXPos, dYPos);
        
        ComplexNumber m_pZIterate = new ComplexNumber(m_pZ.m_dReal, m_pZ.m_dImaginary);
        
        int i = 0;
        for (i = 0; i < m_iIterationDepth; i++) {
          if (m_pZIterate.getLength() > m_dDecisionLimit) {
            break;
          }
          
          
          m_pZIterate.multiplyWithThis(m_pZIterate);
          m_pZIterate.addToThis(m_pC);
          
        }
        
        
        pRaster[x][y] = i;
        
        
        //make the check if we still should proceed
        if (!pImageEffect.isRunning()) return m_pRaster;
        
        
        
        dYPos += dYStep;
      }
      dXPos += dXStep;
    }
    
    //5. set the members back
    if (bCalculateZ) m_pZ = null;
    if (bCalculateC) m_pC = null;
    
    
    m_pRaster = new Raster(pRaster, 0, m_iIterationDepth);
    return m_pRaster;
    
  }
  
  
  /**
   *    Returns the complex number that it is represented by the specified position in the raster.
   *    
   *    @param iXPos The horizontal position as integer.
   *    @param iYPos The vertical position as integer.
   *
   *    @return The {@code ComplexNumber} object representing the complex number of the position in the raster.
   **/
  public ComplexNumber getComplexNumberAt(int iXPos, int iYPos) {
    
    double dReal = m_pWindow[0] + iXPos*getXStep();
    double dImaginary = m_pWindow[2] + iYPos*getYStep();
    
    return new ComplexNumber(dReal, dImaginary);
  }
  
  
  public Point getNearestPixelOfComplexNumber(ComplexNumber pComplexNumber) {
    if (pComplexNumber == null) return new Point(0, 0);
    
    double dXPos = (pComplexNumber.m_dReal - m_pWindow[0])/getXStep();
    double dYPos = (pComplexNumber.m_dImaginary - m_pWindow[2])/getYStep();
    return new Point((int)Math.round(dXPos), (int)Math.round(dYPos));
  }
  
  /***
   *   A testing method to create an image out of a raster calculation.
   *   TO DO: This should be obsolete. JUnitTests should be created.
   *   
   *   @param args The arguments array. TO DO: get rid of this method and create a unit test!!!
   *
   **/ 
  public static void main(String[] args) {
    /*Iterator pIterator = new Iterator();
    pIterator.setIterationDepth(255);
    pIterator.setRasterSize(800, 800);
    int[][] pRaster = pIterator.getRaster();
    
    int iWidth = pRaster.length;
    int iHeight = pRaster[0].length;
    
    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    for (int x = 0; x < iWidth; x++) {    
      for (int y = 0; y < iHeight; y++) {
        int iValue = pRaster[x][y];
        Color pColor = new Color(iValue, iValue, iValue);
        if (iValue == 255) pColor = Color.BLACK;
        pImage.setRGB(x, y, pColor.getRGB());
      }
    }
    
    try {
      ImageIO.write(pImage, "png", new File("test.png"));
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }*/
    
  
  }
  
  
  ////////////////////////////
  // PRIVATE HELPER METHODS //
  ////////////////////////////
  private void initParameters() {
    m_iIterationDepth = -1;
    m_pWindow = new double[4];
    m_pWindow[0] = 0;
    m_pWindow[1] = 0;
    m_pWindow[2] = 0;
    m_pWindow[3] = 0;
    m_iWidthPixels = 0;
    m_iHeightPixels = 0;
    m_pZ = null;
    m_pZ = null;
    m_pC = null;
    m_dDecisionLimit = 0;
    m_pRaster = null;
    m_bRecalculateRaster = true;
    
    
  }
  
  private void fetchParameters(FractalSetParameters pParameters) {
    //1. integer parameters
    m_iIterationDepth = getAndCompareIntParameter(pParameters, PARAM_INT_ITERATION_DEPTH, m_iIterationDepth);
    m_iWidthPixels = getAndCompareIntParameter(pParameters.getWidth(), m_iWidthPixels);
    m_iHeightPixels = getAndCompareIntParameter(pParameters.getHeight(), m_iHeightPixels);
    
    //2. double parameters
    m_dDecisionLimit= getAndCompareDoubleParameter(pParameters, PARAM_DOUBLE_DECISION_LIMIT, m_dDecisionLimit);
    
    //3. complex number parameters
    m_pZ = getAndCompareComplexNumberParameter(pParameters, PARAM_OBJECT_COMPLEX_NUMBER_Z, m_pZ);
    m_pC = getAndCompareComplexNumberParameter(pParameters, PARAM_OBJECT_COMPLEX_NUMBER_C, m_pC);
   
    
    //4. window
    
    ComplexNumber pUpperLeftCorner = pParameters.getUpperLeftWindowCorner();
    ComplexNumber pLowerRightCorner = pParameters.getLowerRightWindowCorner();
    if ((m_pWindow[0] != pUpperLeftCorner.m_dReal) || (m_pWindow[1] != pLowerRightCorner.m_dReal) ||
        (m_pWindow[2] != pUpperLeftCorner.m_dImaginary) || (m_pWindow[3] != pLowerRightCorner.m_dImaginary)) {
      m_bRecalculateRaster = true;
    }
    
    m_pWindow = new double[4];
    m_pWindow[0] = pUpperLeftCorner.m_dReal;
    m_pWindow[1] = pLowerRightCorner.m_dReal;
    m_pWindow[2] = pUpperLeftCorner.m_dImaginary;
    m_pWindow[3] = pLowerRightCorner.m_dImaginary;
    
    
  }
  
  private int getAndCompareIntParameter(FractalSetParameters pParameters, String pParameterKey, int iOldValue) {
    int iNewValue = pParameters.getIntegerParameterValue(pParameterKey);
    return getAndCompareIntParameter(iNewValue, iOldValue);
  }  
  
  private int getAndCompareIntParameter(int iNewValue, int iOldValue) {
    if (iNewValue != iOldValue) {
      m_bRecalculateRaster = true;
    }
    return iNewValue;
  }
  
  private double getAndCompareDoubleParameter(FractalSetParameters pParameters, String pParameterKey, double dOldValue) {
    double dNewValue = pParameters.getDoubleParameterValue(pParameterKey);
    if (dNewValue != dOldValue) {
      m_bRecalculateRaster = true;
    }
    return dNewValue;
  }  
  
  private ComplexNumber getAndCompareComplexNumberParameter(FractalSetParameters pParameters, String pParameterKey, ComplexNumber pOldValue) {
    Object pNewValue = pParameters.getObjectParameterValue(pParameterKey);
    if (pNewValue == null) {
      if (pOldValue != null) m_bRecalculateRaster = true;
      return null;
    }
    
    //copy the value, otherwise there might be problems
    ComplexNumber pNewValue2 = new ComplexNumber((ComplexNumber)pNewValue);
    if (pOldValue == null) {
      m_bRecalculateRaster = true;
      return pNewValue2;
    }
    
    if (!pNewValue2.equals(pOldValue)) m_bRecalculateRaster = true;
    return pNewValue2;
  }
  
  private double getXStep() {
    return (m_pWindow[1]-m_pWindow[0])/(double)m_iWidthPixels;
  }
  
  private double getYStep() {
    return (m_pWindow[3]-m_pWindow[2])/(double)m_iHeightPixels;
  }

}
