package fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.util.Vector;

//library imports
import standard.EffectParameters;
import standard.ImageEffect;
import util.image.ImageCombineOperations;
import util.math.ComplexNumber;

import static fractals.FractalSetParameters.*;

/**
 *  Implements the Mandelbrot set effect.
 **/
public class JuliaSetWithMandelbrotSet extends ImageEffect {

  //private members
  private MandelbrotSetParameters m_pMBParams;
  private JuliaSetParameters m_pJuliaParams;
  private MandelbrotSet m_pMandelbrot;
  private JuliaSet m_pJulia;
  private BufferedImage m_pMandelbrotImage, m_pJuliaImage;
  private boolean m_bJuliaChanged, m_bMBChanged;

  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code MandelbrotSet} object with the default parameters.
   *
   **/
  public JuliaSetWithMandelbrotSet() {
    this(new JuliaSetParameters(), new MandelbrotSetParameters());
  }


  /**
   *   Creates a {@code MandelbrotSet} object that draws a part of the mandelbrot set according to the specified parameters.
   *
   *   @param pParameters The parameters that configure the Mandelbrot set image.
   **/
  public JuliaSetWithMandelbrotSet(JuliaSetParameters pJuliaParams, MandelbrotSetParameters pMBParams) {
    //1. Make an initial check
    if (pJuliaParams == null) pJuliaParams = new JuliaSetParameters();
    if (pMBParams == null) pMBParams = new MandelbrotSetParameters();

    //2. set the member variables
   m_pJuliaParams = pJuliaParams;
   m_pMBParams = pMBParams;

   m_pJulia = new JuliaSet();
   m_pMandelbrot = new MandelbrotSet();

   m_pMandelbrotImage = null;
   m_pJuliaImage = null;

   m_bJuliaChanged = true;
   m_bMBChanged = true;
  }

  //////////////////////////////////
  // PROCESS METHODS              //
  //////////////////////////////////
  /**
   *  Processes the image effect and returns the result image.
   *
   *  @return The result image of the image effect as {@code BufferedImage} object.
   *
   *  @throws Exception if there are errors when processing the image effect.
   **/
  public BufferedImage process() throws Exception {

    if (m_bMBChanged || m_pMandelbrotImage == null) {
      m_pMandelbrot.setEffectParameters(m_pMBParams);
      m_pMandelbrotImage = m_pMandelbrot.process();
    }

    if (m_bJuliaChanged || m_pJuliaImage == null) {
      m_pJulia.setEffectParameters(m_pJuliaParams);
      m_pJuliaImage = m_pJulia.process();
    }


    Vector<BufferedImage> pImages = new Vector<BufferedImage>();
    pImages.add(m_pMandelbrotImage);
    pImages.add(m_pJuliaImage);
    return ImageCombineOperations.concatenateImages(pImages);




  }

  public BufferedImage getJuliaImage() {
    return m_pJuliaImage;
  }
  public BufferedImage getMandelbrotImage() {
    return m_pMandelbrotImage;
  }


  //////////////////////////////////
  // SET METHODS                  //
  //////////////////////////////////
  /**
   *  Sets the image effect parameters that configure the effect.
   *
   *  @param pParameters The parameters for this image effect.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    if (pParameters == null) return;

    if (pParameters instanceof MandelbrotSetParameters) {
      setMandelbrotSetParameters((MandelbrotSetParameters)pParameters);
    }
    else if (pParameters instanceof JuliaSetParameters) {
      setJuliaSetParameters((JuliaSetParameters)pParameters);
    }
  }

  public void setMandelbrotSetParameters(MandelbrotSetParameters pParameters) {
    if (pParameters == null) return;
    m_pMBParams = pParameters;
    m_bMBChanged = true;
  }

  public void setJuliaSetParameters(JuliaSetParameters pParameters) {
    if (pParameters == null) return;
    m_pJuliaParams = pParameters;
    m_bJuliaChanged = true;
  }

  //////////////////////////////////
  // GET METHODS                  //
  //////////////////////////////////

  /**
   *  Returns the image effect parameters that configure the effect.
   *
   *  @return The parameters for this image effect.
   **/
  public EffectParameters getEffectParameters() {
    return getJuliaSetParameters();
  }

  public JuliaSetParameters getJuliaSetParameters() {
    return m_pJuliaParams;
  }

  public MandelbrotSetParameters getMandelbrotSetParameters() {
    return m_pMBParams;
  }

  public boolean isJuliaSet(int iXPos) {
    return (iXPos > m_pMBParams.getWidth());
  }

  public ComplexNumber getComplexNumberAt(int iXPos, int iYPos) {
    if (isJuliaSet(iXPos)) {
      return m_pJulia.getComplexNumberAt(iXPos - m_pMBParams.getWidth(), iYPos);
    }
    else {
      return m_pMandelbrot.getComplexNumberAt(iXPos, iYPos);
    }
  }

  public Point getNearesPixelOfParameterC() {
    ComplexNumber pCParameter = (ComplexNumber)m_pJuliaParams.getObjectParameterValue(PARAM_OBJECT_COMPLEX_NUMBER_C);
    return m_pMandelbrot.getNearestPixelOfComplexNumber(pCParameter);
  }






}
