package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JComboBox;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;

import java.awt.FlowLayout;

import java.util.Vector;


//library imports
import standard.EffectParameters;
import applications.ISettingsListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;


/**
 *  Represents a panel that shows parameters of an image effect.
 *
 */
public abstract class AbstractPropertiesPanel extends JPanel implements ISettingsListener, ActionListener, ComponentListener {

  /*** member variables ***/
  private Vector<IPropertiesPanelListener> m_pPropertiesPanelListeners = null;
  private JComboBox<String> m_pPropertiesSelection = null;
  private Vector<JPanel> m_pPropertyPanels = null;
  private JPanel m_pCurrentPropertyPanel = null;

  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
   /**
   *   Sets the parameters for the image effect that should be modified in this panel.
   *
   *   @param pEffectParameters The parameters that define the settings of the image effect.
   **/
  public abstract void setEffectParameters(EffectParameters pEffectParameters);


  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the image effect parameters that the settings of the properties panel represent.
   *
   *   @return The image effect parameters that the settings of the properties panel represent.
   **/
  public abstract EffectParameters getEffectParameters();

  /**
   *  Returns the file name of the source image file, if possible.
   *  Has to be overwritten by the implementing class.
   *
   *  @return Name of the source image file.
   **/
  public String getFileName() {
    return null;
  }

  //////////////////////////////////////
  // PROTECTED GET METHODS            //
  //////////////////////////////////////

  protected abstract Vector<JPanel> getPropertyPanels();

  protected abstract Vector<String> getPropertyKeys();



  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent
   *  and the properties panel can adapt accordingly.
   *  Children must overwrite the method applyNewSettingsForChildren.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public final void applyNewSettings(Settings pSettings) {
    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    populatePropertiesSelection(pLanguageProperties, getPropertyKeys());

    applyNewSettingsForChildren(pSettings);

  }



  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener object that listens to changes in this properties panel.
   *
   *  @param pListener The listener that listens to changes in this properties panel.
   **/
  public void addPropertiesPanelListener(IPropertiesPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pPropertiesPanelListeners == null) m_pPropertiesPanelListeners = new Vector<IPropertiesPanelListener>();
    if (m_pPropertiesPanelListeners.contains(pListener)) return;

    //2. add it
    m_pPropertiesPanelListeners.add(pListener);
  }

  /**
   *  Removes a listener object that should no longer listen to changes in this properties panel.
   *
   *  @param pListener The listener that should no longer listen to changes in this properties panel.
   **/
  public void removePropertiesPanelListener(IPropertiesPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pPropertiesPanelListeners == null) return;

    //2. add it
    m_pPropertiesPanelListeners.remove(pListener);
  }

  /**
   *  Removes all listeners from this properties panel.
   *
   **/
  public void removeAllPropertiesPanelListener() {
    //1. make some checks
    if (m_pPropertiesPanelListeners == null) return;

    //2. add it
    m_pPropertiesPanelListeners.clear();
  }


  //////////////////////////////////////////////
  // PROTECTED LISTENER METHODS FOR INSTANCES //
  //////////////////////////////////////////////
  /**
   *  Informs all registered listeners that a change in the properties panel has been made. The image effect parameters
   *  of this properties panel are passed on to the listeners.
   *
   *  @param pEffectParameters The image effect parameters that should be passed on to the listeners.
   *
   **/
  protected void fireApplyChanges(EffectParameters pEffectParameters) {
    //1. make some checks
    if (pEffectParameters == null) return;
    if (m_pPropertiesPanelListeners == null) return;

    //2. fire the event
    int iListenerNum = m_pPropertiesPanelListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pPropertiesPanelListeners.get(i).applyChanges(pEffectParameters);
    }


  }

   //////////////////////////
  // ACTION LISTENER       //
  ///////////////////////////
  /**
   *   Applies a reset of the Mandelbrot set effect to the default setting respectively detects
   *   which parameter should be shown now in the properties panel.
   *   Children have to overwrite the method actionPerformedForChildren()
   *
   *   @param e The action event triggering this method.
   **/

  public final void actionPerformed(ActionEvent e) {

    //1. get source
    Object pSource = e.getSource();
    if (pSource == null) return;

    if (pSource.equals(m_pPropertiesSelection)) {
      switchPropertiesPanel();
      return;
    }

    actionPerformedForChildren(e);




  }

  //////////////////////////////////
  // COMPONENT LISTENER           //
  //////////////////////////////////
  /**
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public final void  componentHidden(ComponentEvent e) {}

  /**
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public final void componentMoved(ComponentEvent e) {}

  /**
   *  Resizes the color gradient panel when the size of this panel has been changed.
   *  Children have to overwrite the method actionPerformedForChildren()
   *
   *  @param e The event triggering this method.
   **/
  public final void componentResized(ComponentEvent e) {
    componentResizedForChildren(e, m_pCurrentPropertyPanel);
  }

  /**
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public final void componentShown(ComponentEvent e) {}





  ////////////////////////////////////////////
  // PROTECTED METHODS FOR CHILDREN         //
  ////////////////////////////////////////////
  protected void initLayout() {

    //1. set layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));


    //2. init the properties combo box
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();


    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties, getPropertyKeys());

    //3. set the current properties panel
    m_pPropertyPanels = getPropertyPanels();
    switchPropertiesPanel();

    addComponentListener(this);
  }

  /**
   *  Returns the width of the properties combo box.
   *
   *  @return The width of the properties combo box.
   **/
  protected int getPropertiesSelectionWidth() {
    if (m_pPropertiesSelection == null) return 0;
    return m_pPropertiesSelection.getWidth();
  }

  /////////////////////////////////////////////
  // PROTECTED LISTENER METHODS FOR CHILDREN //
  /////////////////////////////////////////////
  //children need to implement this, if they have to react on action events
  protected void actionPerformedForChildren(ActionEvent e) {

  }

  //children need to implement this, if they have to react on the new settings
  protected void applyNewSettingsForChildren(Settings pSettings) {


  }

  //children need to implement this, if they have to react on resizing events
  protected void componentResizedForChildren(ComponentEvent e, JPanel currentPropertyPanel) {


  }

  //children need to implement this, if there is a special reaction when a properties panel has been changed
  protected void propertyPanelChanged(JPanel currentPropertyPanel) {

  }



  /////////////////////////////////////
  // PRIVATE METHODS                 //
  /////////////////////////////////////

  /**
   *  Creates a combo box for a properties panel that should hold all the different properties that can be selected.
   *
   *  @param pActionListener An action listener that listens to the changes the selected value of the properties combo box. Usually this is the actual properties panel to show the current properties.
   *
   **/
  private JComboBox initPropertiesComboBox(ActionListener pActionListener) {
    m_pPropertiesSelection = new JComboBox<String>();
    m_pPropertiesSelection.addActionListener(pActionListener);
    return m_pPropertiesSelection;
  }

  /**
   *  Returns the selected index of the properties combo box.
   *
   *  @return The selected index of the properties combo box.
   **/
  private int getSelectedPropertiesIndex() {
    if (m_pPropertiesSelection == null) return 0;

    int iSelectedId = m_pPropertiesSelection.getSelectedIndex();
    if (iSelectedId < 0) iSelectedId = 0;
    return iSelectedId;
  }

  /**
   *  Fills the combo box with the property labels defined by the language properties and the given keys.
   *
   *  @param pLanguageProperties The language properties that hold the language specific labels.
   *  @param pPropertyKeys A list of strings that are the keys for the property labels.
   **/
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties, Vector<String> pPropertyKeys) {
    if (m_pPropertiesSelection == null) return;

    switchComboBoxItemLabels(m_pPropertiesSelection, pLanguageProperties, pPropertyKeys);


  }


  private void switchPropertiesPanel() {
    if (m_pPropertyPanels == null) return;

    int iSelectedId = getSelectedPropertiesIndex();
    if (iSelectedId >= m_pPropertyPanels.size()) return;
    JPanel pNewPanel = m_pPropertyPanels.get(iSelectedId);

    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    m_pCurrentPropertyPanel = pNewPanel;
    if (m_pCurrentPropertyPanel != null) add(m_pCurrentPropertyPanel);
    repaint();
    revalidate();

    propertyPanelChanged(m_pCurrentPropertyPanel);


  }


  //////////////////////////////////////
  // PUBLIC STATIC UTILITY METHODS    //
  //////////////////////////////////////
  //TODO: isn't it better to move this somewhere else?
  public static void switchComboBoxItemLabels(
    JComboBox<String> pComboBox,
    LanguageProperties pLanguageProperties,
    Vector<String> pPropertyKeys) {

    if (pComboBox == null) return;
    if (pLanguageProperties == null) return;
    if (pPropertyKeys == null) return;

    int iSelectedId = pComboBox.getSelectedIndex();
    if (iSelectedId < 0) iSelectedId = 0;

    pComboBox.removeAllItems();
    int iPropertyNum = pPropertyKeys.size();
    for (int i = 0; i < iPropertyNum; i++) {
      String pPropertyKey = pPropertyKeys.get(i);
      String[] pSplittedPropertyKey = pPropertyKey.split(";");
      String pValue = "";
      for (int j = 0; j < pSplittedPropertyKey.length; j++) {
        pValue += pLanguageProperties.getProperty(pSplittedPropertyKey[j]);
      }
      pComboBox.addItem(pValue);
    }

    pComboBox.setSelectedIndex(iSelectedId);

  }



}
