package gui;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


import javax.swing.JPanel;
import java.util.Vector;




/**
 *  Represents a panel that shows the result of an image effect in the gui application.
 *
 */
public abstract class JPanelWithListener<PanelListener> extends JPanel {

  /** private listener variables **/
  private Vector<PanelListener> m_pListeners = null;




  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener for this panel.
   *
   *  @param pListener The listener for this panel that should be added.
   **/
  public void addPanelListener(PanelListener pListener) {
    if (m_pListeners == null) m_pListeners = new Vector<PanelListener>();
    m_pListeners.add(pListener);
  }

  /**
   *  Removes the specified listener from this panel.
   *
   *  @param pListener The listener that should be removed.
   **/
  public void removePanelListener(PanelListener pListener) {
    if (m_pListeners == null) return;
    m_pListeners.remove(pListener);
  }


  /**
   *  Removes all listeners from this {@code ComboBoxPanel}.
   *
   **/
  public void removeAllPanelListeners() {
    if (m_pListeners == null) return;
    m_pListeners.clear();
  }

  /////////////////////////////
  // PROTECTED METHODS       //
  /////////////////////////////
  /**
   *  When a change is made to the panel this is communicated to all listeners.
   *
   **/
  protected void applyChanges() {
    if (m_pListeners == null) return;

    beforeApplyChangesToListener();

    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      applyChangesToListener(m_pListeners.get(i));
    }

    afterApplyChangesToListener();
  }

  protected abstract void applyChangesToListener(PanelListener pListener);

  //children can overwrite this to make some settings before the changes have been applied
  protected void beforeApplyChangesToListener() {

  }

  //children can overwrite this to make some settings after the changes have been applied
  protected void afterApplyChangesToListener() {

  }



}
