package gui;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.Box;

import java.util.Vector;


import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

//library imports
import applications.Settings;
import applications.ISettingsListener;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static applications.GuiLanguageProperties.*;

/***
 *  A panel that steers the size of an image effect.
 **/
public class SizingPanel extends JPanelWithListener<ISizingPanelListener> 
  implements ChangeListener, ActionListener, ISettingsListener {

  /*** constants ***/
  public static final int MINIMUM_FIXED_WIDTH = 10;
  public static final int MAXIMUM_FIXED_WIDTH = 10000;
  public static final int DEFAULT_FIXED_WIDTH = 400;
  
  public static final int MINIMUM_FIXED_HEIGHT = 10;
  public static final int MAXIMUM_FIXED_HEIGHT = 10000;
  public static final int DEFAULT_FIXED_HEIGHT = 400;
  
  public static final int RESIZING_MODE_PANELSIZE = 0;
  public static final int RESIZING_MODE_FIXEDSIZE = 1;
  
  
  /*** gui members ***/
  private JLabel m_pSizeModeLabel, m_pFixedWidthLabel, m_pFixedHeightLabel;
  private JComboBox<String> m_pSizeModes;
  private JSpinner m_pFixedWidthSpinner, m_pFixedHeightSpinner;
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *   Creates the {@code SizingPanel}. 
   **/
  public SizingPanel() {
  
   
    //1. set layout
    setLayout(new FlowLayout());
    
    //2. get the language settings
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    
    //3. size mode
    m_pSizeModeLabel = new JLabel();
    m_pSizeModes = new JComboBox<String>();
    m_pSizeModes.addActionListener(this);
    add(m_pSizeModeLabel);
    add(m_pSizeModes);
    
    
    //3. width
    m_pFixedWidthLabel = new JLabel(pLanguageProperties.getProperty(KEY_SIZING_PANEL_WIDTH));
    
    Integer pWidthValue = new Integer(DEFAULT_FIXED_WIDTH);
    Integer pWidthMin = new Integer(MINIMUM_FIXED_WIDTH);
    Integer pWidthMax = new Integer(MAXIMUM_FIXED_WIDTH);
    Integer pWidthStep = new Integer(1);
    SpinnerNumberModel pWidthSpinnerModel = new SpinnerNumberModel(pWidthValue, pWidthMin, pWidthMax, pWidthStep); 
    m_pFixedWidthSpinner = new JSpinner(pWidthSpinnerModel);
    m_pFixedWidthSpinner.addChangeListener(this);
    
    add(m_pFixedWidthLabel);
    add(m_pFixedWidthSpinner);
    
    
    //4. height
    m_pFixedHeightLabel = new JLabel();
    
    Integer pHeightValue = new Integer(DEFAULT_FIXED_HEIGHT);
    Integer pHeightMin = new Integer(MINIMUM_FIXED_HEIGHT);
    Integer pHeightMax = new Integer(MAXIMUM_FIXED_HEIGHT);
    Integer pHeightStep = new Integer(1);
    SpinnerNumberModel pHeightSpinnerModel = new SpinnerNumberModel(pHeightValue, pHeightMin, pHeightMax, pHeightStep); 
    m_pFixedHeightSpinner = new JSpinner(pHeightSpinnerModel);
    m_pFixedHeightSpinner.addChangeListener(this);
    
    add(m_pFixedHeightLabel);
    add(m_pFixedHeightSpinner);
    
    
    
    
    
    
    
    //4. create the fixed height line
    
    //5. set a border
    setBorder(new EmptyBorder(0, 5, 0, 5));
    
    setLabels(pLanguageProperties);
    
  
    
    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the width of the effect image in pixels.
   *
   *   @return The width of the effect image in pixels.
   **/
  public int getEffectWidth() {
    int iFixedWidth = ((Integer)m_pFixedWidthSpinner.getValue()).intValue();
    return iFixedWidth;
  }
  
  /**
   *   Returns the height of the effect image in pixels.
   *
   *   @return The height of the effect image in pixels.
   **/
  public int getEffectHeight() {
    return ((Integer)m_pFixedHeightSpinner.getValue()).intValue();
  }
  
  public int getSizeMode() {
    return m_pSizeModes.getSelectedIndex();
  }
  
  public boolean isPanelSizeMode() {
    return getSizeMode() == RESIZING_MODE_PANELSIZE;
  }
  
  public boolean isFixedSizeMode() {
    return getSizeMode() == RESIZING_MODE_FIXEDSIZE;
  }
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  protected void applyChangesToListener(ISizingPanelListener pListener) {
    pListener.applySizingChanges(this);
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - JSPINNER EVENTS  //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner}.
   *
   *  @param e The event triggered by the {@code JSpinner}.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    m_pSizeModes.setSelectedIndex(RESIZING_MODE_FIXEDSIZE); //set it to fixed size
    applyChanges();
  }
  
  
  ////////////////////////////////////////
  // ACTION LISTENER - JCOMBOBOX EVENTS //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JComboBox}.
   *
   *  @param e The event triggered by the {@code JComboBox}.
   *
   **/
  public void actionPerformed(ActionEvent e) {
    applyChanges();
  }
  
  
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent and the panel can adapt accordingly.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setLabels(pLanguageProperties);
  }
  
  private void setLabels(LanguageProperties pLanguageProperties) {
    
    //3. set the new labels of the combobox
    m_pFixedWidthLabel.setText(pLanguageProperties.getProperty(KEY_SIZING_PANEL_WIDTH));
    m_pFixedHeightLabel.setText(pLanguageProperties.getProperty(KEY_SIZING_PANEL_HEIGHT));
    m_pSizeModeLabel.setText(pLanguageProperties.getProperty(KEY_SIZING_PANEL_SIZE_MODE));
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add(KEY_SIZING_PANEL_SIZE_MODE_PANEL);
    pPropertyKeys.add(KEY_SIZING_PANEL_SIZE_MODE_FIXED);
    AbstractPropertiesPanel.switchComboBoxItemLabels(m_pSizeModes, pLanguageProperties, pPropertyKeys);
    
  }
  
 
}