package gui;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.JComponent;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import java.awt.FlowLayout;
import java.awt.Dimension;
import javax.swing.Box;

import java.util.Vector;


import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


/**
 *  A panel for steering a numeric effect parameter with a slider and a spinner as main gui components.
 **/
public abstract class SliderPanel extends JPanelWithListener<ISliderPanelListener> implements ChangeListener {

  
  /*** gui members ***/
  private JLabel m_pLabel, m_pMinLabel, m_pMaxLabel;
  private JSlider m_pSlider;
  private JSpinner m_pSpinner;
  private boolean m_bChangeListenerInProgress = false;
  private double m_dMin, m_dMax, m_dStepSize;
  
  ///////////////////////////////
  // PROTECTED METHODS         //
  ///////////////////////////////
  
  protected void initPanel(double dMin, double dMax, double dCurrentValue, double dStepSize) {
    //1. make some checks    
    if (dMin > dMax) {
      double dHelp = dMin;
      dMin = dMax;
      dMax = dHelp;
    }
    m_dMin = dMin;
    m_dMax = dMax;
    m_dStepSize = dStepSize;
    
    if (dCurrentValue < dMin) dCurrentValue = dMin;
    if (dCurrentValue > dMax) dCurrentValue = dMax;
    
    
    //2. construct the gui
    //2.1. set the layout and create a box
    setLayout(new FlowLayout(FlowLayout.LEFT));
    m_pLabel = new JLabel("");
    add(m_pLabel);
    
    //2.2. the min and max label and the slider
    m_pMinLabel = new JLabel(getMinMaxLabel(dMin));
    add(m_pMinLabel);
    
    m_pSlider = new JSlider(0, getSliderMax(), getSliderCurrentValue(dCurrentValue));
    m_pSlider.addChangeListener(this);
    add(m_pSlider);
    
    m_pMaxLabel = new JLabel(getMinMaxLabel(dMax));
    add(m_pMaxLabel);
    
    
    //2.3. createthe spinner and add it left and right in a border layout
    Double pDoubleValue = new Double(dCurrentValue);
    Double pDoubleMin = new Double(dMin);
    Double pDoubleMax = new Double(dMax);
    Double pDoubleStepSize = new Double(dStepSize);
    SpinnerNumberModel pSpinnerModel = new SpinnerNumberModel(pDoubleValue, pDoubleMin, pDoubleMax, pDoubleStepSize); 
    m_pSpinner = new JSpinner(pSpinnerModel);
    //set the width of the spinner correctly
    JComponent pEditor = m_pSpinner.getEditor();
    if (pEditor instanceof JSpinner.DefaultEditor) {
      JTextField pTextField = ((JSpinner.DefaultEditor)pEditor).getTextField();
      String pMaxValueString = getMinMaxLabel(m_dMax + m_dStepSize);
      pTextField.setColumns(pMaxValueString.length()+1);
    }
    m_pSpinner.addChangeListener(this);
    add(m_pSpinner);


    
  }
  
  protected abstract String getMinMaxLabel(double dValue);
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the integer value of the effect parameter that is represented by this {@code SliderPanel}.
   *  
   *  @param iValue The integer value of the effect parameter that should be used by this {@code SliderPanel}.
   **/
  public void setIntValue(int iValue) {
    //1. make some checks
    if (iValue < m_dMin) iValue = (int)Math.ceil(m_dMin);
    if (iValue > m_dMax) iValue = (int)Math.floor(m_dMax);
    
    //2. set the value but without reacting to the listeners
    m_bChangeListenerInProgress = true;
    m_pSlider.setValue(getSliderCurrentValue(iValue));
    m_pSpinner.setValue(new Double(iValue));
    m_bChangeListenerInProgress = false;
  }
  
  public void setLabelText(String pText) {
    if (pText == null) pText = "";
    m_pLabel.setText(pText.trim());
    
  }
  
  /**
   *  Sets the integer value of the effect parameter that is represented by this {@code SliderPanel}.
   *  
   *  @param iValue The integer value of the effect parameter that should be used by this {@code SliderPanel}.
   **/
  public void setDoubleValue(double dValue) {
    //1. make some checks
    if (dValue < m_dMin) dValue = m_dMin;
    if (dValue > m_dMax) dValue = m_dMax;
    
    //2. set the value but without reacting to the listeners
    m_bChangeListenerInProgress = true;
    m_pSlider.setValue(getSliderCurrentValue(dValue));
    m_pSpinner.setValue(new Double(dValue));
    m_bChangeListenerInProgress = false;
  }
  
  
  /**
   *  Sets the maximum value of the effect parameter that is represented by this {@code SliderPanel}.
   *  
   *  @param iMax The maximum value of the effect parameter.
   **/
   
  public void setMaximum(double dMax) {
    //1. make some checks
    if (dMax <= m_dMin) return; // it is not valid
    
    //2. adapt current value, if necessary
    double dCurrentValue = getDoubleValue();
    boolean bResetCurrentValue = false;
    if (dMax < dCurrentValue) {
      dCurrentValue = dMax;
      bResetCurrentValue = true;
    }
    
    
    //2. set the max values on the gui elements
    m_bChangeListenerInProgress = true;
    
    m_dMax = dMax;
    m_pSlider.setValue(getSliderCurrentValue(dCurrentValue));
    m_pSlider.setMaximum(getSliderMax());
    
    Double pDoubleValue = new Double(dCurrentValue);
    Double pDoubleMin = new Double(m_dMin);
    Double pDoubleMax = new Double(m_dMax);
    Double pDoubleStep = new Double(1);
    SpinnerNumberModel pSpinnerModel = new SpinnerNumberModel(pDoubleValue, pDoubleMin, pDoubleMax, pDoubleStep); 
    m_pSpinner.setModel(pSpinnerModel);
    
    m_pMaxLabel.setText(getMinMaxLabel(m_dMax));
    
    m_bChangeListenerInProgress = false;
    
    //3. if the current value has been adapted fire a change event
    if (bResetCurrentValue) {
      applyChanges();
    }
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the integer value of the effect parameter that is represented by this {@code SliderPanel}.
   *  
   *  @return The integer value of the effect parameter that is represented by this {@code SliderPanel}.
   **/
  public int getIntValue() {
    return (int)Math.round(getDoubleValue());
  }
  
  public double getDoubleValue() {
    return ((Double)m_pSpinner.getValue()).doubleValue();
  }
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
   protected void  applyChangesToListener(ISliderPanelListener pListener) {
    pListener.applyChanges(this, getIntValue(), getDoubleValue());
  }
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    //1. make a simple check
    if (m_bChangeListenerInProgress) return;
    
    //2. set the change listener progress flag
    m_bChangeListenerInProgress = true;
    
    //3. fetch the new parameter value
    Object pSource = e.getSource();
    double dCurrentValue = 0;
    if (pSource.equals(m_pSlider)) {
      dCurrentValue = getCurrentValueFromSlider(m_pSlider.getValue());
      m_pSpinner.setValue(new Double(dCurrentValue));
    }
    else {
      dCurrentValue = getDoubleValue();
      m_pSlider.setValue(getSliderCurrentValue(dCurrentValue));
    }
  
    
    
    //4. unset the change listener progress flag and fire the apply changes event
    m_bChangeListenerInProgress = false;
    applyChanges();

  }
  
  
  ///////////////////////////////
  // PRIVATE HELPER METHODS    //
  ///////////////////////////////
  private int getSliderMax() {
    return (int)((m_dMax - m_dMin)/m_dStepSize);
  }
  
  private int getSliderCurrentValue(double dCurrentValue) {
    return (int)((dCurrentValue - m_dMin)/m_dStepSize);
  }
  
  private double getCurrentValueFromSlider(int iSliderValue) {
    return iSliderValue*m_dStepSize + m_dMin;
  }
  
 
  
  
 
}