package gui;


/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JLabel;
import javax.swing.JComboBox;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Vector;

//library imports
import applications.Settings;
import applications.ISettingsListener;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static applications.GuiLanguageProperties.*;

/**
 *  Representing a panel with a zoom drop down box to steer the zoom. 
 *  A class that should react to those settings must implement the IZoomPanelListener interface.
 *
 */

public class ZoomPanel extends JPanelWithListener<IZoomPanelListener> 
  implements ActionListener, ISettingsListener {

  /*** member variables ***/
  private JLabel m_pZoomLabel;
  private JComboBox<String> m_pZoomComboBox;
  
  private int m_iOldZoomId;
  
  ///////////////////////////
  // CONSTRUCTOR           //
  ///////////////////////////
  /**
   *  Creates the {@code ZoomPanel}.
   **/
  public ZoomPanel() {
    //0. get the language settings 
    Settings pSettings = SettingsDialog.getSettings(); 
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    
    //1. set layout
    setLayout(new FlowLayout());
    
    //2. set the label
    m_pZoomLabel = new JLabel("Zoom:");
    add(m_pZoomLabel);
    
    //3. create the zoom combobox
    //TODO: put in one method
    Vector<String> pZoomValues = new Vector<String>();
    pZoomValues.add("5 %");
    pZoomValues.add("10 %");
    pZoomValues.add("25 %");
    pZoomValues.add("50 %");
    pZoomValues.add("100 %");
    pZoomValues.add("200 %");
    pZoomValues.add("400 %");
    pZoomValues.add("800 %");
    pZoomValues.add(pLanguageProperties.getProperty(KEY_ZOOM_PANEL_FIT_WIDTH));
    pZoomValues.add(pLanguageProperties.getProperty(KEY_ZOOM_PANEL_FIT_HEIGHT));
    m_pZoomComboBox = new JComboBox<String>(pZoomValues);
    m_pZoomComboBox.setSelectedItem("100 %");
    m_pZoomComboBox.addActionListener(this);
    add(m_pZoomComboBox);
    
    //4. listeners
    m_iOldZoomId = 4;
    
    
  }
  
  //////////////////////////////// 
  // SETTINGS LISTENER METHODS  // 
  //////////////////////////////// 
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent and the properties panel can adapt accordingly.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public void applyNewSettings(Settings pSettings) { 
    //1. make an initial check and set the settings 
    if (pSettings == null) return; 

    //2. load new language settings 
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    //3. set the new labels of the combobox 
    //TODO: put in one method
    int iSelectedId = m_pZoomComboBox.getSelectedIndex();
    m_pZoomComboBox.removeAllItems();
    m_pZoomComboBox.addItem("5 %");
    m_pZoomComboBox.addItem("10 %");
    m_pZoomComboBox.addItem("25 %");
    m_pZoomComboBox.addItem("50 %");
    m_pZoomComboBox.addItem("100 %");
    m_pZoomComboBox.addItem("200 %");
    m_pZoomComboBox.addItem("400 %");
    m_pZoomComboBox.addItem("800 %");
    m_pZoomComboBox.addItem(pLanguageProperties.getProperty(KEY_ZOOM_PANEL_FIT_WIDTH));
    m_pZoomComboBox.addItem(pLanguageProperties.getProperty(KEY_ZOOM_PANEL_FIT_HEIGHT));
    m_pZoomComboBox.setSelectedIndex(iSelectedId);
    
  } 
  
  
  
  //////////////////////////////////
  // ACTION LISTENER              //
  //////////////////////////////////
  /**
   *  Applies the changes of the {@code JComboBox} object.
   *
   *  @param e The event triggered by the {@code JComboBox} object.
   *
   **/
  public void actionPerformed(ActionEvent e) {
    applyChanges();
  }
  
  
  //////////////////////////////////
  // LISTENER METHODS             //
  //////////////////////////////////
  
  
  
  
  protected void applyChangesToListener(IZoomPanelListener pListener) {
    //1. fetch the new value and check if this is really a new value
    int iSelectedId = m_pZoomComboBox.getSelectedIndex();
    if (m_iOldZoomId == iSelectedId) return;
    
    //2. fire the event
    
    //2.1. fit width
    if (iSelectedId == 8) {
      pListener.setFitWidth();
    }
    
    
    //2.2. fit height
    else if (iSelectedId == 9) {
      pListener.setFitHeight();
    }
    
    
    //2.3. zoom factor
    else {
      try {
        int iZoomValue = 5;
        if (iSelectedId == 1) {
          iZoomValue = 10;
        }
        else if (iSelectedId == 2) {
          iZoomValue = 25;
        }
        else if (iSelectedId == 3) {
          iZoomValue = 50;
        }
        else if (iSelectedId == 4) {
          iZoomValue = 100;
        }
        else if (iSelectedId == 5) {
          iZoomValue = 200;
        }
        else if (iSelectedId == 6) {
          iZoomValue = 400;
        }
        else if (iSelectedId == 7) {
          iZoomValue = 800;
        }

        double dZoomValue = ((double)iZoomValue)/100;
        pListener.setZoomFactor(dZoomValue);
      }
      catch (Exception pException) {
        pException.printStackTrace();
      }
    }
    
    //3. set the current value
    m_iOldZoomId = iSelectedId;
  }
  

}