package gui.color;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import color.ColorTransitionParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.image.ImageLoadPanel;
import gui.image.IImageLoadPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static color.ColorTransitionParameters.*;
import static applications.ColorTransitionLanguageProperties.*;


/**
 *  A properties panel for the color transition effect.
 **/
public class ColorTransitionPropertiesPanel extends AbstractPropertiesPanel
  implements ISliderPanelListener, IColorDisplayListener, IImageLoadPanelListener, ActionListener {


  //TODO integrate this one into the image transition effect???
  /*** member variables ***/
  private ColorTransitionParameters m_pColorTransitionParameters;

  /*** gui member variables ***/
  private ImageLoadPanel m_pImageLoadPanel;
  private SliderPanel m_pLengthSliderPanel;
  private ColorDisplay m_pColorDisplay;
  private JPanel m_pCornerPanel;


  private JCheckBox m_pTopCheckBox, m_pBottomCheckBox;

  //TODO: merge this with the image transition effect!!!

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code ColorTransitionPropertiesPanel} object with the default parameters
   *  of the {@code ColorTransition} effect.
   **/
  public ColorTransitionPropertiesPanel() {
    //1. init members
    m_pColorTransitionParameters = new ColorTransitionParameters();



    //3. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addPanelListener(this);

    //3. color display
    m_pColorDisplay = new ColorDisplay();
    m_pColorDisplay.setColor(new Color(50, 50, 255));
    m_pColorDisplay.addPanelListener(this);


    //4. transition length
    m_pLengthSliderPanel = new IntegerSliderPanel(0, 10000, 50);
    m_pLengthSliderPanel.addPanelListener(this);

    m_pCornerPanel = new JPanel();
    m_pCornerPanel.setLayout(new FlowLayout());

    //TODO: translate the checkbox labels as well
    m_pTopCheckBox = new JCheckBox("Top: ", false);
    m_pTopCheckBox.addActionListener(this);
    m_pCornerPanel.add(m_pTopCheckBox);

    m_pBottomCheckBox = new JCheckBox("Bottom: ", true);
    m_pBottomCheckBox.addActionListener(this);
    m_pCornerPanel.add(m_pBottomCheckBox);

    initLayout();

  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code ColorTransitionParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code ColorTransitionParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pColorTransitionParameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pImageLoadPanel);
    pPanels.add(m_pColorDisplay);
    pPanels.add(m_pLengthSliderPanel);
    pPanels.add(m_pCornerPanel);




    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_IMAGE);
    pKeys.add(KEY_COLOR);
    pKeys.add(KEY_TRANSITION_LENGTH);
    pKeys.add(KEY_EDGES);

    return pKeys;
  }

  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code ColorTransitionPropertiesPanel} object.
   *  The set operation is only successful if a {@code ColorTransitionParameters} object has been specified.
   *
   *  @param pParameters The {@code ColorTransitionParameters} object representing the effect parameters
   *                     that should be represented by this {@code ColorTransitionPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof ColorTransitionParameters) {
      m_pColorTransitionParameters = (ColorTransitionParameters)pParameters;

      updateControls();
    }
  }



  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the color transition length has been changed in the slider panel for this parameter, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that steers the transition length parameter.
   *  @param iValue The length of the transition in pixels.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pColorTransitionParameters == null) m_pColorTransitionParameters = new ColorTransitionParameters();

    //2. set the new parameters
    if (pSliderPanel.equals(m_pLengthSliderPanel)) {
      m_pColorTransitionParameters.setIntegerParameterValue(PARAM_INT_TRANSITION_LENGTH, iValue);
    }



    //2. fire the apply changes event
    fireApplyChanges(m_pColorTransitionParameters);
  }



  /////////////////////////////////////////
  // ICOLOR DISPLAY LISTENER             //
  /////////////////////////////////////////
  /**
   *  If the color of the color transition is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that steers the color parameter of the transition.
   *  @param pColor The color that should be used for the transition.
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pColorTransitionParameters == null) m_pColorTransitionParameters = new ColorTransitionParameters();

    //1. set the new colors
    m_pColorTransitionParameters.setColorParameterValue(PARAM_COLOR_TRANSITION_COLOR, m_pColorDisplay.getColor());

    //2. fire the apply changes event
    fireApplyChanges(m_pColorTransitionParameters);
  }


  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image where the color transition should be painted is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image on which the transition is applied.
   *  @param pImage The image on which the color transition should be painted.
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pColorTransitionParameters == null) m_pColorTransitionParameters = new ColorTransitionParameters();

    //1. set the new colors
    m_pColorTransitionParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);

    //2. fire the apply changes event
    fireApplyChanges(m_pColorTransitionParameters);
  }

  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  This should only be used to modify the parameter panel that is currently shown. But at the moment it is also used
   *  to communicate if the top or the bottom checkbox have been modified. In this case
   *  the effect parameters are adapted and the listeners are notified. (TO DO: In future those checkboxes should be in a separate panel.)
   *
   *  @param pEvent The action event triggering this method.
   **/
  public void actionPerformedForChildren(ActionEvent pEvent) {

    //1. property selection
    Object pSource = pEvent.getSource();

    if (pSource.equals(m_pTopCheckBox)) {
      m_pColorTransitionParameters.setBooleanParameterValue(PARAM_BOOL_TRANSITION_TOP, m_pTopCheckBox.isSelected());
      fireApplyChanges(m_pColorTransitionParameters);
      return;
    }

    if (pSource.equals(m_pBottomCheckBox)) {
      m_pColorTransitionParameters.setBooleanParameterValue(PARAM_BOOL_TRANSITION_BOTTOM, m_pBottomCheckBox.isSelected());
      fireApplyChanges(m_pColorTransitionParameters);
      return;
    }



  }




  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;


    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }


  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pColorTransitionParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    //m_pBrickColorDisplay.setColor(m_pColorTransitionParameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }



}
