package gui.color;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import color.NoiseGeneratorParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.image.ImageLoadPanel;
import gui.image.IImageLoadPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static color.NoiseGeneratorParameters.*;
import static applications.NoiseGeneratorLanguageProperties.*;


/**
 *  A properties panel for the noise generator effect.
 **/
public class NoiseGeneratorPropertiesPanel extends AbstractPropertiesPanel
  implements ISliderPanelListener, IImageLoadPanelListener, ChangeListener {

  /*** member variables ***/
  private NoiseGeneratorParameters m_pNoiseGeneratorParameters;

  /*** gui member variables ***/
  private ImageLoadPanel m_pImageLoadPanel;
  private SliderPanel m_pSaltAndPepperIntensitySliderPanel, m_pGaussianIntensitySliderPanel;
  private SliderPanel m_pGaussianStandardDeviationSliderPanel, m_pRandomColorIntensitySliderPanel;
  private JPanel m_pNoiseTypesPanel;
  private JCheckBox m_pUseSaltAndPepperNoise, m_pUseGaussianNoise, m_pUseRandomColorNoise;

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code NoiseGeneratorPropertiesPanel} object with the default parameters
   *  of the {@code NoiseGenerator} effect.
   **/
  public NoiseGeneratorPropertiesPanel() {
    //1. init members
    m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();


    //3. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addPanelListener(this);

    //4. noise types
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    m_pNoiseTypesPanel = createNoiseTypesPanel(pLanguageProperties);


    //5. salt and pepper intensity
    m_pSaltAndPepperIntensitySliderPanel = new IntegerSliderPanel(0, 100, 50);
    m_pSaltAndPepperIntensitySliderPanel.addPanelListener(this);

    //6. gaussian intensity
    m_pGaussianIntensitySliderPanel = new IntegerSliderPanel(0, 100, 50);
    m_pGaussianIntensitySliderPanel.addPanelListener(this);

    //7. gaussian standard deviation
    m_pGaussianStandardDeviationSliderPanel = new IntegerSliderPanel(0, 255, 128);
    m_pGaussianStandardDeviationSliderPanel.addPanelListener(this);

    //8. random color intensity
    m_pRandomColorIntensitySliderPanel = new IntegerSliderPanel(0, 100, 50);
    m_pRandomColorIntensitySliderPanel.addPanelListener(this);



    initLayout();
  }

  //////////////////////////////////////
  // PRIVATE PANEL GENERATION METHODS //
  //////////////////////////////////////
  private JPanel createNoiseTypesPanel(LanguageProperties pLanguageProperties) {
    //1. create the panel and set the layout
    JPanel pPanel = new JPanel();
    pPanel.setLayout(new FlowLayout());

    //2. add the checkboxes
    //2.1. salt & pepper noise
    m_pUseSaltAndPepperNoise = new JCheckBox(pLanguageProperties.getProperty(KEY_USE_SALT_PEPPER));
    m_pUseSaltAndPepperNoise.setSelected(true);
    m_pUseSaltAndPepperNoise.addChangeListener(this);
    pPanel.add(m_pUseSaltAndPepperNoise);

    //2.2. gaussian noise
    m_pUseGaussianNoise = new JCheckBox(pLanguageProperties.getProperty(KEY_USE_GAUSS));
    m_pUseGaussianNoise.addChangeListener(this);
    pPanel.add(m_pUseGaussianNoise);

    //2.3. random color noise
    m_pUseRandomColorNoise = new JCheckBox(pLanguageProperties.getProperty(KEY_USE_RAND_COL));
    m_pUseRandomColorNoise.addChangeListener(this);
    pPanel.add(m_pUseRandomColorNoise);


    return pPanel;
  }

  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code NoiseGeneratorParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code NoiseGeneratorParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pNoiseGeneratorParameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pImageLoadPanel);
    pPanels.add(m_pNoiseTypesPanel);
    pPanels.add(m_pSaltAndPepperIntensitySliderPanel);
    pPanels.add(m_pGaussianIntensitySliderPanel);
    pPanels.add(m_pGaussianStandardDeviationSliderPanel);
    pPanels.add(m_pRandomColorIntensitySliderPanel);


    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_IMAGE);
    pKeys.add(KEY_NOISE_TYPES);
    pKeys.add(KEY_SALT_PEPPER_INTENSITY);
    pKeys.add(KEY_GAUSS_INTENSITY);
    pKeys.add(KEY_GAUSS_STDDEV);
    pKeys.add(KEY_RAND_COL_INTENSITY);

    return pKeys;
  }



  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code NoiseGeneratorPropertiesPanel} object.
   *  The set operation is only successful if a {@code NoiseGeneratorParameters} object has been specified.
   *
   *  @param pParameters The {@code NoiseGeneratorParameters} object representing the effect parameters
   *                     that should be represented by this {@code NoiseGeneratorPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof NoiseGeneratorParameters) {
      m_pNoiseGeneratorParameters = (NoiseGeneratorParameters)pParameters;

      updateControls();
    }
  }



  ////////////////////////////////////////
  // SLIDER PANEL LISTENER              //
  ////////////////////////////////////////
  /**
   *  If the intensities of the noises have been adapted in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();

    //TODO: switch Sliders to Double???
    //2. set the new parameters
    if (pSliderPanel.equals(m_pSaltAndPepperIntensitySliderPanel)) {
      //double dValue = (double)iValue/100;
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_SALT_AND_PEPPER_INTENSITY, dValue/100);
    }

    if (pSliderPanel.equals(m_pGaussianIntensitySliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_INTENSITY, dValue/100);
    }

    if (pSliderPanel.equals(m_pGaussianStandardDeviationSliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_STANDARD_DEVIATION, dValue);
    }

    if (pSliderPanel.equals(m_pRandomColorIntensitySliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_RANDOM_COLOR_INTENSITY, dValue/100);
    }



    //2. fire the apply changes event
    fireApplyChanges(m_pNoiseGeneratorParameters);
  }






  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image to which the effect should be applied is changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image to which the effect is applied.
   *  @param pImage The image to which the effect should be applied.
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();

    //1. set the new colors
    m_pNoiseGeneratorParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);

    //2. fire the apply changes event
    fireApplyChanges(m_pNoiseGeneratorParameters);
  }



  /////////////////////////////////////////
  // CHANGE LISTENER                     //
  /////////////////////////////////////////
  /**
   *  If the checkboxes have been modified that steer which noise type is used, this is communicated with this method.
   *  The effect parameters are adapted and the listeners are notified.
   *
   *  @param e The action event triggering this method.
   **/
  public void stateChanged(ChangeEvent e) {
    //0. make a simple check
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();

    //1. set the checkbox flags
    boolean bRepaint = false;
    if (m_pUseSaltAndPepperNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE, m_pUseSaltAndPepperNoise.isSelected());
      bRepaint = true;
    }

    if (m_pUseGaussianNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE, m_pUseGaussianNoise.isSelected());
      bRepaint = true;
    }

    if (m_pUseRandomColorNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE, m_pUseRandomColorNoise.isSelected());
      bRepaint = true;
    }

    //2. fire the apply changes event
    if (bRepaint) fireApplyChanges(m_pNoiseGeneratorParameters);
  }



  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();


    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }

    //5. check boxes
    m_pUseSaltAndPepperNoise.setText(pLanguageProperties.getProperty(KEY_USE_SALT_PEPPER));
    m_pUseGaussianNoise.setText(pLanguageProperties.getProperty(KEY_USE_GAUSS));
    m_pUseRandomColorNoise.setText(pLanguageProperties.getProperty(KEY_USE_RAND_COL));


  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pNoiseGeneratorParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    //m_pBrickColorDisplay.setColor(m_pNoiseGeneratorParameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }




}
