package gui.drawing;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import drawing.BrickTextureParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.ComboBoxPanel;
import gui.IComboBoxPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static drawing.BrickTextureParameters.*;
import static applications.BrickTextureLanguageProperties.*;


/**
 *  A properties panel for the brick texture effect.
 **/
public class BrickTexturePropertiesPanel extends AbstractPropertiesPanel
  implements ISliderPanelListener, IColorDisplayListener, IComboBoxPanelListener {

  /*** member variables ***/
  private BrickTextureParameters m_pBrickTexturePrameters;

  /*** gui member variables ***/
  private SliderPanel m_pBrickColorRandomSliderPanel, m_pBrickWidthSliderPanel, m_pBrickHeightSliderPanel;
  private ColorDisplay m_pBrickColorDisplay, m_pBrickBorderColorDisplay;
  private ComboBoxPanel m_pRandomModes, m_pBevelModes;
  private JCheckBox m_pDrawBrickBorder;

  private JPanel m_pBrickBorderPanel;


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code BrickTexturePropertiesPanel} object with the default parameters
   *  of the {@code BrickTexture} effect.
   **/
  public BrickTexturePropertiesPanel() {
    //1. init members
    m_pBrickTexturePrameters = null;


    //TODO: Sizing Panel is missing!!!
    //TODO: Panel generation should be in separate methods
    //3. create the panel
    //3.1. brick color
    m_pBrickColorDisplay = new ColorDisplay();
    m_pBrickColorDisplay.setColor(Color.CYAN);
    m_pBrickColorDisplay.addPanelListener(this);

    //3.2. randomize magnitude
    m_pBrickColorRandomSliderPanel = new IntegerSliderPanel(0, 500, 150);
    m_pBrickColorRandomSliderPanel.addPanelListener(this);

    //3.3. color random mode - TODO: bundle this into one method
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    Vector<String> pRandomModeValues = new Vector<String>();
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RGB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_R));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_G));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_B));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RG));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_GB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HSB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_H));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_S));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_BR));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HS));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_SB));
    m_pRandomModes = new ComboBoxPanel("", pRandomModeValues, 0);
    m_pRandomModes.addPanelListener(this);

    //3.4. bevel mode
    Vector<String> pBevelModeValues = new Vector<String>();
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_NONE));
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_RAISED));
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_SUNK));
    m_pBevelModes = new ComboBoxPanel("", pBevelModeValues, 1);
    m_pBevelModes.addPanelListener(this);


    //3.5. brick border color
    m_pBrickBorderPanel = new JPanel();
    m_pBrickBorderPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));

    m_pDrawBrickBorder = new JCheckBox(pLanguageProperties.getProperty(KEY_DRAW_BORDER));
    m_pDrawBrickBorder.setSelected(false);
    m_pDrawBrickBorder.addActionListener(this);
    m_pBrickBorderPanel.add(m_pDrawBrickBorder);

    m_pBrickBorderColorDisplay = new ColorDisplay();
    m_pBrickBorderColorDisplay.setColor(Color.BLACK);
    m_pBrickBorderColorDisplay.addPanelListener(this);
    m_pBrickBorderPanel.add(m_pBrickBorderColorDisplay);

    //3.6. brick width
    m_pBrickWidthSliderPanel = new IntegerSliderPanel(0, 500, 20);
    m_pBrickWidthSliderPanel.addPanelListener(this);

    //3.7. brick height
    m_pBrickHeightSliderPanel = new IntegerSliderPanel(0, 500, 10);
    m_pBrickHeightSliderPanel.addPanelListener(this);





    initLayout();

  }




  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code BrickTextureParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code BrickTextureParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pBrickTexturePrameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pBrickColorDisplay);
    pPanels.add(m_pBrickColorRandomSliderPanel);
    pPanels.add(m_pRandomModes);
    pPanels.add(m_pBevelModes);
    pPanels.add(m_pBrickBorderPanel);
    pPanels.add(m_pBrickWidthSliderPanel);
    pPanels.add(m_pBrickHeightSliderPanel);


    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();

    pKeys.add(KEY_BRICK_COLOR);
    pKeys.add(KEY_BRICK_COL_RAND_MAGN);
    pKeys.add(KEY_RANDOM_MODE);
    pKeys.add(KEY_BEVEL_MODE);
    pKeys.add(KEY_BRICK_BORDER_COLOR);
    pKeys.add(KEY_BRICK_WIDTH);
    pKeys.add(KEY_BRICK_HEIGHT);

    return pKeys;
  }


  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code BrickTexturePropertiesPanel} object.
   *  The set operation is only successful if a {@code BrickTextureParameters} object has been specified.
   *
   *  @param pParameters The {@code BrickTextureParameters} object representing the effect parameters
   *                     that should be represented by this {@code BrickTexturePropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof BrickTextureParameters) {
      m_pBrickTexturePrameters = (BrickTextureParameters)pParameters;

      updateControls();
    }
  }



  ////////////////////////////////////////
  // SLIDER PANEL LISTENER              //
  ////////////////////////////////////////
  /**
   *  If the brick dimensions or the randomize magnitude have been adapted in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();

    //2. set the new parameters
    if (pSliderPanel.equals(m_pBrickColorRandomSliderPanel)) {
      m_pBrickTexturePrameters.setDoubleParameterValue(PARAM_DOUBLE_RANDOMIZE_MAGNITUDE, dValue);
    }

    if (pSliderPanel.equals(m_pBrickWidthSliderPanel)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BRICK_WIDTH, iValue);
    }

    if (pSliderPanel.equals(m_pBrickHeightSliderPanel)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BRICK_HEIGHT, iValue);
    }

    //2. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }

  ////////////////////////////////////////
  // COMBOBOX PANEL LISTENER            //
  ////////////////////////////////////////
  /**
   *  If bevel or the randomize mode have been adapted in the corresponding combobox panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pComboBoxPanel The {@code ComboBoxPanel} object that has been modified.
   *  @param iSelectedIndex The selected index of the combo box that has been modified.
   **/
  public void applyChanges(ComboBoxPanel pComboBoxPanel, int iSelectedIndex) {

    //1. make some simple checks
    if (pComboBoxPanel == null) return;
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();

    //2. set the new parameters
    if (pComboBoxPanel.equals(m_pBevelModes)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_BEVEL_MODE, iSelectedIndex);
    }

    if (pComboBoxPanel.equals(m_pRandomModes)) {
      m_pBrickTexturePrameters.setIntegerParameterValue(PARAM_INT_RANDOM_MODE, iSelectedIndex);
    }



    //3. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }


  /////////////////////////////////////////
  // ICOLOR DISPLAY LISTENER             //
  /////////////////////////////////////////
  /**
   *  If the color of the brick or the brick border has been changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that has been changed.
   *  @param pColor The color that has been set for the {@code ColorDisplay} object.
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();

    //1. set the new colors
    m_pBrickTexturePrameters.setColorParameterValue(PARAM_COLOR_BRICK, m_pBrickColorDisplay.getColor());
    m_pBrickTexturePrameters.setColorParameterValue(PARAM_COLOR_BORDER, m_pBrickBorderColorDisplay.getColor());

    //2. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }

  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  Detects which parameter should be shown now in the properties panel and if the brick border
   *  should be drawn.
   *
   *  @param pEvent The action event triggering this method.
   **/
  public void actionPerformedForChildren(ActionEvent pEvent) {


    //1. draw brick border checkbox
    Object pSource = pEvent.getSource();
    if (pSource.equals(m_pDrawBrickBorder)) {
      drawBrickBorderActionPerformed();
      return;
    }



  }



  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();


    //4. other gui elements
    m_pDrawBrickBorder.setText(pLanguageProperties.getProperty(KEY_DRAW_BORDER));

    Vector<String> pBevelModeValues = new Vector<String>();
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_NONE));
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_RAISED));
    pBevelModeValues.add(pLanguageProperties.getProperty(KEY_BEVEL_MODE_SUNK));
    m_pBevelModes.setValueLabels(pBevelModeValues);


    Vector<String> pRandomModeValues = new Vector<String>();
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RGB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_R));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_G));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_B));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RG));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_RB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_GB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HSB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_H));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_S));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_BR));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HS));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_HB));
    pRandomModeValues.add(pLanguageProperties.getProperty(KEY_RANDOM_MODE_SB));
    m_pRandomModes.setValueLabels(pRandomModeValues);



  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pBrickTexturePrameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    m_pBrickColorDisplay.setColor(m_pBrickTexturePrameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }

  private void drawBrickBorderActionPerformed() {
    //1. make a simple check
    if (m_pBrickTexturePrameters == null) m_pBrickTexturePrameters = new BrickTextureParameters();

    //2. draw border
    boolean bDrawBrickBorder = m_pDrawBrickBorder.isSelected();
    m_pBrickTexturePrameters.setBooleanParameterValue(PARAM_BOOLEAN_DRAW_BORDER, bDrawBrickBorder);

    //3. fire the apply changes event
    fireApplyChanges(m_pBrickTexturePrameters);
  }



}
