package gui.drawing;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.util.Vector;

//Library imports
import standard.EffectParameters;
import drawing.NetworkParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.SizingPanel;
import gui.ISizingPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import gui.color.TransparentColorPanel;
import gui.color.ITransparentColorPanelListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static drawing.NetworkParameters.*;
import static applications.NetworkLanguageProperties.*;


/**
 *  A properties panel for the networks effect.
 **/
public class NetworkPropertiesPanel extends AbstractPropertiesPanel
  implements  ISliderPanelListener, IColorDisplayListener, ActionListener,
              ITransparentColorPanelListener, ISizingPanelListener {

  /*** member variables ***/
  private NetworkParameters m_pNetworkParameters;

  /*** gui member variables ***/
  private SliderPanel m_pPointNumSliderPanel, m_pMaxConnectionNumSliderPanel, m_pPointSizeSliderPanel;
  private SliderPanel m_pPointIntendSliderPanel;
  private ColorDisplay m_pBackgroundColorDisplay;
  private ColorDisplay m_pPointColorDisplay1, m_pPointColorDisplay2, m_pPointColorDisplay3, m_pPointColorDisplay4;
  private TransparentColorPanel m_pLineColorPanel;
  private SizingPanel m_pSizingPanel;

  private JPanel m_pPointColorPanel;


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code NetworkPropertiesPanel} object with the default parameters
   *  of the {@code Network} effect.
   **/
  public NetworkPropertiesPanel() {
    //1. init members
    m_pNetworkParameters = null;




    //3. create the panel for the point number
    m_pPointNumSliderPanel = new IntegerSliderPanel(2, 10000, 200);
    m_pPointNumSliderPanel.addPanelListener(this);

    //4. create the panel for the maximum number of connections
    m_pMaxConnectionNumSliderPanel = new IntegerSliderPanel(1, 1000, 10);
    m_pMaxConnectionNumSliderPanel.addPanelListener(this);

    //4. create the panel for the maximum number of connections
    m_pPointSizeSliderPanel = new IntegerSliderPanel(1, 100, 30);
    m_pPointSizeSliderPanel.addPanelListener(this);

    //6. create the panel for the maximum number of connections
    m_pBackgroundColorDisplay = new ColorDisplay();
    m_pBackgroundColorDisplay.setColor(Color.WHITE);
    m_pBackgroundColorDisplay.addPanelListener(this);

    //7. point color panel
    m_pPointColorPanel = new JPanel();
    m_pPointColorPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));

    m_pPointColorDisplay1 = new ColorDisplay();
    m_pPointColorDisplay1.setColor(Color.BLACK);
    m_pPointColorDisplay1.addPanelListener(this);
    m_pPointColorPanel.add(m_pPointColorDisplay1);

    m_pPointColorDisplay2 = new ColorDisplay();
    m_pPointColorDisplay2.setColor(Color.BLUE);
    m_pPointColorDisplay2.addPanelListener(this);
    m_pPointColorPanel.add(m_pPointColorDisplay2);

    m_pPointColorDisplay3 = new ColorDisplay();
    m_pPointColorDisplay3.setColor(new Color(170, 0, 170));
    m_pPointColorDisplay3.addPanelListener(this);
    m_pPointColorPanel.add(m_pPointColorDisplay3);

    m_pPointColorDisplay4 = new ColorDisplay();
    m_pPointColorDisplay4.setColor(Color.RED);
    m_pPointColorDisplay4.addPanelListener(this);
    m_pPointColorPanel.add(m_pPointColorDisplay4);

    //8. line color panel
    m_pLineColorPanel = new TransparentColorPanel();
    m_pLineColorPanel.setColor(new Color(0.0f, 0.0f, 0.0f, 0.1f));
    m_pLineColorPanel.addPanelListener(this);

    //9. sizing panel
    m_pSizingPanel = new SizingPanel();
    m_pSizingPanel.addPanelListener(this);

    //10. point intend panel
    m_pPointIntendSliderPanel = new IntegerSliderPanel(0, 100, 10);
    m_pPointIntendSliderPanel.addPanelListener(this);


    initLayout();


  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code NetworkParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code NetworkParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pNetworkParameters;
  }

  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pPointNumSliderPanel);
    pPanels.add(m_pMaxConnectionNumSliderPanel);
    pPanels.add(m_pPointSizeSliderPanel);
    pPanels.add(m_pBackgroundColorDisplay);
    pPanels.add(m_pPointColorPanel);
    pPanels.add(m_pLineColorPanel);
    pPanels.add(m_pSizingPanel);
    pPanels.add(m_pPointIntendSliderPanel);


    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_POINT_NUM);
    pKeys.add(KEY_CONNECTION_NUM);
    pKeys.add(KEY_POINT_SIZE);
    pKeys.add(KEY_BACKGROUND_COLOR);
    pKeys.add(KEY_POINT_COLOR);
    pKeys.add(KEY_LINE_COLOR);
    pKeys.add(KEY_SIZE);
    pKeys.add(KEY_POINT_INTEND);

    return pKeys;
  }

  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code NetworkPropertiesPanel} object.
   *  The set operation is only successful if a {@code NetworkParameters} object has been specified.
   *
   *  @param pParameters The {@code NetworkParameters} object representing the effect parameters
   *                     that should be represented by this {@code NetworkPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof NetworkParameters) {
      m_pNetworkParameters = (NetworkParameters)pParameters;

      updateControls();
    }
  }



  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the number of points or connections or the maximum point drawing size or the point intend
   *  have been adapted in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pNetworkParameters == null) m_pNetworkParameters = new NetworkParameters();

    //1. set the new parameters
    if (pSliderPanel.equals(m_pPointNumSliderPanel)) {
      m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_POINT_NUM, iValue);
    }

    if (pSliderPanel.equals(m_pMaxConnectionNumSliderPanel)) {
      m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_MAX_CONNECTION_NUM, iValue);
    }

    if (pSliderPanel.equals(m_pPointSizeSliderPanel)) {
      m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_MAX_POINT_DRAWING_SIZE, iValue);
    }

    if (pSliderPanel.equals(m_pPointIntendSliderPanel)) {
      m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_POINT_INTEND, iValue);
    }

    //2. fire the apply changes event
    fireApplyChanges(m_pNetworkParameters);
  }


  /////////////////////////////////////////
  // ICOLOR PANEL LISTENER               //
  /////////////////////////////////////////
  /**
   *  If the color of the background or the points has been changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that has been changed.
   *  @param pColor The color that has been set for the {@code ColorDisplay} object.
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pNetworkParameters == null) m_pNetworkParameters = new NetworkParameters();

    //1. set the new color
    if (pColorDisplay.equals(m_pBackgroundColorDisplay)) {
      m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_BACKGROUND_COLOR, pColor);
    }
    else if (pColorDisplay.equals(m_pPointColorDisplay1)) {
      m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_POINT_COLOR1, pColor);
    }
    else if (pColorDisplay.equals(m_pPointColorDisplay2)) {
      m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_POINT_COLOR2, pColor);
    }
    else if (pColorDisplay.equals(m_pPointColorDisplay3)) {
      m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_POINT_COLOR3, pColor);
    }
    else if (pColorDisplay.equals(m_pPointColorDisplay4)) {
      m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_POINT_COLOR4, pColor);
    }

    //2. fire the apply changes event
    fireApplyChanges(m_pNetworkParameters);
  }

  /////////////////////////////////////////
  // ITRANSPARENT COLOR PANEL LISTENER   //
  /////////////////////////////////////////
  /**
   *  If the transparent color of the connections has been changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pTransparentColorPanel The {@code TransparentColorPanel} object that has been changed.
   *  @param pColor The color that has been set for the {@code TransparentColorPanel} object.
   **/
  public void applyNewColor(TransparentColorPanel pTransparentColorPanel, Color pColor) {
    //0. make a simple check
    if (m_pNetworkParameters == null) m_pNetworkParameters = new NetworkParameters();

    //1. set the new color
    m_pNetworkParameters.setColorParameterValue(PARAM_COLOR_LINE_COLOR, pColor);

    //2. fire the apply changes event
    fireApplyChanges(m_pNetworkParameters);
  }

  /////////////////////////////////
  // SIZING PANEL LISTENER       //
  /////////////////////////////////
   /**
   *  If the image size has been changed, this is communicated with this method,
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSizingPanel The {@code SizingPanel} object that has been changed.
   **/
  public void applySizingChanges(SizingPanel pSizingPanel) {
    //1. make a simple check
    if (m_pNetworkParameters == null) m_pNetworkParameters = new NetworkParameters();

    //2. set the new sizes
    m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_WIDTH, pSizingPanel.getEffectWidth());
    m_pNetworkParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, pSizingPanel.getEffectHeight());

    //TODO: Panel Size is not supported yet!!!

    //3. apply the changes
    fireApplyChanges(m_pNetworkParameters);
  }





  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    //4. sizing panel
    if (m_pSizingPanel != null) {
      m_pSizingPanel.applyNewSettings(pSettings);
    }

  }



  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pNetworkParameters == null) return;

    //2. TO DO: update all controls according to the settings
    //   in the effect parameters and make sure that no action is fired!
    //m_pLineColorDisplay.setColor(m_pNetworkParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR));
  }






}
