package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

//Library imports
import standard.EffectParameters;
import fractals.FractalSetParameters;
import gui.JPanelWithListener;
import gui.AbstractPropertiesPanel;
import gui.AbstractImageEffectPanel;
import gui.IImageEffectPanelListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;

import util.math.ComplexNumber;


import applications.Settings;
import applications.SettingsDialog;
import applications.ISettingsListener;
import applications.LanguageProperties;

import static applications.FractalSetLanguageProperties.*;

/**
 *  A properties panel for the mandelbrot effect.
 **/
public class FractalSetButtonNavigationPanel
  extends JPanelWithListener<IFractalSetButtonNavigationPanelListener>
  implements ISettingsListener, ActionListener
   {


  /*** gui member variables ***/
  private JLabel m_pStepSizeLabel;
  private JButton m_pMoveDown, m_pMoveLeft, m_pMoveRight, m_pMoveUp;
  private boolean m_bMoveDown, m_bMoveUp, m_bMoveRight, m_bMoveLeft;
  private JSpinner m_pStepSize;


  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  public FractalSetButtonNavigationPanel() {
    createNavigationPanel();

    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    setLabels(pLanguageProperties);
  }




  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////


  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////


  protected void applyChangesToListener(IFractalSetButtonNavigationPanelListener pListener) {
    double dStepSize = ((Double)m_pStepSize.getValue()).doubleValue();
    if (m_bMoveLeft) {
      pListener.moveLeft(this, dStepSize);
    }
    else if (m_bMoveRight) {
      pListener.moveRight(this, dStepSize);
    }
    else if (m_bMoveUp) {
      pListener.moveUp(this, dStepSize);
    }
    else if (m_bMoveDown) {
      pListener.moveDown(this, dStepSize);
    }




  }

  protected void afterApplyChangesToListener() {
    m_bMoveLeft = false;
    m_bMoveRight = false;
    m_bMoveUp = false;
    m_bMoveDown = false;
  }






  //////////////////////////////////////////
  // ACTION LISTENER - BUTTON EVENTS      //
  //////////////////////////////////////////
  /**
   *   Applies a reset of the Mandelbrot set effect to the default setting respectively detects
   *   which parameter should be shown now in the properties panel.
   *
   *   @param e The action event triggering this method.
   **/
  public void actionPerformed(ActionEvent e) {

    //1. get source
    Object pSource = e.getSource();
    if (pSource == null) return;

    if (pSource.equals(m_pMoveDown)) {
      m_bMoveDown = true;
    }
    else if (pSource.equals(m_pMoveUp)) {
      m_bMoveUp = true;
    }
    else if (pSource.equals(m_pMoveLeft)) {
      m_bMoveLeft = true;
    }
    else if (pSource.equals(m_pMoveRight)) {
      m_bMoveRight = true;
    }

    applyChanges();




  }









  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettings(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setLabels(pLanguageProperties);


  }


  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////





  private void createNavigationPanel() {
    setLayout(new FlowLayout());

    //move buttons
    m_pMoveLeft = new JButton();
    m_pMoveLeft.addActionListener(this);
    add(m_pMoveLeft);

    m_pMoveRight = new JButton();
    m_pMoveRight.addActionListener(this);
    add(m_pMoveRight);

    m_pMoveUp = new JButton();
    m_pMoveUp.addActionListener(this);
    add(m_pMoveUp);

    m_pMoveDown = new JButton();
    m_pMoveDown.addActionListener(this);
    add(m_pMoveDown);

    //step size
    m_pStepSizeLabel = new JLabel();
    add(m_pStepSizeLabel);

    double dMin = 0.001;
    double dMax = 2;
    double dStep = 0.001;
    SpinnerNumberModel pStepSizeModel = new SpinnerNumberModel(0.5, dMin, dMax, dStep);
    m_pStepSize = new JSpinner(pStepSizeModel);
    add(m_pStepSize);
  }


  private void setLabels(LanguageProperties pLanguageProperties) {
    m_pMoveLeft.setText(pLanguageProperties.getProperty(KEY_MOVE_LEFT));
    m_pMoveRight.setText(pLanguageProperties.getProperty(KEY_MOVE_RIGHT));
    m_pMoveUp.setText(pLanguageProperties.getProperty(KEY_MOVE_UP));
    m_pMoveDown.setText(pLanguageProperties.getProperty(KEY_MOVE_DOWN));
    m_pStepSizeLabel.setText(pLanguageProperties.getProperty(KEY_MOVE_STEP_SIZE));
  }
}
