package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

//Library imports
import standard.EffectParameters;
import fractals.FractalSetParameters;
import gui.JPanelWithListener;
import gui.AbstractPropertiesPanel;
import gui.AbstractImageEffectPanel;
import gui.IImageEffectPanelListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;

import util.math.ComplexNumber;


import applications.Settings;
import applications.SettingsDialog;
import applications.ISettingsListener;
import applications.LanguageProperties;

import static applications.FractalSetLanguageProperties.*;

/**
 *  A properties panel for the mandelbrot effect.
 **/
public class FractalSetNavigationPanel
  extends JPanelWithListener<IFractalSetNavigationPanelListener>
  implements ISettingsListener, ChangeListener, ActionListener
   {


  /*** gui member variables ***/
  private JButton m_pResetFractalSet;
  private JSpinner m_pLeftUpperX, m_pLeftUpperY, m_pRightLowerX, m_pRightLowerY, m_pStepSize;
  private boolean m_bChangeListenerInAction, m_bApplyReset, m_bApplyCornerChange;
  private JLabel m_pStepSizeLabel, m_pLeftUpperLabel, m_pRightLowerLabel;


  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  public FractalSetNavigationPanel() {
    createNavigationPanel();

    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    setLabels(pLanguageProperties);
  }




  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the effect parameters
   *                     that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   **/
  public void setWindowCorners(double dLeftUpperX, double dLeftUpperY, double dRightLowerX, double dRightLowerY) {

      m_bChangeListenerInAction = true;

      m_pLeftUpperX.setValue(dLeftUpperX);
      m_pLeftUpperY.setValue(dLeftUpperY);

      m_pRightLowerX.setValue(dRightLowerX);
      m_pRightLowerY.setValue(dRightLowerY);

      m_bChangeListenerInAction = false;
  }


  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  protected void beforeChangesToListenerFinished() {
    m_bChangeListenerInAction = true;
  }

  protected void applyChangesToListener(IFractalSetNavigationPanelListener pListener) {
    if (m_bApplyReset) {
      pListener.applyResetFractalSet(this);
    }

    if (m_bApplyCornerChange) {
      pListener.applyCornerChange(this,
        ((Double)m_pLeftUpperX.getValue()).doubleValue(),
        ((Double)m_pLeftUpperY.getValue()).doubleValue(),
        ((Double)m_pRightLowerX.getValue()).doubleValue(),
        ((Double)m_pRightLowerY.getValue()).doubleValue());

    }


  }

  protected void afterApplyChangesToListener() {
    m_bApplyReset = false;
    m_bApplyCornerChange = false;
    m_bChangeListenerInAction = false;
  }






  //////////////////////////////////////////
  // ACTION LISTENER - BUTTON EVENTS      //
  //////////////////////////////////////////
  /**
   *   Applies a reset of the Mandelbrot set effect to the default setting respectively detects
   *   which parameter should be shown now in the properties panel.
   *
   *   @param e The action event triggering this method.
   **/
  public void actionPerformed(ActionEvent e) {

    //1. get source
    Object pSource = e.getSource();
    if (pSource == null) return;
    if (m_bChangeListenerInAction) return;

    //2. reset the mandelbrot set
    if ((m_pResetFractalSet != null) && (pSource.equals(m_pResetFractalSet))) {
      m_bApplyReset = true;
      applyChanges();
    }


  }





  ////////////////////////////////////////
  // CHANGE LISTENER - SPINNER EVENTS   //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {

    if (m_bChangeListenerInAction) return;

    //3. fetch the new parameter value
    Object pSource = e.getSource();
    if (pSource.equals(m_pLeftUpperX) || pSource.equals(m_pLeftUpperY) || pSource.equals(m_pRightLowerX) || pSource.equals(m_pRightLowerY)) {
      m_bApplyCornerChange = true;
      applyChanges();
    }




  }




  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettings(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setLabels(pLanguageProperties);


  }


  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////





  private void createNavigationPanel() {
    setLayout(new FlowLayout());

    //reset the mandelbrot set
    m_pResetFractalSet = new JButton();
    m_pResetFractalSet.addActionListener(this);
    add(m_pResetFractalSet);

    //left upper corner
    m_pLeftUpperLabel = new JLabel();
    add(m_pLeftUpperLabel);

    double dMin = -10;
    double dMax = 10;
    double dStep = 0.01;
    SpinnerNumberModel pLeftUpperXModel = new SpinnerNumberModel(-2, dMin, dMax, dStep);
    m_pLeftUpperX = new JSpinner(pLeftUpperXModel);
    m_pLeftUpperX.addChangeListener(this);
    add(m_pLeftUpperX);

    SpinnerNumberModel pLeftUpperYModel = new SpinnerNumberModel(1.5, dMin, dMax, dStep);
    m_pLeftUpperY = new JSpinner(pLeftUpperYModel);
    m_pLeftUpperY.addChangeListener(this);
    add(m_pLeftUpperY);


    //left upper corner
    m_pRightLowerLabel = new JLabel();
    add(m_pRightLowerLabel);

    SpinnerNumberModel pRightLowerXModel = new SpinnerNumberModel(1, dMin, dMax, dStep);
    m_pRightLowerX = new JSpinner(pRightLowerXModel);
    m_pRightLowerX.addChangeListener(this);
    add(m_pRightLowerX);

    SpinnerNumberModel pRightLowerYModel = new SpinnerNumberModel(1.5, dMin, dMax, dStep);
    m_pRightLowerY = new JSpinner(pRightLowerYModel);
    m_pRightLowerY.addChangeListener(this);
    add(m_pRightLowerY);

  }


  private void setLabels(LanguageProperties pLanguageProperties) {
    m_bChangeListenerInAction = true;
    m_pResetFractalSet.setText(pLanguageProperties.getProperty(KEY_RESET));
    m_pLeftUpperLabel.setText(pLanguageProperties.getProperty(KEY_LEFT_UPPER_CORNER));
    m_pRightLowerLabel.setText(pLanguageProperties.getProperty(KEY_RIGHT_LOWER_CORNER));
    m_bChangeListenerInAction = false;
  }
}
