package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Color;

import java.awt.image.BufferedImage;

//library imports
import standard.EffectParameters;
import fractals.FractalSetEffect;
import fractals.FractalSetParameters;
import util.math.ComplexNumber;
import util.color.ColorGradient;
import gui.AbstractInteractionImageEffectPanel;
import gui.AbstractPropertiesPanel;
import gui.ISizingPanelListener;
import gui.SizingPanel;


import static fractals.MandelbrotSetParameters.*;

/**
 *  Handles the interaction with a Mandelbrot set.
 **/

public abstract class FractalSetPanel extends AbstractInteractionImageEffectPanel
  implements MouseListener, MouseMotionListener {



  /** member variables **/
  private FractalSetParameters m_pParameters = null;
  private FractalSetEffect m_pEffect = null;
  private BufferedImage m_pImage = null;
  private int m_iWidth, m_iHeight;
  private FractalSetPropertiesPanel m_pMandelbrotSetPropertiesPanel = null;

  /** interaction members **/
  private Point m_pStartPoint = null;
  private Point m_pEndPoint = null;






  /////////////////////////////////////
  // PROTECTED METHODS               //
  /////////////////////////////////////
  protected void initPanel(FractalSetParameters pParameters, FractalSetEffect pEffect) {
    m_pParameters = pParameters;
    m_pEffect = pEffect;
    setDefaultFractalSet();

    addMouseListener(this);
    addMouseMotionListener(this);

    repaint();
  }

  protected final void parametersChanged() {
    applyChanges();
    initEffectRepaint();
  }


  ///////////////////////////////////////
  // SET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Sets the effect parameters for the {@code MandelbrotSet} effect that is shown by this {@code MandelbrotSetPanel} object.
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the Mandelbrot set effect parameters.
   **/
  public void setFractalSetParameters(FractalSetParameters pParameters) {
    if (pParameters == null) return;

    m_pParameters = pParameters;
    initEffectRepaint();
  }


  /**
   *  Sets the effect parameters for the {@code MandelbrotSet} effect that is shown by this {@code MandelbrotSetPanel} object.
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pEffectParameters The {@code MandelbrotSetParameters} object representing the Mandelbrot set effect parameters.
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;

    if (pEffectParameters instanceof FractalSetParameters) {
      setFractalSetParameters((FractalSetParameters)pEffectParameters);
    }

  }




  /**
   *  Sets the effect back to the default settings.
   **/
  public void setDefaultFractalSet() {
    m_pParameters.setDefaultFractalSet();
    parametersChanged();
  }

  public void setWindowCorners(double dUpperLeftReal, double  dUpperLeftImaginary, double dLowerRightReal, double dLowerRightImaginary) {
    m_pParameters.setWindowCorners(dUpperLeftReal, dUpperLeftImaginary, dLowerRightReal, dLowerRightImaginary);
    parametersChanged();
  }

  public void setIterationDepth(int iIterationDepth) {
    m_pParameters.setIntegerParameterValue(PARAM_INT_ITERATION_DEPTH, iIterationDepth);
    parametersChanged();
  }

  public void setColorGradient(ColorGradient pColorGradient) {
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, pColorGradient);
    parametersChanged();
  }

  public void moveDown(double dStepSize) {
    m_pParameters.moveDown(dStepSize);
    parametersChanged();
  }

  public void moveUp(double dStepSize) {
    m_pParameters.moveUp(dStepSize);
    parametersChanged();
  }

  public void moveLeft(double dStepSize) {
    m_pParameters.moveLeft(dStepSize);
    parametersChanged();
  }

  public void moveRight(double dStepSize) {
    m_pParameters.moveRight(dStepSize);
    parametersChanged();
  }


  ///////////////////////////////////////
  // GET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public FractalSetParameters getFractalSetParameters() {
    return m_pParameters;
  }

  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return getFractalSetParameters();
  }

  /**
   *  Returns the {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   *
   *  @return The {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   **/
  public abstract AbstractPropertiesPanel getPropertiesPanel();



  //////////////////////////////////////
  // DRAWING HELPER METHODS           //
  //////////////////////////////////////
  protected void drawInteraction(Graphics2D pGraphics) {
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {
      pGraphics.setColor(Color.WHITE);
      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)m_iWidth;
      int iYSize = (int)(iXSize*dAspectRatio);
      if (iXSize > 0) {
        pGraphics.drawRect(m_pStartPoint.x, m_pStartPoint.y, iXSize, iYSize);
      }
      else {
        pGraphics.drawRect(m_pStartPoint.x+iXSize, m_pStartPoint.y+iYSize, -iXSize, -iYSize);
      }

    }
  }


  protected void setEffectSize(int iWidth, int iHeight) {
    if ((iWidth != m_iWidth) || (iHeight != m_iHeight)) {
      m_pParameters.setEffectSize(iWidth, iHeight);
      m_iWidth = iWidth;
      m_iHeight = iHeight;
      parametersChanged();
    }
  }








  /////////////////////////////////
  // MOUSE LISTENER INTERFACE    //
  /////////////////////////////////
  /**
   *  If the mouse is double clicked the default Mandelbrot set is shown again.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseClicked(MouseEvent e) {
    //doubleclick reset to default
    if (e.getClickCount() == 2) {
      setDefaultFractalSet();
    }
  }

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseEntered(MouseEvent e) {}

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseExited(MouseEvent e) {}

  /**
   *  Sets a start point for a zoom operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mousePressed(MouseEvent e) {
    m_pStartPoint = new Point(e.getX(), e.getY());
  }

  /**
   *  Realizes the zooming.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseReleased(MouseEvent e) {
    //set the new coordinates of the mandelbrot set
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {


      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)m_iWidth;
      int iYSize = (int)(iXSize*dAspectRatio);

      int iXPos1, iXPos2, iYPos1, iYPos2;
      if (iXSize > 0) {
        iXPos1 = m_pStartPoint.x;
        iYPos1 = m_pStartPoint.y;

        iXPos2 = iXPos1 + iXSize;
        iYPos2 = iYPos1 + iYSize;

      }
      else {
        iXPos1 = m_pStartPoint.x + iXSize;
        iYPos1 = m_pStartPoint.y + iYSize;

        iXPos2 = m_pStartPoint.x;
        iYPos2 = m_pStartPoint.y;

      }

      //correct the iXPos and the iYPos by the image offset
      int iXOffset = getImageXOffset();
      iXPos1 -= iXOffset;
      iXPos2 -= iXOffset;

      int iYOffset = getImageYOffset();
      iYPos1 -= iYOffset;
      iYPos2 -= iYOffset;


      ComplexNumber pUpperLeft = m_pEffect.getComplexNumberAt(iXPos1, iYPos1);
      ComplexNumber pLowerRight = m_pEffect.getComplexNumberAt(iXPos2, iYPos2);

      m_pParameters.setWindowCorners(pUpperLeft, pLowerRight);
      parametersChanged();
    }




    //reset the interaction variables
    m_pStartPoint = null;
    m_pEndPoint = null;

    //repaint
    repaint();
  }

  /////////////////////////////////////
  // MOUSE MOTION LISTENER INTERFACE //
  /////////////////////////////////////
  /**
   *  Sets an end point for the zooming operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseDragged(MouseEvent e) {
    if (m_pStartPoint != null) {
      m_pEndPoint = new Point(e.getX(), e.getY());
      repaint();
    }
  }

  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseMoved(MouseEvent e) {}






  ////////////////////////////////
  // PRIVATE METHODS            //
  ////////////////////////////////
  private void initEffectRepaint() {
    if (m_pEffect == null) return;

    m_pEffect.setFractalSetParameters(m_pParameters);
    m_pEffect.processAsThread(this);
  }




}
