package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;

import java.util.Vector;

//Library imports
import standard.EffectParameters;
import fractals.FractalSetParameters;
import gui.AbstractPropertiesPanel;
import gui.AbstractImageEffectPanel;
import gui.IImageEffectPanelListener;
import gui.SliderPanel;
import gui.IntegerSliderPanel;
import gui.ISliderPanelListener;
import gui.SizingPanel;
import gui.ISizingPanelListener;

import gui.color.ColorGradientPanel;
import gui.color.IColorGradientPanelListener;
import util.color.ColorGradient;
import util.math.ComplexNumber;


import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

import static applications.FractalSetLanguageProperties.*;

/**
 *  A properties panel for the mandelbrot effect.
 **/
public abstract class FractalSetPropertiesPanel
  extends AbstractPropertiesPanel
  implements
    ISliderPanelListener, IColorGradientPanelListener, IImageEffectPanelListener,
    IFractalSetNavigationPanelListener, IFractalSetButtonNavigationPanelListener, ChangeListener {

  /*** member variables ***/
  private FractalSetPanel m_pFractalSetPanel;

  /*** gui member variables ***/
  private SliderPanel m_pIterationDepthSliderPanel;
  private ColorGradientPanel m_pColorGradientPanel;
  private SizingPanel m_pSizingPanel;
  private FractalSetNavigationPanel m_pNavigationPanel;
  private FractalSetButtonNavigationPanel m_pButtonNavigationPanel;
  private boolean m_bChangeListenerInAction;


  ////////////////////////////////
  // PROTECTED METHODS          //
  ////////////////////////////////
  /**
   *  Creates a {@code MandelbrotSetPropertiesPanel} object with the parameters that are set to a given  {@code MandelbrotSetPanel}.
   *
   *  @param pMandelbrotSetPanel The {@code MandelbrotSetPanel} object from which the effect properties are taken.
   **/
  protected void initPanel(FractalSetPanel pFractalSetPanel) {
    //1. init members
    m_pFractalSetPanel = pFractalSetPanel;
    m_pFractalSetPanel.addPanelListener(this);



    //4. create the panel for the iteration depth
    m_pIterationDepthSliderPanel = new IntegerSliderPanel(10, 1000, 100);
    m_pIterationDepthSliderPanel.addPanelListener(this);


    //5. color gradient
    m_pColorGradientPanel = new ColorGradientPanel();
    m_pColorGradientPanel.addPanelListener(this);
    m_pColorGradientPanel.setMinimumSize(new Dimension(400, 50));
    m_pColorGradientPanel.setPreferredSize(new Dimension(400, 50));
    //m_pColorGradientPanel.setMinimumSize(new Dimension(400, 50));








    //7. sizing
    m_pSizingPanel = new SizingPanel();
    //m_pSizingPanel.addSizingPanelListener(this);    // not necessary. handled in the mandelbrot set panel
    m_pSizingPanel.addPanelListener(m_pFractalSetPanel);  //necessary because only this panel knows its size

    //8. navigation
    m_pNavigationPanel = new FractalSetNavigationPanel();
    m_pNavigationPanel.addPanelListener(this);

    m_pButtonNavigationPanel = new FractalSetButtonNavigationPanel();
    m_pButtonNavigationPanel.addPanelListener(this);



    //8. create the properties panel and populate it
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    setLabels(pLanguageProperties);


    m_bChangeListenerInAction = false;

    initLayout();


  }

  //this is to implement, if there are further properties
  protected Vector<String> getAdditionalPropertyKeys() {
    return null;
  }

  //this is to implement, if there are further properties
  protected Vector<JPanel> getAdditionalPropertyPanels() {
    return null;
  }



  //this is to implement, if there are further properties that are waiting for a change listener event
  protected void changeListenerAction(ChangeEvent e) {

  }

  //this is to implement, if there are further properties that are waiting for a change listener event
  protected void applyChangesFromSliderPanel(SliderPanel pPanel, int iValue, double dValue) {

  }

  protected void setAdditionalLabels(LanguageProperties pLanguageProperties) {

  }


  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pFractalSetPanel.getEffectParameters();
  }


  public Vector<JPanel> getPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pIterationDepthSliderPanel);
    pPanels.add(m_pColorGradientPanel);
    pPanels.add(m_pSizingPanel);
    pPanels.add(m_pNavigationPanel);
    pPanels.add(m_pButtonNavigationPanel);
    Vector<JPanel> pAdditionalPanels = getAdditionalPropertyPanels();
    if (pAdditionalPanels != null) {
      pPanels.addAll(pAdditionalPanels);
    }
    return pPanels;

  }

  public Vector<String> getPropertyKeys() {
    Vector<String> pKeys = new Vector<String>();
    pKeys.add(KEY_ITERATION_DEPTH);
    pKeys.add(KEY_COLOR_GRADIENT);
    pKeys.add(KEY_SIZE);
    pKeys.add(KEY_NAVIGATION);
    pKeys.add(KEY_NAVIGATION2);

    Vector<String> pAdditionalKeys = getAdditionalPropertyKeys();
    if (pAdditionalKeys != null) {
      pKeys.addAll(pAdditionalKeys);
    }
    return pKeys;
  }

  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the effect parameters
   *                     that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;

    if (pParameters instanceof FractalSetParameters) {
      FractalSetParameters pFractalSetParameters = (FractalSetParameters)pParameters;

      ComplexNumber pUpperLeftCorner = pFractalSetParameters.getUpperLeftWindowCorner();
      ComplexNumber pRightLowerCorner = pFractalSetParameters.getLowerRightWindowCorner();
      m_pNavigationPanel.setWindowCorners(pUpperLeftCorner.m_dReal, pUpperLeftCorner.m_dImaginary,
        pRightLowerCorner.m_dReal, pRightLowerCorner.m_dImaginary);

    }
  }

  //////////////////////////////////////////////
  // PROTECTED LISTENER METHODS FOR INSTANCES //
  //////////////////////////////////////////////
  /**
   *  Informs all registered listeners that a change in the properties panel has been made. The image effect parameters
   *  of this properties panel are passed on to the listeners.
   *
   *  @param pEffectParameters The image effect parameters that should be passed on to the listeners.
   *
   **/
  protected void fireApplyChanges(EffectParameters pEffectParameters) {
    super.fireApplyChanges(pEffectParameters);

    setEffectParameters(pEffectParameters);
  }


  ///////////////////////////////////
  // IMAGE EFFECT PANEL LISTENER   //
  ///////////////////////////////////
  public void applyImageEffectPanelChanges(AbstractImageEffectPanel pPanel) {
    setEffectParameters(pPanel.getEffectParameters());
  }


  ///////////////////////////////////
  // SLIDER PANEL LISTENER         //
  ///////////////////////////////////
  /**
   *  If the iteration depth has been adapted
   *  in the corresponding slider panel, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified.
   **/
  public void applyChanges(SliderPanel pSliderPanel, int iValue, double dValue) {

    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pFractalSetPanel == null) return;

    //2. set the new parameters
    if (pSliderPanel.equals(m_pIterationDepthSliderPanel)) {
      m_pFractalSetPanel.setIterationDepth(iValue);
    }
    else {
      applyChangesFromSliderPanel(pSliderPanel, iValue, dValue);
    }


  }

  ////////////////////////////////////////
  // COLOR GRADIENT PANEL LISTENER      //
  ////////////////////////////////////////
  /**
   *  If the color gradient has been modified in the specified {@code ColorGradientPanel}, it is
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorGradientPanel The {@code ColorGradientPanel} object that has been modified.
   **/
  public void colorGradientChanged(ColorGradientPanel pColorGradientPanel) {
    //1. make some checks
    if (pColorGradientPanel == null) return;
    if (m_pFractalSetPanel == null) return;

    //2. set the color gradient
    ColorGradient pColorGradient = pColorGradientPanel.getColorGradient();
    if (pColorGradient == null) return;

    m_pFractalSetPanel.setColorGradient(pColorGradient);

  }




  ////////////////////////////////////////
  // COMPONENT LISTENER                 //
  ////////////////////////////////////////
  protected void componentResizedForChildren(ComponentEvent e, JPanel currentPropertyPanel) {
    if (currentPropertyPanel.equals(m_pColorGradientPanel)) {
      m_pColorGradientPanel.setPreferredSize(new Dimension(getColorGradientPanelWidth(), 50));
      m_pColorGradientPanel.repaint();
      m_pColorGradientPanel.revalidate();

    }

  }


  ////////////////////////////////////////
  // CHANGE LISTENER - SPINNER EVENTS   //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    //TODO: check if this is still needed
    if (m_bChangeListenerInAction) return;
    m_bChangeListenerInAction = true;

    //3. fetch the new parameter value
    changeListenerAction(e);

    m_bChangeListenerInAction = false;


  }

  /////////////////////////////////////
  // NAVIGATION PANEL LISTENER       //
  /////////////////////////////////////
  public void applyResetFractalSet(FractalSetNavigationPanel panel) {
    m_bChangeListenerInAction = true;
    m_pFractalSetPanel.setDefaultFractalSet();
    m_bChangeListenerInAction = false;
  }

  public void applyCornerChange(FractalSetNavigationPanel panel,
    double dLeftUpperX, double dLeftUpperY, double dRightLowerX, double dRightLowerY) {

    m_bChangeListenerInAction = true;
    m_pFractalSetPanel.setWindowCorners(dLeftUpperX, dLeftUpperY, dRightLowerX, dRightLowerY);
    m_bChangeListenerInAction = false;
  }


  ////////////////////////////////////////////
  // BUTTON NAVIGATION PANEL LISTENER       //
  ////////////////////////////////////////////
  public void moveRight(FractalSetButtonNavigationPanel source, double dStepSize) {
    m_pFractalSetPanel.moveRight(dStepSize);
  }
  public void moveLeft(FractalSetButtonNavigationPanel source, double dStepSize) {
    m_pFractalSetPanel.moveLeft(dStepSize);
  }
  public void moveDown(FractalSetButtonNavigationPanel source, double dStepSize) {
    m_pFractalSetPanel.moveDown(dStepSize);
  }
  public void moveUp(FractalSetButtonNavigationPanel source, double dStepSize) {
    m_pFractalSetPanel.moveUp(dStepSize);
  }




  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed,
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels.
   **/
  public void applyNewSettingsForChildren(Settings pSettings) {

    //1. make an initial check and set the settings
    if (pSettings == null) return;

    //2. load new language settings
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();
    setLabels(pLanguageProperties);



    //4. sizing panel
    if (m_pSizingPanel != null) {
      m_pSizingPanel.applyNewSettings(pSettings);
    }

    if (m_pNavigationPanel != null) {
      m_pNavigationPanel.applyNewSettings(pSettings);
    }

    if (m_pButtonNavigationPanel != null) {
      m_pButtonNavigationPanel.applyNewSettings(pSettings);
    }




  }


  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////

  private int getColorGradientPanelWidth() {
    return getWidth() - 50 - getPropertiesSelectionWidth();
  }







  private void setLabels(LanguageProperties pLanguageProperties) {

    setAdditionalLabels(pLanguageProperties);
  }
}
