package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;

import java.awt.Color;

//library imports
import standard.BooleanParameter;
import standard.IntegerParameter;
import standard.DoubleParameter;
import standard.ColorParameter;


/** 
 *  Handles a configuration file and loads its values. The values can be retrieved via this class. 
 *  TO DO: It would be better to use java.util.Properties. (https://de.wikipedia.org/wiki/Java-Properties-Datei)
 **/

public class Configuration {

  private Hashtable<String, String> m_pParameters;
  private String m_pFileName;


  /**
   *  Creates a {@code Configuration} object that loads the specified file. 
   *
   *  @param pFileName The file name as string. 
   **/
  public Configuration(String pFileName) {
    //1. init the members
    m_pParameters = null;
    m_pFileName = pFileName;
  }
  
  /////////////////////////////
  // GET METHODS             //
  /////////////////////////////
  /**
   *  Returns the file name that is used by this @code{Configuration} object.
   *
   *  @return The file name that is used by this @code{Configuration} object.
   **/
  public String getFileName() {
    return m_pFileName;
  }
  
  /**
   *  Returns the parameter value as string, without throwing any Exception. If an Exception ocurrs in the load process, then 
   *  null is returned. 
   *
   *  @param pParameter The key of the parameter for which the value should be returned. 
   *
   *  @return The value of the parameter as string.
   **/
  public String getParameterValueWithoutException(String pParameter) {
    try {
      return getParameterValue(pParameter);
    }
    catch (Exception pException) {
      return null;
    }
  }
  
  /**
   *  Returns the parameter value as string. 
   *
   *  @param pParameter The key of the parameter for which the value should be returned. 
   *
   *  @throws Exception When the parameter value is not found or if the configuration file could not be loaded.
   *
   *  @return The value of the parameter as string.
   **/
  public String getParameterValue(String pParameter) throws Exception {
    //1. make some checks
    if (pParameter == null) throw new NullPointerException("No parameter name was specified!");
    pParameter = pParameter.trim().toLowerCase();
    
    //2. fetch the parameters if necessary
    if (m_pParameters == null) {
      fetchParameters();
    }
    
    //3. return the value of the parameter
    String pValue = m_pParameters.get(pParameter);
    if (pValue == null) {
      throw new Exception("The parameter '" + pParameter + "' does not exist!");
    }
    return pValue;
    
  }
  
  /**
   *  Returns the boolean parameter value from a configuration file as {@code BooleanParameter}. 
   *
   *  @param pParameter The key of the parameter for which the {@code BooleanParameter} object should be returned. 
   *
   *  @throws Exception When the parameter value is not found, if the configuration file could not be loaded or 
   *                    the value of the parameter has not the correct format. 
   *
   *  @return The boolean parameter value from a configuration file as {@code BooleanParameter}. 
   **/
  public BooleanParameter getBooleanParameter(String pParameter) throws Exception {
    //1. fetch the string value
    String pValue = getParameterValue(pParameter);
    pValue = pValue.trim();
    
    //2. check the true values
    String[] pTrueValues = {"1", "t", "true"};
    for (int i = 0; i < pTrueValues.length; i++) {
      if (pValue.equalsIgnoreCase(pTrueValues[i])) {
        return new BooleanParameter(true);
      }
    }
    
    
    //3. check the false values
    String[] pFalseValues = {"0", "f", "false"};
    for (int i = 0; i < pFalseValues.length; i++) {
      if (pValue.equalsIgnoreCase(pFalseValues[i])) {
        return new BooleanParameter(false);
      }
    }
    
    throw new Exception ("The value '" + pValue + "' for the parameter '" + pParameter + 
    "' has not the correct format for a BooleanParameter!");
  }
  
  
  /**
   *  Returns the integer parameter value from a configuration file as {@code IntegerParameter}. 
   *
   *  @param pParameter The key of the parameter for which the {@code IntegerParameter} object should be returned. 
   *
   *  @throws Exception When the parameter value is not found, if the configuration file could not be loaded or 
   *                    the value of the parameter has not the correct format. 
   *
   *  @return The integer parameter value from a configuration file as {@code IntegerParameter}. 
   **/
  public IntegerParameter getIntegerParameter(String pParameter) throws Exception {
    //1. fetch the string value
    String pValue = getParameterValue(pParameter);
    pValue = pValue.trim();
    
    //2. try to parse the value
    try {
      int iValue = Integer.parseInt(pValue);
      return new IntegerParameter(iValue, Integer.MIN_VALUE, Integer.MAX_VALUE);
    }
    catch (Exception pException) {
      throw new Exception ("The value '" + pValue + "' for the parameter '" + pParameter + 
      "' has not the correct format for an IntegerParameter!");
    }
  }
  
  /**
   *  Returns the double parameter value from a configuration file as {@code DoubleParameter}. 
   *
   *  @param pParameter The key of the parameter for which the {@code DoubleParameter} object should be returned. 
   *
   *  @throws Exception When the parameter value is not found, if the configuration file could not be loaded or 
   *                    the value of the parameter has not the correct format. 
   *
   *  @return The double parameter value from a configuration file as {@code DoubleParameter}. 
   **/
  public DoubleParameter getDoubleParameter(String pParameter) throws Exception {
    //1. fetch the string value
    String pValue = getParameterValue(pParameter);
    pValue = pValue.trim();
    
    //2. try to parse the value
    try {
      double dValue = Double.parseDouble(pValue);
      return new DoubleParameter(dValue, Double.MIN_VALUE, Double.MAX_VALUE);
    }
    catch (Exception pException) {
      throw new Exception ("The value '" + pValue + "' for the parameter '" + pParameter + 
      "' has not the correct format for an DoubleParameter!");
    }
  }
  
  
  /**
   *  Returns the color parameter value from a configuration file as {@code ColorParameter}. 
   *
   *  @param pParameter The key of the parameter for which the {@code ColorParameter} object should be returned. 
   *
   *  @throws Exception When the parameter value is not found, if the configuration file could not be loaded or 
   *                    the value of the parameter has not the correct format. 
   *
   *  @return The color parameter value from a configuration file as {@code ColorParameter}. 
   **/
  public ColorParameter getColorParameter(String pParameter) throws Exception {
    //1. fetch the string value
    String pValue = getParameterValue(pParameter);
    pValue = pValue.trim();
  
  
    //2. parse pre defined values
    Hashtable<String, Color> pColorMapping = new Hashtable<String, Color>();
    pColorMapping.put("black", Color.BLACK);
    pColorMapping.put("blue", Color.BLUE);
    pColorMapping.put("cyan", Color.CYAN);
    pColorMapping.put("darkgray", Color.DARK_GRAY);
    pColorMapping.put("gray", Color.GRAY);
    pColorMapping.put("green", Color.GREEN);
    pColorMapping.put("lightgray", Color.LIGHT_GRAY);
    pColorMapping.put("magenta", Color.MAGENTA);
    pColorMapping.put("orange", Color.ORANGE);
    pColorMapping.put("pink", Color.PINK);
    pColorMapping.put("red", Color.RED);
    pColorMapping.put("white", Color.WHITE);
    pColorMapping.put("yellow", Color.YELLOW);
    
    Color pColor = pColorMapping.get(pValue.toLowerCase());
    if (pColor != null) {
      return new ColorParameter(pColor);
    }
    
    //3. parse rgb code 255,255,255
    String[] pRGB = pValue.split(",");
    if (pRGB.length != 3) {
      throw new Exception("The value '" + pValue + "' for the parameter '" + pParameter + 
      "' has not the correct format for a ColorParameter, because there are not 3 color components!");
    }
    
    try {
      int iRed = Integer.parseInt(pRGB[0]);
      int iGreen = Integer.parseInt(pRGB[1]);
      int iBlue = Integer.parseInt(pRGB[2]);
      return new ColorParameter(new Color(iRed, iGreen, iBlue));
    }
    catch (Exception pException) {
      throw new Exception("The value '" + pValue + "' for the parameter '" + pParameter + 
      "' has not the correct format for a ColorParameter, because the color components do not have the correct format!");
    }
  }
  
  //FUTURE METHODS
  // public IntegerParameter getIntegerParameter(String pParameter)
  
  ///////////////////////////////
  // SET METHODS               //
  ///////////////////////////////
  /**
   *   Sets the name of the configuration file. 
   *
   *   @param pFileName The name of the configuration file as string.
   **/
  public void setFileName(String pFileName) {
    //1. set the members to null, if necessary
    if (pFileName == null) {
      m_pFileName = null;
      m_pParameters = null;
      return;
    }
    
    //2. do something if the filename changes
    if ((m_pFileName == null) || (!pFileName.equals(m_pFileName))) {
      m_pFileName = pFileName;
      m_pParameters = null;
    }
  }
  
  
  //////////////////////////////////////
  // PRIVATE HELPER METHODS           //
  //////////////////////////////////////
  private void fetchParameters() throws Exception {
    //1. make some checks
    if (m_pFileName == null) throw new NullPointerException("No file name for the configuration was specified!");
    File pFile = new File(m_pFileName);
    if (!pFile.exists()) throw new Exception("The file '" + pFile.getAbsolutePath() + "' does not exist!");
    
    //2. open the file for reading
    try {
      BufferedReader pReader = new BufferedReader(new FileReader(pFile));
      String pLine = null;
      m_pParameters = new Hashtable<String, String>();
      
      while((pLine = pReader.readLine()) != null) {
        
        //I. prepare the line
        pLine = pLine.trim();
        
        //II. skip lines that start with #
        if (pLine.startsWith("#")) continue;
      
        //III. process the line
        String pKey = null;
        String pValue = null;
        int iIndex = pLine.indexOf("=");
        
        if (iIndex == -1) {
          pKey = pLine;
          pValue = "";
        }
        else {
          pKey = pLine.substring(0, iIndex).trim();
          pValue = pLine.substring(iIndex+1, pLine.length()).trim();
        }
        
        m_pParameters.put(pKey.toLowerCase(), pValue);
        
        
      }
      
      pReader.close();
      
    }
    catch (Exception pException) {
      throw new Exception("The configuration file '" + pFile.getAbsolutePath() + "' could not be parsed!", pException);
    }
  }


}