package motionanalysis;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.util.Vector;


//library imports
import standard.VideoEffect;
import standard.IVideo;

import util.image.ImageDifferenceOperations;
import util.image.ImageTransformations;
import util.image.FillOperations;
import util.image.ImageCombineOperations;


//static import for using static constants
import static motionanalysis.FrameDifferencingParameters.*;



/**
 *  Implements a frame differencing algorithm to detect differences 
 *  between two consecutive images (frames) of a video or image sequence.
 **/
public class FrameDifferencing extends VideoEffect {


  /**** private variables ****/
  private FrameDifferencingParameters m_pParameters;


  //////////////////////
  // CONSTRUCTOR      //
  //////////////////////
  /**
   *  Creates a frame differencing effect object that can process the given video with the given 
   *  effect parameters.
   *
   *  @param pVideo The video that should be processed.
   *  @param pParameters The effect parameters. 
   *
   *  @throws NullPointerException If either the video or the effect parameters are null.
   **/
  public FrameDifferencing(IVideo pVideo, FrameDifferencingParameters pParameters) throws NullPointerException {

    //1. make some checks
    if (pVideo == null) throw new NullPointerException("No video was defined!");
    if (pParameters == null) throw new NullPointerException("No parameters were defined!");
  
    //1. seth the video variables
    setVideo(pVideo);

    //2. set the parameters
    setParameters(pParameters);


  }


  //////////////////////
  // SET METHODS      //
  //////////////////////
  /**
   *   Sets the effect parameters. If the effect parameters are null, nothing is done.
   *
   *   @param pParameters The effect parameters that should be used.
   **/
  public void setParameters(FrameDifferencingParameters pParameters) {
    if (pParameters == null) return;
    m_pParameters = pParameters;
  }


  //////////////////////
  // GET METHODS      //
  //////////////////////





  /** 
   *  Processes the image differencing effect according to the specified parameters and applies it on the specified video. 
   *
   **/
  public void process() {
    //TO DO: split this method up into several private methods?
    //       set video via a father video effect class?
  
    //1. fetch important information from father class and make some simple checks
    IVideo pVideo = getVideo();
    if (pVideo == null) {
      fireInterruptedEvent("No valid video was specified!!!");
      return;
    }
    
    fireProcessingStarted();

    int iPosition = getPosition();
    int iFrameNum = pVideo.getFrameNumber();


    //2. fetch settings from the keyframe that are valid for each iteration
    boolean bConvertDiffImage = m_pParameters.getBooleanParameterValue(PARAM_BOOL_CONVERT_DIFF_IM);
    boolean bShowOrigFrame = m_pParameters.getBooleanParameterValue(PARAM_BOOL_SHOW_ORIG_FRAME);
    boolean bShowDiffIm = m_pParameters.getBooleanParameterValue(PARAM_BOOL_SHOW_DIFF_IM);
    boolean bShowMotionMask = m_pParameters.getBooleanParameterValue(PARAM_BOOL_SHOW_MOTION_MASK);
    boolean bShowHistory = m_pParameters.getBooleanParameterValue(PARAM_BOOL_SHOW_HISTORY);
    boolean bShowResult = m_pParameters.getBooleanParameterValue(PARAM_BOOL_SHOW_RESULT);
    boolean bHistoryOn = m_pParameters.getBooleanParameterValue(PARAM_BOOL_HISTORY_ON);
    boolean bHistoryFade = m_pParameters.getBooleanParameterValue(PARAM_BOOL_HISTORY_FADE);
    boolean bHistoryKeep = m_pParameters.getBooleanParameterValue(PARAM_BOOL_HISTORY_KEEP);

    //3. fetch the old frame
    BufferedImage pOldFrame = pVideo.getFrame(iPosition);
    iPosition++;
	
    //4. run through all frames of the video
    BufferedImage pNewFrame;
    BufferedImage pDiffImage;
    BufferedImage pMotionMask;
    BufferedImage pFilledFrame;
    BufferedImage pResultImage;

    //5. if the history is not set, just work through the image sequence with image operations
    if (!bHistoryOn) {
      for (int i = iPosition; i < iFrameNum; i++) {

        //I. fetch the new frame
        pNewFrame = pVideo.getFrame(i);

        //II. create the absolute color difference image and convert it to a grayscale image
        pDiffImage = ImageDifferenceOperations.calculateAbsDiffImage(pOldFrame, pNewFrame);
        if (bConvertDiffImage) {
          ImageTransformations.convertToGrayscaleImage2(pDiffImage);
        }
       
        //III. create the binary image by thresholding
        int iThreshold = m_pParameters.getIntegerParameterValue(PARAM_INT_THRESHOLD);
        pMotionMask = ImageTransformations.convertToBinaryImageCopy(pDiffImage, iThreshold);

        //IV. fill the original frame with the motion mask
        Color pMarkerColor = m_pParameters.getColorParameterValue(PARAM_COLOR_HISTORY_COLOR);
        pFilledFrame = FillOperations.fillMaskCopy(pNewFrame, pMarkerColor, pMotionMask);

        //V.. set the variables for the next run
        pOldFrame = pNewFrame;

        //VI. create the result image
        Vector<BufferedImage> pImages = new Vector<BufferedImage>(4);
        if (bShowOrigFrame) pImages.add(pNewFrame);
        if (bShowDiffIm) pImages.add(pDiffImage);
        if (bShowMotionMask) pImages.add(pMotionMask);
        if (bShowResult) pImages.add(pFilledFrame);

        if (pImages.size() > 1) {
          pResultImage = ImageCombineOperations.concatenateImages(pImages);
        }
        else if (pImages.size() == 1) {
          pResultImage = pImages.get(0);
        }
        else {
          pResultImage = null;
        }


        //VII. fire events
        fireSetCurrentVideoFrame(pResultImage, i, iFrameNum);

        double dProgress = (i+1)/(double)iFrameNum;
        fireProgressStatusEvent(dProgress, "Frame differencing: Frame " + (i+1) + " from " + iFrameNum + " rendered.");


        //VIII. check if aborted
        if (isVideoEffectAborted()) {
          fireInterruptedEvent("The processing of the video was interrupted by the user.");
          return;
        }
      }
    }
    
    //6. use a history map for the frame differencing
    else {
    
      //6.1. create and init a history map
      int iWidth = pOldFrame.getWidth();
      int iHeight = pOldFrame.getHeight();
      int[][] pHistoryMap = new int[iWidth][iHeight];

      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {
          pHistoryMap[x][y] = -1;
        }
      }
      
      
      //6.2. work through the image sequence
      BufferedImage pHistoryImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
      
      for (int i = iPosition; i < iFrameNum; i++) {

        //I. fetch the new frame
        pNewFrame = pVideo.getFrame(i);

        //II. create the absolute color difference image and convert it to a grayscale image
        pDiffImage = ImageDifferenceOperations.calculateAbsDiffImage(pOldFrame, pNewFrame);
        if (bConvertDiffImage) {
          ImageTransformations.convertToGrayscaleImage2(pDiffImage);
        }
       
        //III. create the binary image by thresholding
        int iThreshold = m_pParameters.getIntegerParameterValue(PARAM_INT_THRESHOLD);
        pMotionMask = ImageTransformations.convertToBinaryImageCopy(pDiffImage, iThreshold);
        
        //IV. include the motion mask in the history map
        int iMinWidth = Math.min(pMotionMask.getWidth(), iWidth);
        int iMinHeight = Math.min(pMotionMask.getHeight(), iHeight);
        int iWhiteRGB = Color.WHITE.getRGB();
        
        for (int x = 0; x < iMinWidth; x++) {
          for (int y = 0; y < iMinHeight; y++) {
            if (pMotionMask.getRGB(x,y) == iWhiteRGB) {
              pHistoryMap[x][y] = i; //set the current position in the video
            }
          }
        }
        
        
        //V. calculate the history image
        //V.I fading is active
        if (bHistoryFade) {
        
          int iHistoryFrameNum = m_pParameters.getIntegerParameterValue(PARAM_INT_HISTORY_FRAME_NUM);
          int iHistoryLimit = i - iHistoryFrameNum;
          if (iHistoryLimit < 0) iHistoryLimit = 0;
          
          
          int iFadeFrameNum = m_pParameters.getIntegerParameterValue(PARAM_INT_HISTORY_FADE_FRAME_NUM);
          int iFadeLimit = i - iHistoryFrameNum - iFadeFrameNum; 
          if (iFadeLimit < 0) iFadeLimit = 0;
          int[] pShades = new int[iFadeFrameNum];
          double dShadeStep = 255/(double)iFadeLimit;
          
          
          for (int iId = 0; iId < iFadeFrameNum; iId++) {
            int iCurrentShade = (int)(iId*dShadeStep);
            if (iCurrentShade > 255) iCurrentShade = 255;
            pShades[iId] = (255 << 24) | (iCurrentShade << 16) | (iCurrentShade << 8) | iCurrentShade;
          }
          
          
          int iBlackRGB = Color.BLACK.getRGB();
        
          for (int x = 0; x < iMinWidth; x++) {
            for (int y = 0; y < iMinHeight; y++) {
            
              if (pHistoryMap[x][y] >= iHistoryLimit) {
                pHistoryImage.setRGB(x,y, iWhiteRGB);
              }
              else if (pHistoryMap[x][y] < iFadeLimit) {
                pHistoryImage.setRGB(x,y, iBlackRGB);
              }
              else {
                int iShadeId = pHistoryMap[x][y] - iFadeLimit;
                pHistoryImage.setRGB(x,y, pShades[iShadeId]);
              }
              
            }
          }
          
        }
        
        //V. II. fading is not active
        else {
          int iHistoryFrameNum = m_pParameters.getIntegerParameterValue(PARAM_INT_HISTORY_FRAME_NUM);
          int iHistoryLimit = i - iHistoryFrameNum;
          if (iHistoryLimit < 0) iHistoryLimit = 0;
          if (bHistoryKeep) iHistoryLimit = 0;
          
          int iBlackRGB = Color.BLACK.getRGB();
          
          for (int x = 0; x < iMinWidth; x++) {
            for (int y = 0; y < iMinHeight; y++) {
            
              if (pHistoryMap[x][y] >= iHistoryLimit) {
                pHistoryImage.setRGB(x,y, iWhiteRGB);
              }
              else {
                pHistoryImage.setRGB(x,y, iBlackRGB);
              }
            
            }
          }
        }
    
        //VI. fill the original frame with the history
        Color pMarkerColor = m_pParameters.getColorParameterValue(PARAM_COLOR_HISTORY_COLOR);
        pFilledFrame = FillOperations.fillMaskCopy(pNewFrame, pMarkerColor, pHistoryImage);

        //VII. set the variables for the next run
        pOldFrame = pNewFrame;

        //VIII. create the result image
        Vector<BufferedImage> pImages = new Vector<BufferedImage>(4);
        if (bShowOrigFrame) pImages.add(pNewFrame);
        if (bShowDiffIm) pImages.add(pDiffImage);
        if (bShowMotionMask) pImages.add(pMotionMask);
        if (bShowHistory) pImages.add(pHistoryImage);
        if (bShowResult) pImages.add(pFilledFrame);
        

        if (pImages.size() > 1) {
          pResultImage = ImageCombineOperations.concatenateImages(pImages);
        }
        else if (pImages.size() == 1) {
          pResultImage = pImages.get(0);
        }
        else {
          pResultImage = null;
        }


        //IX. fire events
        fireSetCurrentVideoFrame(pResultImage, i, iFrameNum);

        double dProgress = (i+1)/(double)iFrameNum;
        fireProgressStatusEvent(dProgress, "Frame differencing: Frame " + (i+1) + " from " + iFrameNum + " rendered.");


        //X. check if aborted
        if (isVideoEffectAborted()) {
          fireInterruptedEvent("The processing of the video was interrupted by the user.");
          return;
        }
      } 
    
    }

    //finally. ended properly
    fireEndedProperlyEvent();


  }


}
