package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import javax.swing.JCheckBox;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/**
 *  Represents an image or video effect parameter that has a boolean value. 
 **/
public class BooleanParameter implements IParameter, ActionListener {


  /*** member variables ***/
  private boolean m_bParameterValue;
  private JCheckBox m_pCheckBox;


  /////////////////////////
  // CONSTRUCTOR         //
  /////////////////////////
  /**
   *  Creates a {@code BooleanParameter} object with the specified boolean value. 
   *
   * @param bValue The boolean value that should be represented by this {@code BooleanParameter} object.
   **/
  public BooleanParameter(boolean bValue) {

    //1. set the member variables
    m_bParameterValue = bValue;
    m_pCheckBox = null;
  }



  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /**
   * Sets the boolean value that should be represented by this {@code BooleanParameter} object.
   *
   * @param bValue The boolean value that should be represented by this {@code BooleanParameter} object.
   **/   
  public void setValue(boolean bValue) {
    m_bParameterValue = bValue;

    if (m_pCheckBox != null) m_pCheckBox.setSelected(bValue);

  }
  
  
  /**
   * Sets the boolean value that should be represented by this {@code BooleanParameter} object.
   * The boolean value is representated by a string. 1, t, true (independant of the case) are 
   * mapped to true. 0, f, false are mapped to false.
   *
   * @param pValue The string value representing the boolean that should be represented by this {@code BooleanParameter} object.
   *
   * @return true, if the boolean value could be successfully set. False otherwise.
   **/   
  public boolean setValue(String pValue) {
    //1. make a simple check
    if (pValue == null) return false;
    pValue = pValue.trim();
    
    //2. check the true values
    String[] pTrueValues = {"1", "t", "true"};
    for (int i = 0; i < pTrueValues.length; i++) {
      if (pValue.equalsIgnoreCase(pTrueValues[i])) {
        setValue(true);
        return true;
      }
    }
    
    
    //3. check the false values
    String[] pFalseValues = {"0", "f", "false"};
    for (int i = 0; i < pFalseValues.length; i++) {
      if (pValue.equalsIgnoreCase(pFalseValues[i])) {
        setValue(false);
        return true;
      }
    }
    
    return false;
  }


  /////////////////////////
  // GET METHODS         //
  /////////////////////////
  
  /**
   * Returns the boolean value that is represented by this {@code BooleanParameter} object.
   *
   * @return The boolean value that is represented by this {@code BooleanParameter} object.
   **/   
  public boolean getValue() {
    return m_bParameterValue;
  }

  
  
  /**
   * Returns gui element of a {@code BooleanParameter} as it is needed for an image effect.
   * In this case it is a {@code JCheckBox} object.
   *
   * @param pLabelText The text for the label of the checkbox as string.
   *
   * @return The gui element of a {@code BooleanParameter} as it is needed for an image effect. 
   **/   
  
  public JCheckBox getImageEffectGUIElement(String pLabelText) {

    //1. make a simple check
    if (pLabelText == null) pLabelText = "";

    //2. create and return the checkbox
    m_pCheckBox = new JCheckBox(pLabelText);
    m_pCheckBox.setSelected(m_bParameterValue);
    m_pCheckBox.addActionListener(this);

    return m_pCheckBox;
  }



  ////////////////////////////////
  // ACTION LISTENER INTERFACE  //
  ////////////////////////////////
  /**
   *  If the value of the checkbox that represents this parameter in the gui has been changed, 
   *  also the value of this {@code BooleanParameter} as adapted. 
   *
   *  @param e The event that triggered the value change.
   **/
  public void actionPerformed(ActionEvent e) {

    //1. make some checks
    if (m_pCheckBox == null) return;


    //2. set the value
    m_bParameterValue = m_pCheckBox.isSelected();

  }


}
