package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java api imports
import java.util.Vector;
import java.awt.image.BufferedImage;

/**
 *  Abstract class defining the interface for video effects and 
 *  implementing helping methods that are valid for all video effects.
 **/

public abstract class VideoEffect extends ThreadEventManager implements Runnable {


  //List of VideoEffectListener objects
  private Vector<IVideoEffectListener>   m_pVideoEffectListeners  = null;

  //flag for running the video effect as thread
  private boolean m_bAbort;


  /*** private video variables ***/
  private IVideo        m_pVideo              = null;
  private int           m_iPosition           = 0;
  private BufferedImage m_pCurrentEffectImage = null;



  /////////////////////////////////////////////////
  // METHODS FOR HANDLING VIDEO EFFECT LISTENERS //
  /////////////////////////////////////////////////

  
  /**
   *  Adds a video effect listener object and adds it also as thread listener. 
   *
   *  @param pVideoEffectListener The video effect listener that should be added. 
   *
   **/
  public void addVideoEffectListener(IVideoEffectListener pVideoEffectListener) {

    //1. make a simple check
    if (pVideoEffectListener == null) return;

    //2. create the list of IVideoEffectListener objects
    if (m_pVideoEffectListeners == null) {
      m_pVideoEffectListeners = new Vector<IVideoEffectListener>();
    }

    //3. add the IVideoEffectListener object
    m_pVideoEffectListeners.add(pVideoEffectListener);

    //4. also add the VideoEffectListener as thread listener, so that
    //   it receives common thread events
    addThreadListener(pVideoEffectListener);
  }


  
  /**
   *  Adds a video effect listener object and adds it also as thread listener at the specified position.
   *  TO DO: Is the position parameter really necessary? It does not seem to be correctly implemented.    
   *
   *  @param pVideoEffectListener The video effect listener that should be added. 
   *  @param iPosition The position where the video effect listener should be added. 
   *
   **/
  public void addVideoEffectListener(IVideoEffectListener pVideoEffectListener, int iPosition) {

    //1. make a simple check
    if (pVideoEffectListener == null) return;

    //2. create the list of IVideoEffectListener objects
    if (m_pVideoEffectListeners == null) {
      m_pVideoEffectListeners = new Vector<IVideoEffectListener>();
    }

    //3. add the IThreadListener object
    if (iPosition < 0) addVideoEffectListener(pVideoEffectListener);
    if (iPosition > m_pVideoEffectListeners.size()) addVideoEffectListener(pVideoEffectListener);
    m_pVideoEffectListeners.add(pVideoEffectListener);

    //4. also add the VideoEffectListener as thread listener, so that
    //   it receives common thread events
    addThreadListener(pVideoEffectListener, iPosition);


  }


  /**
   *  Removes the specified video effect listener. 
   *
   *  @param pVideoEffectListener The video effect listener object that should be removed. 
   **/
  public void removeThreadListener(IVideoEffectListener pVideoEffectListener) {
    //1. make a simple check
    if (m_pVideoEffectListeners == null) return;
    if (pVideoEffectListener == null) return;

    //2. remove the IVideoEffectListener object, as soon as no pending operation is executed anymore
    m_pVideoEffectListeners.remove(pVideoEffectListener);

    //3. also remove the VideoEffectListener from the thread listeners, so that
    //   it does no longer receive common thread events
    removeThreadListener(pVideoEffectListener);

  }

  /**
   *  Removes all video effect listeners. 
   **/
  public void removeAllVideoEffectListeners() {
    //1. make a simple check
    if (m_pVideoEffectListeners == null) return;

    //2. remove all IVideoEffectListener objects, as soon as no pending operation is executed anymore
    m_pVideoEffectListeners.removeAllElements();

    //3. also remove all IVideoEffectListener from the ThreadEventList
    removeAllThreadListeners();
  }




  /////////////////////////////////////////////////
  // METHODS FOR COMMUNICATING VIDEOEFFECTEVENTS //
  /////////////////////////////////////////////////
  /**
   *  Sets the current effect frame, which can be retrieved afterwards by the getCurrentFrame method, and
   *  communicates the current status of the video effect to all
   *  registered listeners. This method should only used by classes extending the <code>VideoEffect</code>
   *  class.
   *
   *  @param pCurrentFrame The current frame as {@code BufferedImage} object. 
   *  @param iFrameId The current frame id as integer. 
   *  @param iFrameNumber The total number of frames as integer. 
   **/
  protected void fireSetCurrentVideoFrame(BufferedImage pCurrentFrame, int iFrameId, int iFrameNumber) {
    //1. make a simple value check
    if (m_pVideoEffectListeners == null) return;
    if (pCurrentFrame == null) return;


    //2. set the current frame as current effect frame member variable
    m_pCurrentEffectImage = pCurrentFrame;

    //3. fire the progress status events
    IVideoEffectListener pListener;

    for (int i = 0; i < m_pVideoEffectListeners.size(); i++) {
      pListener = m_pVideoEffectListeners.get(i);
      pListener.setCurrentVideoFrame(pCurrentFrame, iFrameId, iFrameNumber);
    }

  }

  /**
   *  Notifies all listeners that the processing of the video effect has been started. 
   *  This method should only used by classes extending the <code>VideoEffect</code>
   *  class.
   **/
  protected void fireProcessingStarted() {
    //1. make a simple value check
    if (m_pVideoEffectListeners == null) return;

    //3. fire the progress status events
    IVideoEffectListener pListener;

    for (int i = 0; i < m_pVideoEffectListeners.size(); i++) {
      pListener = m_pVideoEffectListeners.get(i);
      pListener.processingStarted();
    }
  
  }


  //////////////////////
  // SET METHODS      //
  //////////////////////

  /**
   *  Sets the <code>IVideo</code> object which should be processed by the <code>VideoEffect</code> object.
   *
   *  @param pVideo The <code>IVideo</code> object which represents the frames of the original video.
   *
   **/
  public void setVideo(IVideo pVideo)  {
    //1. make a simple check
    if (pVideo == null) return;

    //2. set the new video
    m_pVideo = pVideo;
    m_iPosition = 0;
  }



  /**
   *  Sets the current effect frame. This method should only used by classes extending the <code>VideoEffect</code>
   *  class. The reason for this is, that all other classes call the getCurrentFrame method of this class.
   **/


  //////////////////////
  // GET METHODS      //
  //////////////////////
  /**
   *  Returns the current frame that is processed. 
   *
   *  @return The current frame that is processed as {@code BufferedImage} object.
   **/
  public BufferedImage getCurrentFrame() {
    //1. if no video was set
    if ((m_pCurrentEffectImage == null)) {
      return new BufferedImage(720, 576, BufferedImage.TYPE_INT_RGB);
    }

    return m_pCurrentEffectImage;
  }

  /**
   *  Returns the current frame id that is processed. 
   *
   *  @return The current frame id that is processed. 
   **/
  public int getPosition() {
    return m_iPosition;
  }

  /**
   *  Returns the video object that should be processed. 
   *
   *  @return The video object that should be processed. 
   **/
  public IVideo getVideo() {
    return m_pVideo;
  }

  /**
   *  Returns true if the processing of the video effect got aborted.
   *
   *  @return true if the processing of the video effect got aborted.
   **/
  public boolean isVideoEffectAborted() {
    return m_bAbort;
  }


  /////////////////////////////////////////////
  // THREAD METHODS - RUNNABLE INTERFACE     //
  /////////////////////////////////////////////
  /**
   *  Aborts the processing of the video effect.
   *
   **/
  public void abort() {
    m_bAbort = true;
  }

  /**
   *  Starts the processing of the video effect.
   *
   **/
  public void run() {
    m_bAbort = false;
    process();
  }


  /////////////////////////////////////////////
  // VIDEO EFFECT NAVIGATION                 //
  /////////////////////////////////////////////
  /**
   *  Sets the video back to the start. 
   **/
  public void goToFirstFrame() {
    if (m_iPosition != 0) {
      m_iPosition = 0;
      m_pCurrentEffectImage = null;
    }
  }

  /**
   *  Sets the video to the last frame. 
   **/
  public void goToLastFrame() {
    //1. make a simple check
    if (m_pVideo == null) return;

    //2. go to the last frame
    if (m_iPosition != m_pVideo.getFrameNumber()-1) {
      m_iPosition = m_pVideo.getFrameNumber()-1;
      m_pCurrentEffectImage = null;
    }

  }

  /**
   *  Sets the video to the next available frame. 
   **/
  public void goToNextFrame() {
    //1. make a simple check
    if (m_pVideo == null) return;

    //2. go to the next frame
    if (m_iPosition < m_pVideo.getFrameNumber()-1) {
      m_iPosition++;
      m_pCurrentEffectImage = null;
    }

  }

  /**
   *  Sets the video to the previous frame. 
   **/
  public void goToPreviousFrame() {
    //1. make a simple check
    if (m_pVideo == null) return;

    //2. go to the next frame
    if (m_iPosition > 0) {
      m_iPosition--;
      m_pCurrentEffectImage = null;
    }

  }

  /**
   *  The actual processing method of the video effect that has to be implemented by each 
   *  video effect class.
   **/
  public abstract void process();



}