package util.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;
import java.util.Random;

/**
 *  A color randomizer. 
 **/
public class RandomColor  {

  /*** member variables ***/
  private Color m_pColor;
  private Random m_pRandom;
  private double m_dRandomMagnitude;
  
  private float m_fHue, m_fSaturation, m_fBrightness;

  
  
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /** 
   *  Creates a color randomizer with the color black, a default {@code Random} object and a randomizer magnitude of 150. 
   **/
  public RandomColor() {
    setColor(Color.BLACK);
    setRandom(new Random());
    setRandomMagnitude(150);
  }
  
  /** 
   *  Creates a color randomizer with the specified, a default {@code Random} object and a randomizer magnitude of 150. 
   * 
   *  @param pColor The color that should be used as base color for the randomize process.
   **/
  public RandomColor(Color pColor) {
    setColor(pColor);
    setRandom(new Random());
    setRandomMagnitude(150);
  }
  
  /** 
   *  Creates a color randomizer with the specified, the specified {@code Random} object and a randomizer magnitude of 150. 
   * 
   *  @param pColor The color that should be used as base color for the randomize process.
   *  @param pRandom The {@code Random} object that should be used to generate the random numbers.
   **/
  public RandomColor(Color pColor, Random pRandom) {
    setColor(pColor);
    setRandom(pRandom);
    setRandomMagnitude(150);
  }
  
  /** 
   *  Creates a color randomizer with the specified, the specified {@code Random} object and the specified randomizer magnitude. 
   * 
   *  @param pColor The color that should be used as base color for the randomize process.
   *  @param pRandom The {@code Random} object that should be used to generate the random numbers.
   *  @param dMagnitude The randomizer magnitude as double. 
   **/
  public RandomColor(Color pColor, Random pRandom, double dMagnitude) {
    setColor(pColor);
    setRandom(pRandom);
    setRandomMagnitude(dMagnitude);
  }
   
  
  
  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /** 
   *  Sets the color that should be the starting point for the randomizer process. If no color is specified, the color is set to black. 
   * 
   *  @param pColor The color that should be used as base color for the randomize process.
   **/

  public void setColor(Color pColor) {
    
    //1. make a simple check
    if (pColor == null) m_pColor = Color.BLACK;
    
    //2. set the color
    m_pColor = pColor;
    
    //3. set the HSB values -> always use setColor otherwise they wont be set!!!
    float[] pHSBValues = Color.RGBtoHSB(m_pColor.getRed(), m_pColor.getGreen(), m_pColor.getBlue(), null);
    m_fHue = pHSBValues[0];
    m_fSaturation = pHSBValues[1];
    m_fBrightness = pHSBValues[2];
  }
  
  /** 
   *  Sets the {@code Random} object that should be used to generate the random numbers for the color calculation.
   * 
   *  @param pRandom The {@code Random} object that should be used to generate the random numbers.
   **/
  public void setRandom(Random pRandom) {
    if (pRandom == null) pRandom = new Random();
    m_pRandom = pRandom;
  }
  
  
  /** 
   *  Sets the randomizer magnitude. 
   * 
   *  @param dMagnitude The randomizer magnitude as double. 
   **/
  public void setRandomMagnitude(double dMagnitude) {
    if (dMagnitude < 0) dMagnitude *= -1;
    m_dRandomMagnitude = dMagnitude;
  }
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Calculates a new random color by modifying the specified RGB channels of the base color of this object. 
   *  The randomize magnitude is used to multiply a random double offset between -0.5 and 0.5. The offsets are then applied to the base color.
   *  
   *  @param bRandomizeRed A flag indicating if the red color channel should be modified.
   *  @param bRandomizeGreen A flag indicating if the green color channel should be modified.
   *  @param bRandomizeBlue A flag indicating if the blue color channel should be modified.
   *
   *  @return A random color
   **/
  public Color getNextColorRGB(boolean bRandomizeRed, boolean bRandomizeGreen, boolean bRandomizeBlue) {
    //1. calculate the offsets
    double[] pOffsets = new double[3];
    for (int i = 0; i < 3; i++) {
      pOffsets[i] = (m_pRandom.nextDouble()-0.5)*m_dRandomMagnitude;
    }
    
    //2. calculate the new color based on the original color
    int iRed = m_pColor.getRed();
    if (bRandomizeRed) {
      iRed = (int)(iRed + pOffsets[0]);
      if (iRed > 255) iRed = 255;
      if (iRed < 0) iRed = 0;
    }

    int iGreen = m_pColor.getGreen();
    if (bRandomizeGreen) {
      iGreen = (int)(iGreen + pOffsets[1]);
      if (iGreen > 255) iGreen = 255;
      if (iGreen < 0) iGreen = 0;
    }

    int iBlue = m_pColor.getBlue();
    if (bRandomizeBlue) {
      iBlue = (int)(iBlue + pOffsets[2]);
      if (iBlue > 255) iBlue = 255;
      if (iBlue < 0) iBlue = 0;
    }

    return new Color(iRed, iGreen, iBlue);
    
  }
  
  
  /**
   *  Calculates a new random color by modifying the specified HSB parameters of the base color of this object. 
   *  The randomize magnitude divided by 500 is used to multiply a random double offset between -0.5 and 0.5. 
   *  The offsets are then applied to the HSB parameters of the base color.
   *  
   *  @param bRandomizeHue A flag indicating if the hue should be modified.
   *  @param bRandomizeSaturation A flag indicating if the saturation should be modified.
   *  @param bRandomizeBrightness A flag indicating if the brightness should be modified.
   *
   *  @return A random color
   **/
  public Color getNextColorHSB(boolean bRandomizeHue, boolean bRandomizeSaturation, boolean bRandomizeBrightness) {
    //1. calculate the offsets
    double[] pOffsets = new double[3];
    for (int i = 0; i < 3; i++) {
      pOffsets[i] = (m_pRandom.nextDouble()-0.5)*m_dRandomMagnitude/500;
    }
    
    //2. calculate the new color based on the original color
    float fHue = m_fHue;
    if (bRandomizeHue) {
      fHue = (float)(fHue + pOffsets[0]);
      while (fHue > 1) fHue -= 1.0f;
      while (fHue < 0) fHue += 1.0f;
    }

    float fSaturation = m_fSaturation;
    if (bRandomizeSaturation) {
      fSaturation = (float)(fSaturation + pOffsets[1]);
      if (fSaturation > 1) fSaturation = 1.0f;
      if (fSaturation < 0) fSaturation = 0.0f;
    }

    float fBrightness = m_fBrightness;
    if (bRandomizeBrightness) {
      fBrightness = (float)(fBrightness + pOffsets[2]);
      if (fBrightness > 1) fBrightness = 1.0f;
      if (fBrightness < 0) fBrightness = 0.0f;
    }

    return Color.getHSBColor(fHue, fSaturation, fBrightness);
    
  }
  
  
}
