package util.image;

/*
 *  Copyright (C) 2017 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.Graphics2D;
import java.awt.GradientPaint;
import java.awt.Color;
import java.awt.image.BufferedImage;




/**
 *   Provides methods to generate example images for effects that need an image on which they can operate.
 **/
public class ExampleImageGenerator {


    /*** member variables ***/
    private int m_iWidth, m_iHeight;
    private Color m_pLeftUpperColor, m_pRightLowerColor;
    
   /***
   *   Creates an image creator object with the default settings
   *
   *   <ul>
   *     <li>The image site is 400 x 400 pixels</li>
   *     <li>A linear color transition from white (left upper corner) to green (right lower corner) is drawn.</li>
   *   </ul>
   *
   **/
  public ExampleImageGenerator() {

    m_iWidth = 400;
    m_iHeight = 400;
    m_pLeftUpperColor = Color.WHITE;
    m_pRightLowerColor = Color.GREEN;
    
  }
  
  ////////////////////////////////////
  // SET METHODS                    //
  ////////////////////////////////////
  /**
   *  Sets the width of the image that should be generated. 
   *
   *  @param iWidth The width of the image that should be generated in pixels. Values lower that 1 are mapped to 1.
   **/
  public void setWidth(int iWidth) {
    if (iWidth < 1) iWidth = 1;
    m_iWidth = iWidth;
  }
  
  /**
   *  Sets the height of the image that should be generated. 
   *
   *  @param iHeight The height of the image that should be generated in pixels. Values lower that 1 are mapped to 1.
   **/
  public void setHeight(int iHeight) {
    if (iHeight < 1) iHeight = 1;
    m_iHeight = iHeight;
  }
  
  /**
   *  Sets the color in the left upper corner that should be used for the color transition. 
   *
   *  @param pColor The color in the left upper corner that should be used for the color transition. If the color is null, nothing is changed.
   **/
  public void setLeftUpperColor(Color pColor) {
    if (pColor == null) return;
    m_pLeftUpperColor = pColor;
  }
  
  /**
   *  Sets the color in the left upper corner that should be used for the color transition. 
   *
   *  @param pColor The color in the left upper corner that should be used for the color transition. If the color is null, nothing is changed.
   **/
  public void setRightLowerColor(Color pColor) {
    if (pColor == null) return;
    m_pRightLowerColor = pColor;
  }
  
  
  
  ///////////////////////////////////////
  // GET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Generates the example image according to the given settings. 
   *
   *  @return The example image according to the given settings. 
   **/
  public BufferedImage getImage() {
    BufferedImage pImage = new BufferedImage(m_iWidth, m_iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pImage.createGraphics();
    GradientPaint pPaint = new GradientPaint(0.0f, 0.0f, m_pLeftUpperColor, (float)m_iWidth, (float)m_iHeight, m_pRightLowerColor);
    pGraphics.setPaint(pPaint);
    pGraphics.fillRect(0, 0, m_iWidth, m_iHeight);
    
    return pImage;
  }
  
}
  
  
  
