package util.image;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;




/***
 *   This class provides static methods for different kinds of image difference operations.
 *   All kinds of difference operations based on two images are implemented here.
 *   TO DO: Get rid of static methods and use object oriented style?
 *
 *   <br><br>Written by J&uuml;rgen Platzer.<br>
 *   Tested by J&uuml;rgen Platzer.<br>
 *   Documented by J&uuml;rgen Platzer.
 **/
public class ImageDifferenceOperations {


  ////////////////////////////////
  // COLOR DIFFERENCES          //
  ////////////////////////////////
  /**
   *  Calculates the absolute difference between the images. As the absolute difference of
   *  the color is calculated the order of the given source images does not matter. Transparency is not considered.
   *
   *  @param pImage1 <code>BufferedImage</code> object representing the first image for the difference operation.
   *  @param pImage2 <code>BufferedImage</code> object representing the second image for the difference operation.
   *
   *  @return The absolute color difference image as <code>BufferedImage</code> object.
   *
   */
  public static BufferedImage calculateAbsDiffImage(BufferedImage pImage1, BufferedImage pImage2) {

    //1. make some checks
    if (pImage1 == null) return null;
    if (pImage2 == null) return null;

    //2. set variables
    int iWidth1 = pImage1.getWidth();
    int iWidth2 = pImage2.getWidth();
    int iWidth = Math.max(iWidth1, iWidth2);

    int iHeight1 = pImage1.getHeight();
    int iHeight2 = pImage2.getHeight();
    int iHeight = Math.max(iHeight1, iHeight2);

    BufferedImage pDiffImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    boolean bHaveSameSize = ((iWidth1 == iWidth2) && (iHeight1 == iHeight2));

    //3. calculate the absolute color difference image
    int iRGB1, iRed1, iGreen1, iBlue1;
    int iRGB2, iRed2, iGreen2, iBlue2;
    int iRGBDiff, iRedDiff, iGreenDiff, iBlueDiff;


    //3.1. if both images have the same size
    if (bHaveSameSize) {
      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {

          //I. fetch the RGB values of the first image
          iRGB1 = pImage1.getRGB(x,y);
          iRed1 = (iRGB1 >> 16) & 0xff;
          iGreen1 = (iRGB1 >> 8) & 0xff;
          iBlue1 = (iRGB1 >> 0) & 0xff;

          //II. fetch the RGB values of the second image
          iRGB2 = pImage2.getRGB(x,y);
          iRed2 = (iRGB2 >> 16) & 0xff;
          iGreen2 = (iRGB2 >> 8) & 0xff;
          iBlue2 = (iRGB2 >> 0) & 0xff;


          //III. calculate the absolute difference
          iRedDiff = Math.abs(iRed1 - iRed2);
          iGreenDiff = Math.abs(iGreen1 - iGreen2);
          iBlueDiff = Math.abs(iBlue1 - iBlue2);

          iRGBDiff = (255 << 24) | (iRedDiff << 16) | (iGreenDiff << 8) | iBlueDiff;

          //IV. set the RGB value at the diff image
          pDiffImage.setRGB(x,y, iRGBDiff);

        }
      }
    }

    //3.2. if both image do not have the same size
    else {
      //I. calculate the difference image in the overlap region
      int iOverlapWidth = Math.min(iWidth1, iWidth2);
      int iOverlapHeight = Math.min(iHeight1, iHeight2);

      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {

          //I. fetch the RGB values of the first image
          iRGB1 = pImage1.getRGB(x,y);
          iRed1 = (iRGB1 >> 16) & 0xff;
          iGreen1 = (iRGB1 >> 8) & 0xff;
          iBlue1 = (iRGB1 >> 0) & 0xff;

          //II. fetch the RGB values of the second image
          iRGB2 = pImage2.getRGB(x,y);
          iRed2 = (iRGB2 >> 16) & 0xff;
          iGreen2 = (iRGB2 >> 8) & 0xff;
          iBlue2 = (iRGB2 >> 0) & 0xff;


          //III. calculate the absolute difference
          iRedDiff = Math.abs(iRed1 - iRed2);
          iGreenDiff = Math.abs(iGreen1 - iGreen2);
          iBlueDiff = Math.abs(iBlue1 - iBlue2);

          iRGBDiff = (255 << 24) | (iRedDiff << 16) | (iGreenDiff << 8) | iBlueDiff;

          //IV. set the RGB value at the diff image
          pDiffImage.setRGB(x,y, iRGBDiff);

        }
      }

      //II. render image1 in regions where it is bigger as the min
      if (iWidth1 > iOverlapWidth) {
        for (int x = iOverlapWidth; x < iWidth1; x++) {
          for (int y = 0; y < iOverlapHeight; y++) {
            pDiffImage.setRGB(x,y, pImage1.getRGB(x,y));
          }
        }
      }

      if (iHeight1 > iOverlapHeight) {
        for (int x = 0; x < iOverlapWidth; x++) {
          for (int y = iOverlapHeight; y < iHeight1; y++) {
            pDiffImage.setRGB(x,y, pImage1.getRGB(x,y));
          }
        }
      }

      if ((iWidth1 > iOverlapWidth) && (iHeight1 > iOverlapHeight)) {
        for (int x = iOverlapWidth; x < iWidth1; x++) {
          for (int y = iOverlapHeight; y < iHeight1; y++) {
            pDiffImage.setRGB(x,y, pImage1.getRGB(x,y));
          }
        }
      }


      //III. render image2 in regions where it is bigger as the min
      if (iWidth2 > iOverlapWidth) {
        for (int x = iOverlapWidth; x < iWidth2; x++) {
          for (int y = 0; y < iOverlapHeight; y++) {
            pDiffImage.setRGB(x,y, pImage2.getRGB(x,y));
          }
        }
      }

      if (iHeight2 > iOverlapHeight) {
        for (int x = 0; x < iOverlapWidth; x++) {
          for (int y = iOverlapHeight; y < iHeight2; y++) {
            pDiffImage.setRGB(x,y, pImage2.getRGB(x,y));
          }
        }
      }

      if ((iWidth2 > iOverlapWidth) && (iHeight2 > iOverlapHeight)) {
        for (int x = iOverlapWidth; x < iWidth2; x++) {
          for (int y = iOverlapHeight; y < iHeight2; y++) {
            pDiffImage.setRGB(x,y, pImage2.getRGB(x,y));
          }
        }
      }




    }


    //4. return the color difference image
    return pDiffImage;


  }

  ////////////////////////////////
  // LUMINANCE DIFFERENCES      //
  ////////////////////////////////

  /**
   *  Calculates the absolute difference between the luminance images created from the given source images. This method
   *  calculates the luminance for each pixel according to L = 0.3*R + 0.59*G + 0.11*B. As the absolute difference of
   *  the luminance is calculated the order of the given source images does not matter.
   *
   *  @param pImage1 <code>BufferedImage</code> object representing the first image for the difference operation.
   *  @param pImage2 <code>BufferedImage</code> object representing the second image for the difference operation.
   *
   *  @return The absolute luminance difference image as <code>BufferedImage</code> object.
   *
   */
  public static BufferedImage calculateAbsLumDiffImage(BufferedImage pImage1, BufferedImage pImage2) {
    return calculateAbsLumDiffImage(pImage1, pImage2, 1);
  }



  /**
   *  Calculates the absolute difference between the luminance images created from the given source images and allows
   *  the amplification or alleviation of the difference values by a multiplicator. This method
   *  calculates the luminance for each pixel according to L = 0.3*R + 0.59*G + 0.11*B. As the absolute difference of
   *  the luminance is calculated the order of the given source images does not matter. The multiplicator scales the difference
   *  values.Consequently only multiplicator greater or equal 0 are allowed. If a multiplicator smaller than 0 is provided, it is set to 0, which
   *  results in a black difference image independent of the image contents.
   *
   *  @param pImage1        <code>BufferedImage</code> object representing the first image for the difference operation.
   *  @param pImage2        <code>BufferedImage</code> object representing the second image for the difference operation.
   *  @param dMultiplicator A multiplicator that scales the difference values, to amplify or attenuate the detected differences.
   *
   *  @return The absolute luminance difference image as <code>BufferedImage</code> object scaled by a multiplicator.
   *
   */
  public static BufferedImage calculateAbsLumDiffImage(BufferedImage pImage1, BufferedImage pImage2, double dMultiplicator) {

    //1. make some checks
    if (pImage1 == null) return null;
    if (pImage2 == null) return null;
    if (dMultiplicator < 0) dMultiplicator = 0;

    //2. set variables
    int iWidth = Math.max(pImage1.getWidth(), pImage2.getWidth());
    int iHeight = Math.max(pImage1.getHeight(), pImage2.getHeight());
    BufferedImage pDiffImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    if (dMultiplicator == 0) return pDiffImage;



    //3. convert both input images to luminance arrays
    double[][] pLumImage1 = calculateLuminanceImage(pImage1);
    double[][] pLumImage2 = calculateLuminanceImage(pImage2);

    //4. calculate the absolute difference in the luminance arrays

    //4. a) if the luminance arrays have the same size
    if ((pLumImage1.length == pLumImage2.length) && (pLumImage1[0].length == pLumImage2[0].length)) {

      double dAbsLumDiff = 0;
      int iAbsLumDiff = 0;
      Color pColor = null;

      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {

          dAbsLumDiff = Math.abs(pLumImage1[x][y] - pLumImage2[x][y])*dMultiplicator;

          if (dAbsLumDiff > 255) dAbsLumDiff = 255;
          iAbsLumDiff = (int)dAbsLumDiff;

          pColor = new Color(iAbsLumDiff, iAbsLumDiff, iAbsLumDiff);

          pDiffImage.setRGB(x, y, pColor.getRGB());

        }
      }

    }

    //4. b) if the luminance arrays do not have the same size
    else {

      //I. paint the image white
      Graphics2D pGraphics = pDiffImage.createGraphics();
      pGraphics.setColor(Color.WHITE);
      pGraphics.fillRect(0, 0, iWidth, iHeight);

      //II. calculate the remaining image space where both source images overlap
      int iMinWidth = Math.max(pImage1.getWidth(), pImage2.getWidth());
      int iMinHeight = Math.max(pImage1.getHeight(), pImage2.getHeight());


      double dAbsLumDiff = 0;
      int iAbsLumDiff = 0;
      Color pColor = null;

      for (int x = 0; x < iMinWidth; x++) {
        for (int y = 0; y < iMinHeight; y++) {

          dAbsLumDiff = Math.abs(pLumImage1[x][y] - pLumImage2[x][y]);

          if (dAbsLumDiff > 255) dAbsLumDiff = 255;
          iAbsLumDiff = (int)dAbsLumDiff;

          pColor = new Color(iAbsLumDiff, iAbsLumDiff, iAbsLumDiff);

          pDiffImage.setRGB(x, y, pColor.getRGB());

        }
      }



    }


    return pDiffImage;



  }




  /**
   *  Calculates the difference between the luminance images created from the given source images. This method
   *  calculates the luminance for each pixel according to L = 0.3*R + 0.59*G + 0.11*B. 
   *
   *  @param pImage1        <code>BufferedImage</code> object representing the first image for the difference operation.
   *  @param pImage2        <code>BufferedImage</code> object representing the second image for the difference operation.

   *  @return The luminance difference image as <code>BufferedImage</code> object scaled by a multiplicator.
   *
   */
  public static BufferedImage calculateLumDiffImage(BufferedImage pImage1, BufferedImage pImage2) {

    //1. make some checks
    if (pImage1 == null) return null;
    if (pImage2 == null) return null;

    //2. set variables
    int iWidth = Math.max(pImage1.getWidth(), pImage2.getWidth());
    int iHeight = Math.max(pImage1.getHeight(), pImage2.getHeight());
    BufferedImage pDiffImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);



    //3. convert both input images to luminance arrays and create an intermediate luminance difference array
    double[][] pLumImage1 = calculateLuminanceImage(pImage1);
    double[][] pLumImage2 = calculateLuminanceImage(pImage2);
    double[][] pLumDiff = new double[iWidth][iHeight];

    double dLumDiff = 0;
    double dMaxLumDiff = 0;
    double dMinLumDiff = 0;


    //4. calculate the difference in the luminance arrays

    //4. a) if the luminance arrays have the same size
    if ((pLumImage1.length == pLumImage2.length) && (pLumImage1[0].length == pLumImage2[0].length)) {


      for (int x = 0; x < iWidth; x++) {
        for (int y = 0; y < iHeight; y++) {

          dLumDiff = pLumImage1[x][y] - pLumImage2[x][y];

          if (dLumDiff > dMaxLumDiff) dMaxLumDiff = dLumDiff;
          if (dLumDiff < dMinLumDiff) dMinLumDiff = dLumDiff;

          pLumDiff[x][y] = dLumDiff;

        }
      }

    }

    //4. b) if the luminance arrays do not have the same size
    else {


      //I. calculate the differences for the image space where both source images overlap
      int iMinWidth = Math.max(pImage1.getWidth(), pImage2.getWidth());
      int iMinHeight = Math.max(pImage1.getHeight(), pImage2.getHeight());


      for (int x = 0; x < iMinWidth; x++) {
        for (int y = 0; y < iMinHeight; y++) {

          dLumDiff = pLumImage1[x][y] - pLumImage2[x][y];

          if (dLumDiff > dMaxLumDiff) dMaxLumDiff = dLumDiff;
          if (dLumDiff < dMinLumDiff) dMinLumDiff = dLumDiff;

          pLumDiff[x][y] = dLumDiff;

        }
      }


      //II. set all entries where only one image is present to dMaxLumDiff
      //II.a) the right border
      for (int x = iMinWidth; x < iWidth; x++) {
        for (int y = 0; y < iMinHeight; y++) {
          pLumDiff[x][y] = dMaxLumDiff;
        }
      }

      //II.a) the border at the bottom
      for (int x = 0; x < iWidth; x++) {
        for (int y = iMinHeight; y < iHeight; y++) {
          pLumDiff[x][y] = dMaxLumDiff;
        }
      }




    }



    //5. map the difference values to colors
    double pFactor = 255/(dMaxLumDiff - dMinLumDiff);
    int iMappedValue = 0;
    Color pColor = null;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {
        iMappedValue = (int)((pLumDiff[x][y]-dMinLumDiff)*pFactor);
        if (iMappedValue > 255) iMappedValue = 255;

        pColor = new Color(iMappedValue, iMappedValue, iMappedValue);
        pDiffImage.setRGB(x, y, pColor.getRGB());

      }
    }


    return pDiffImage;




  }






  /////////////////////////////////
  // PRIVATE HELPER METHODS      //
  /////////////////////////////////
  /**
   *   Computes for each pixel of the given a luminance value and returns these calculations as two dimensional
   *   double array. The luminance is calculated as 0.2125*RED + 0.7154*GREEN + 0.0721*BLUE. TO DO: Make this method available so that other sources can use it!!!
   *
   *   @param pImage The <code>BufferedImage</code> object for which the luminance values should be calculated.
   *
   *   @return A two dimensional doulbe array holding the luminance values for the pixels of the given image.
   **/
  private static double[][] calculateLuminanceImage(BufferedImage pImage) {
    //1. initial check
    if (pImage == null) return null;

    //2. set variables
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();

    double[][] pLuminanceImage = new double[iWidth][iHeight];


    Color pColor = null;
    int iRed = 0;
    int iGreen = 0;
    int iBlue = 0;
    double dLuminance = 0;


    //create the luminance image
    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        pColor = new Color(pImage.getRGB(x,y));
        iRed = pColor.getRed();
        iGreen = pColor.getGreen();
        iBlue = pColor.getBlue();

        dLuminance = 0.2125*iRed + 0.7154*iGreen + 0.0721*iBlue;

        pLuminanceImage[x][y] = dLuminance;

      }
    }

    return pLuminanceImage;

  }


}
